@@ -19,6 +19,15 @@
 #ifndef CDROM_INTERFACE_H_INCLUDED
 #define CDROM_INTERFACE_H_INCLUDED
 
+typedef enum
+{
+ CDIF_FORMAT_AUDIO,
+ CDIF_FORMAT_MODE1,
+ CDIF_FORMAT_MODE2,
+ CDIF_FORMAT_PSX,
+ CDIF_FORMAT_CDI
+} CDIF_Track_Format;
+
 bool CDIF_Open(const char *device_name);
 bool CDIF_Close(void);
 
@@ -27,12 +36,16 @@ void CDIF_Deinit(void);
 
 int32 CDIF_GetFirstTrack();
 int32 CDIF_GetLastTrack();
-uint32 CDIF_GetTrackStartPositionMSF(int32 track);
+
+bool CDIF_GetTrackStartPositionMSF(int32 track, int &min, int &sec, int &frame);
+bool CDIF_GetTrackFormat(int32 track, CDIF_Track_Format &format);
+
 uint32 CDIF_GetTrackStartPositionLBA(int32 track);
-bool CDIF_ReadSector(uint8* pBuf, uint32 sector, uint32 nSectors);
-bool CDIF_ReadSubChannelQ(int8* pBuf, bool bCallback);
+int CDIF_FindTrackByLBA(uint32 LBA);
+
+bool CDIF_ReadSector(uint8* pBuf, uint8 *SubQBuf, uint8 *SubRWBuf, uint32 sector, uint32 nSectors);
 uint32 CDIF_GetSectorCountLBA(void);
-bool CDIF_ReadAudioSector(int16 *buffer, uint32 read_sec);
+bool CDIF_ReadAudioSector(int16 *buffer, uint8 *SubQBuf, uint8 *SubRWBuf, uint32 read_sec);
 
 #endif /* CDROM_INTERFACE_H_INCLUDED */
 
