@@ -45,36 +45,42 @@
 #include "cheat.h"
 #include "fps.h"
 #include "debugger.h"
+#include "memdebugger.h"
+#include "help.h"
+#include "video-state.h"
+#include "remote.h"
 
 bool pending_save_state, pending_snapshot, pending_save_movie;
 static volatile Uint32 MainThreadID = 0;
 static bool ffnosound;
 
-static const char *CSD_xres = "Full-screen horizontal resolution.";
-static const char *CSD_yres = "Full-screen vertical resolution.";
-static const char *CSD_xscale = "The scaling factor for the X axis.";
-static const char *CSD_yscale = "The scaling factor for the Y axis.";
-static const char *CSD_xscalefs = "The scaling factor for the X axis in fullscreen mode.";
-static const char *CSD_yscalefs = "The scaling factor for the Y axis in fullscreen mode.";
-static const char *CSD_scanlines = "Enable scanlines with specified transparency.";
-static const char *CSD_stretch = "Stretch to fill screen.";
-static const char *CSD_videoip = "Enable bilinear interpolation.";
-static const char *CSD_special = "Enable specified special video scaler.";
+static const char *CSD_xres = gettext_noop("Full-screen horizontal resolution.");
+static const char *CSD_yres = gettext_noop("Full-screen vertical resolution.");
+static const char *CSD_xscale = gettext_noop("The scaling factor for the X axis.");
+static const char *CSD_yscale = gettext_noop("The scaling factor for the Y axis.");
+static const char *CSD_xscalefs = gettext_noop("The scaling factor for the X axis in fullscreen mode.");
+static const char *CSD_yscalefs = gettext_noop("The scaling factor for the Y axis in fullscreen mode.");
+static const char *CSD_scanlines = gettext_noop("Enable scanlines with specified transparency.");
+static const char *CSD_stretch = gettext_noop("Stretch to fill screen.");
+static const char *CSD_videoip = gettext_noop("Enable bilinear interpolation.");
+static const char *CSD_special = gettext_noop("Enable specified special video scaler.");
 
 #ifdef MDFN_WANT_OPENGL_SHADERS
-static const char *CSD_pixshader = "Enable specified OpenGL pixel shader.";
+static const char *CSD_pixshader = gettext_noop("Enable specified OpenGL pixel shader.");
 #endif
 
+static std::vector <MDFNSetting> NeoDriverSettings;
 static MDFNSetting DriverSettings[] =
 {
   #ifdef NETWORK
-  { "nethost", "Network play server hostname.", MDFNST_STRING, "mednafen.com" },
-  { "netport", "Port to connect to on the server.", MDFNST_UINT, "4046", "1", "65535" },
-  { "netpassword", "Password to connect to the netplay server.", MDFNST_STRING, "" },
-  { "netlocalplayers", "Number of local players for network play.", MDFNST_UINT, "1", "1", "8" },
-  { "netnick", "Nickname to use for network play chat.", MDFNST_STRING, "" },
-  { "netgamekey", "Key to hash with the MD5 hash of the game.", MDFNST_STRING, "" },
-  { "netmerge", "Merge input to this player # on the server.", MDFNST_UINT, "0" },
+  { "nethost", gettext_noop("Network play server hostname."), MDFNST_STRING, "mednafen.com" },
+  { "netport", gettext_noop("Port to connect to on the server."), MDFNST_UINT, "4046", "1", "65535" },
+  { "netpassword", gettext_noop("Password to connect to the netplay server."), MDFNST_STRING, "" },
+  { "netlocalplayers", gettext_noop("Number of local players for network play."), MDFNST_UINT, "1", "1", "8" },
+  { "netnick", gettext_noop("Nickname to use for network play chat."), MDFNST_STRING, "" },
+  { "netgamekey", gettext_noop("Key to hash with the MD5 hash of the game."), MDFNST_STRING, "" },
+  { "netmerge", gettext_noop("Merge input to this player # on the server."), MDFNST_UINT, "0" },
+  { "netsmallfont", gettext_noop("Use small(tiny!) font for netplay chat console."), MDFNST_BOOL, "0" },
   #endif
 
   { "nes.xres", CSD_xres, MDFNST_UINT, "640" },
@@ -175,75 +181,88 @@ static MDFNSetting DriverSettings[] =
   { "pcfx.pixshader", CSD_pixshader, MDFNST_STRING, "none", NULL, NULL },
   #endif
 
-  { "fs", "Enable fullscreen mode.", MDFNST_BOOL, "0", },
-  { "vdriver", "Select video driver.  0 = OpenGL, 1 = SDL surface blits", MDFNST_UINT, "0", "0", "1" },
-  { "doublebuf", "Enable video double buffering.", MDFNST_BOOL, "1" },
+  { "player.xres", CSD_xres, MDFNST_UINT, "800" },
+  { "player.yres", CSD_yres, MDFNST_UINT, "600" },
+  { "player.xscale", CSD_xscale, MDFNST_FLOAT, "2" },
+  { "player.yscale", CSD_yscale, MDFNST_FLOAT, "2" },
+  { "player.xscalefs", CSD_xscalefs, MDFNST_FLOAT, "2" },
+  { "player.yscalefs", CSD_yscalefs, MDFNST_FLOAT, "2" },
+  { "player.scanlines", CSD_scanlines, MDFNST_UINT, "0" },
+  { "player.stretch", CSD_stretch, MDFNST_BOOL, "0" },
+  { "player.videoip", CSD_videoip, MDFNST_BOOL, "0" },
+  { "player.special", CSD_special, MDFNST_STRING, "none", NULL, NULL, MDFND_ValidateSpecialScalerSetting },
+  #ifdef MDFN_WANT_OPENGL_SHADERS
+  { "player.pixshader", CSD_pixshader, MDFNST_STRING, "none", NULL, NULL },
+  #endif
 
-  { "ffspeed", "Fast-forwarding speed multiplier.", MDFNST_UINT, "4", "1", "15" },
-  { "fftoggle", "Treat the fast-forward button as a toggle.", MDFNST_BOOL, "0" },
-  { "ffnosound", "Silence sound output when fast-forwarding.", MDFNST_BOOL, "0" },
-  { "autofirefreq", "Auto-fire frequency.", MDFNST_UINT, "3", "0", "1000" },
-  { "analogthreshold", "Threshold for detecting a \"button\" press on analog axis, in percent.", MDFNST_FLOAT, "75", "0", "100" },
-  { "ckdelay", "The length of time, in milliseconds, that a button/key corresponding to a \"dangerous\" command like power, reset, exit, etc. must be pressed before the command is executed.", MDFNST_UINT, "0", "0", "99999" },
-  { "nes.fcexp", "Select Famicom expansion-port device.", MDFNST_STRING, "none", NULL, NULL, MDFND_ValidateFamicomExpansionPort },
-  { "nes.input1", "Select input device for input port 1.", MDFNST_STRING, "gamepad", NULL, NULL, MDFND_ValidateNESInputPort },
-  { "nes.input2", "Select input device for input port 2.", MDFNST_STRING, "gamepad", NULL, NULL , MDFND_ValidateNESInputPort },
-  { "nothrottle", "Disable speed throttling when sound is disabled.", MDFNST_BOOL, "0"},
-  { "autosave", "Automatically save and load save states when a game is closed or loaded, respectively.", MDFNST_BOOL, "0"},
-  { "sounddriver", "Select sound driver.", MDFNST_STRING, "default", NULL, NULL },
-  { "sounddevice", "Select sound output device.", MDFNST_STRING, "default", NULL, NULL },
-  { "soundvol", "Sound volume level, in percent.", MDFNST_UINT, "100", "0", "150" },
-  { "sound", "Enable sound emulation.", MDFNST_BOOL, "1" },
-  { "soundbufsize", "Specifies the desired size of the sound buffer, in milliseconds.", MDFNST_UINT, 
+  { "wswan.xres", CSD_xres, MDFNST_UINT, "640" },
+  { "wswan.yres", CSD_yres, MDFNST_UINT, "480" },
+  { "wswan.xscale", CSD_xscale, MDFNST_FLOAT, "3" },
+  { "wswan.yscale", CSD_yscale, MDFNST_FLOAT, "3" },
+  { "wswan.xscalefs", CSD_xscalefs, MDFNST_FLOAT, "2" },
+  { "wswan.yscalefs", CSD_yscalefs, MDFNST_FLOAT, "2" },
+  { "wswan.scanlines", CSD_scanlines, MDFNST_UINT, "0" },
+  { "wswan.stretch", CSD_stretch, MDFNST_BOOL, "0" },
+  { "wswan.videoip", CSD_videoip, MDFNST_BOOL, "0" },
+  { "wswan.special", CSD_special, MDFNST_STRING, "none", NULL, NULL, MDFND_ValidateSpecialScalerSetting },
+  #ifdef MDFN_WANT_OPENGL_SHADERS
+  { "wswan.pixshader", CSD_pixshader, MDFNST_STRING, "none", NULL, NULL },
+  #endif
+
+  { "fs", gettext_noop("Enable fullscreen mode."), MDFNST_BOOL, "0", },
+  { "vdriver", gettext_noop("Select video driver.  0 = OpenGL, 1 = SDL surface blits"), MDFNST_UINT, "0", "0", "1" },
+  { "glvsync", gettext_noop("Attempt to synchronize OpenGL page flips to vertical retrace period."), MDFNST_BOOL, "1" },
+
+  { "ffspeed", gettext_noop("Fast-forwarding speed multiplier."), MDFNST_UINT, "4", "1", "15" },
+  { "fftoggle", gettext_noop("Treat the fast-forward button as a toggle."), MDFNST_BOOL, "0" },
+  { "ffnosound", gettext_noop("Silence sound output when fast-forwarding."), MDFNST_BOOL, "0" },
+  { "autofirefreq", gettext_noop("Auto-fire frequency."), MDFNST_UINT, "3", "0", "1000" },
+  { "analogthreshold", gettext_noop("Threshold for detecting a \"button\" press on analog axis, in percent."), MDFNST_FLOAT, "75", "0", "100" },
+  { "ckdelay", gettext_noop("The length of time, in milliseconds, that a button/key corresponding to a \"dangerous\" command like power, reset, exit, etc. must be pressed before the command is executed."), MDFNST_UINT, "0", "0", "99999" },
+  { "nothrottle", gettext_noop("Disable speed throttling when sound is disabled."), MDFNST_BOOL, "0"},
+  { "autosave", gettext_noop("Automatically save and load save states when a game is closed or loaded, respectively."), MDFNST_BOOL, "0"},
+  { "sounddriver", gettext_noop("Select sound driver."), MDFNST_STRING, "default", NULL, NULL },
+  { "sounddevice", gettext_noop("Select sound output device."), MDFNST_STRING, "default", NULL, NULL },
+  { "soundvol", gettext_noop("Sound volume level, in percent."), MDFNST_UINT, "100", "0", "150" },
+  { "sound", gettext_noop("Enable sound emulation."), MDFNST_BOOL, "1" },
+  { "soundbufsize", gettext_noop("Specifies the desired size of the sound buffer, in milliseconds."), MDFNST_UINT, 
    #ifdef WIN32
    "52"
    #else
    "32"
    #endif
    ,"1", "1000" },
-  { "soundrate", "Specifies the sound playback rate, in frames per second(\"Hz\").", MDFNST_UINT, "48000", "8192", "48000"},
-  { NULL }
+  { "soundrate", gettext_noop("Specifies the sound playback rate, in frames per second(\"Hz\")."), MDFNST_UINT, "48000", "8192", "48000"},
+  { "helpenabled", gettext_noop("Enable the help screen."), MDFNST_BOOL, "1" },
 };
 
 
 static SDL_Thread *GameThread;
 static uint32 *VTBuffer[2] = { NULL, NULL };
 static MDFN_Rect *VTLineWidths[2] = { NULL, NULL };
-static uint32 *VTOSDBuffer[2] = { NULL, NULL };
 
 static volatile int VTBackBuffer = 0;
-static SDL_mutex *VTMutex = NULL, *EVMutex = NULL, *DebuggerMutex = NULL;
+static SDL_mutex *VTMutex = NULL, *EVMutex = NULL, *GameMutex = NULL;
 
 static volatile uint32 *VTReady;
 static volatile MDFN_Rect *VTLWReady;
-static volatile uint32 *VTOSDReady;
 static volatile MDFN_Rect VTDisplayRect;
 
-void LockDebuggerMutex(bool lock)
+void LockGameMutex(bool lock)
 {
  if(lock)
-  SDL_mutexP(DebuggerMutex);
+  SDL_mutexP(GameMutex);
  else
-  SDL_mutexV(DebuggerMutex);
+  SDL_mutexV(GameMutex);
 }
 
 //static
  char *soundrecfn=0;	/* File name of sound recording. */
 
-static int inited=0;
 static char *DrBaseDirectory;
 
-static void DriverKill(void);
-int gametype;
-
 MDFNGI *CurGame=NULL;
 
-static void ParseGI(MDFNGI *gi)
-{
- InitOtherInput(gi);
- gametype=gi->nes.type;
-}
-
 void MDFND_PrintError(const char *s)
 {
  if(SDL_ThreadID() != MainThreadID)
@@ -276,34 +295,6 @@ void MDFND_Message(const char *s)
   fputs(s,stdout);
 }
 
-static CFGSTRUCT fceuconfig[]={
-	ADDCFGSTRUCT(InputConfig),
-	ENDCFGSTRUCT
-};
-
-static void SaveInputMapConfig(void)
-{	
-        std::string tdir;
-
-        tdir = std::string(DrBaseDirectory);
-        tdir += PSS;
-        tdir += "inputmap.cfg";
-
-        SaveMDFNConfig(tdir.c_str(), fceuconfig);
-}
-
-static void LoadInputMapConfig(void)
-{
-	std::string tdir;
-
-        tdir = std::string(DrBaseDirectory);
-        tdir += PSS;
-        tdir += "inputmap.cfg";
-
-        LoadMDFNConfig(tdir.c_str(),fceuconfig);
-	InputUserActiveFix();
-}
-
 static void CreateDirs(void)
 {
  char *subs[6]={"mcs","mcm","snaps","gameinfo","sav","cheats"};
@@ -325,53 +316,54 @@ typedef struct
 {
  int number;
  const char *message;
+ const bool SafeTryExit;
 } SignalInfo;
 
 static SignalInfo SignalDefs[] =
 {
  #ifdef SIGINT
- { SIGINT, "How DARE you interrupt me!\n" },
+ { SIGINT, gettext_noop("How DARE you interrupt me!\n"), TRUE },
  #endif
 
  #ifdef SIGTERM
- { SIGTERM, "MUST TERMINATE ALL HUMANS\n" },
+ { SIGTERM, gettext_noop("MUST TERMINATE ALL HUMANS\n"), TRUE },
  #endif
 
  #ifdef SIGHUP
- { SIGHUP, "Reach out and hang-up on someone.\n" },
+ { SIGHUP, gettext_noop("Reach out and hang-up on someone.\n"), FALSE },
  #endif
 
  #ifdef SIGSEGV
- { SIGSEGV, "Iyeeeeeeeee!!!  A segmentation fault has occurred.  Have a fluffy day.\n" },
+ { SIGSEGV, gettext_noop("Iyeeeeeeeee!!!  A segmentation fault has occurred.  Have a fluffy day.\n"), FALSE },
  #endif
 
  #ifdef SIGPIPE
- { SIGPIPE, "The pipe has broken!  Better watch out for floods...\n" },
+ { SIGPIPE, gettext_noop("The pipe has broken!  Better watch out for floods...\n"), FALSE },
  #endif
 
  #ifdef SIGBUS
  /* SIGBUS can == SIGSEGV on some platforms, so put it after SIGSEGV */
- { SIGBUS, "I told you to be nice to the driver.\n" },
+ { SIGBUS, gettext_noop("I told you to be nice to the driver.\n"), FALSE },
  #endif
 
  #ifdef SIGFPE
- { SIGFPE, "Those darn floating points.  Ne'er know when they'll bite!\n" },
+ { SIGFPE, gettext_noop("Those darn floating points.  Ne'er know when they'll bite!\n"), FALSE },
  #endif
 
  #ifdef SIGALRM
- { SIGALRM, "Don't throw your clock at the meowing cats!\n" },
+ { SIGALRM, gettext_noop("Don't throw your clock at the meowing cats!\n"), TRUE },
  #endif
 
  #ifdef SIGABRT
- { SIGABRT, "Abort, Retry, Ignore, Fail?\n" },
+ { SIGABRT, gettext_noop("Abort, Retry, Ignore, Fail?\n"), FALSE },
  #endif
  
  #ifdef SIGUSR1
- { SIGUSR1, "Killing your processes is not nice.\n" },
+ { SIGUSR1, gettext_noop("Killing your processes is not nice.\n"), TRUE },
  #endif
 
  #ifdef SIGUSR2
- { SIGUSR2, "Killing your processes is not nice.\n" },
+ { SIGUSR2, gettext_noop("Killing your processes is not nice.\n"), TRUE },
  #endif
 };
 
@@ -383,22 +375,27 @@ static void SetSignals(void (*t)(int))
 
 static void CloseStuff(int signum)
 {
-	//if(GameThread && SDL_ThreadID() == SDL_GetThreadID(GameThread))
-	//{
-	// printf("%d, %d %d %d\n",SDL_ThreadID(), MainThreadID, getpid(), getppid());
-	// kill(getpid(), signum);
-	// for(;;) SDL_Delay(10000);
-	//}
-	//DriverKill();
         printf(_("\nSignal %d has been caught and dealt with...\n"),signum);
 
 	for(unsigned int x = 0; x < sizeof(SignalDefs) / sizeof(SignalInfo); x++)
+	{
 	 if(SignalDefs[x].number == signum)
 	 {
 	  printf("%s", _(SignalDefs[x].message));
+	  if(SignalDefs[x].SafeTryExit)
+	  {
+	   SDL_Event evt;
+
+	   memset(&evt, 0, sizeof(SDL_Event));
+
+	   evt.user.type = SDL_QUIT;
+	   SDL_PushEvent(&evt);
+	   return;
+	  }
+
 	  break;
 	 }
-
+	}
         if(GameThread && SDL_ThreadID() == MainThreadID)
 	{
 	 SDL_KillThread(GameThread);
@@ -412,7 +409,7 @@ static ARGPSTRUCT *MDFN_Internal_Args = NULL;
 
 static int HokeyPokeyFallDown(const char *name, const char *value)
 {
- if(!MDFNI_SetSetting(name + 1, value))
+ if(!MDFNI_SetSetting(name, value))
   return(0);
  return(1);
 }
@@ -443,10 +440,8 @@ static void MakeMednafenArgsStruct(void)
 
  for(x = 0; x < settings->size(); x++)
  {
-  MDFN_Internal_Args[x].name = (char *)malloc(strlen( (*settings)[x].name) + 1 + 1);
-  MDFN_Internal_Args[x].name[0] = '-';
-  strcpy(MDFN_Internal_Args[x].name + 1, (*settings)[x].name);
-  MDFN_Internal_Args[x].description = (*settings)[x].desc->description;
+  MDFN_Internal_Args[x].name = strdup((*settings)[x].name);
+  MDFN_Internal_Args[x].description = _((*settings)[x].desc->description);
   MDFN_Internal_Args[x].var = NULL;
   MDFN_Internal_Args[x].subs = (void *)HokeyPokeyFallDown;
   MDFN_Internal_Args[x].substype = 0x2000;
@@ -460,15 +455,18 @@ static int netconnect = 0;
 static char * loadcd = NULL;
 static int DoArgs(int argc, char *argv[], char **filename)
 {
+	int ShowCLHelp = 0;
+
         ARGPSTRUCT MDFNArgs[]={
-	 {"-loadcd", "Load and boot a CD for the specified system.", 0, &loadcd, 0x4001},
-         {"-inputcfg", "Configure mapping of physical device inputs to specified virtual device.", 0, (void *)MDFND_InputCfg,0x2000},
-         {"-ckconfig", "Configure mapping of physical buttons to specified command.", 0, (void *)MDFND_CKConfig, 0x2000},
-	 {"-soundrecord", "Record sound output to the specified filename in the MS WAV format.", 0,&soundrecfn,0x4001},
+	 {"help", gettext_noop("Show help!"), &ShowCLHelp, 0, 0 },
+	 #ifdef NEED_CDEMU
+	 {"loadcd", gettext_noop("Load and boot a CD for the specified system."), 0, &loadcd, 0x4001},
+	 #endif
+	 {"soundrecord", gettext_noop("Record sound output to the specified filename in the MS WAV format."), 0,&soundrecfn,0x4001},
          {0,NULL, (int *)MDFN_Internal_Args, 0, 0},
 
          #ifdef NETWORK
-	 {"-connect", "Connect to the remote server and start network play.", &netconnect, 0, 0 },
+	 {"connect", gettext_noop("Connect to the remote server and start network play."), &netconnect, 0, 0 },
          #endif
 	 {0,0,0,0}
         };
@@ -481,17 +479,18 @@ static int DoArgs(int argc, char *argv[], char **filename)
 	 printf(_("\tPlease refer to the documentation for option parameters and usage.\n\n"));
 	 return(0);
 	}
-	else if(argc == 2 && (!strcasecmp(argv[1], "-help") || !strcasecmp(argv[1], "--help")))
-	{
-         printf(usage_string, argv[0]);
-	 ShowArgumentsHelp(MDFNArgs);
-	 return(0);
-	}
 	else
 	{
 	 if(!ParseArguments(argc - 1, &argv[1], MDFNArgs, filename))
 	  return(0);
 
+	 if(ShowCLHelp)
+	 {
+          printf(usage_string, argv[0]);
+          ShowArgumentsHelp(MDFNArgs);
+	  return(0);
+	 }
+
 	 if(*filename == NULL && loadcd == NULL)
 	 {
 	  puts(_("No game filename specified!"));
@@ -518,18 +517,21 @@ int LoadGame(const char *path)
 	pending_save_movie = 0;
 	pending_snapshot = 0;
 
+	#ifdef NEED_CDEMU
 	if(loadcd)
 	{
 	 if(!(tmp = MDFNI_LoadCD(loadcd, path)))
 		return(0);
 	}
 	else
+	#endif
 	{
          if(!(tmp=MDFNI_LoadGame(path)))
 	  return 0;
 	}
-	CurGame=tmp;
-        ParseGI(tmp);
+	CurGame = tmp;
+	InitOtherInput(tmp);
+
         RefreshThrottleFPS(1);
         SDL_mutexP(VTMutex);
         NeedVideoChange = -1;
@@ -568,15 +570,19 @@ int CloseGame(void)
 	if(MDFN_GetSettingB("autosave"))
 	 MDFNI_SaveState(NULL, "ncq", NULL, NULL);
 
+	ResyncOtherInput();
+
 	MDFNI_CloseGame();
-	DriverKill();
+
+        KillOtherInput();
+	KillJoysticks();
+	KillSound();
 
 	CurGame = 0;
 
 	if(soundrecfn)
          MDFNI_EndWaveRecord();
 
-	InputUserActiveFix();
 	return(1);
 }
 
@@ -610,19 +616,19 @@ static int GameLoopPaused = 0;
 
 void DebuggerFudge(void)
 {
+          LockGameMutex(0);
+
 	  int MeowCowHowFlown = VTBackBuffer;
 
           MDFND_Update((uint32 *)VTBuffer[VTBackBuffer], NULL, 0);
 	  VTBackBuffer = MeowCowHowFlown;
 
-	  LockDebuggerMutex(0);
-
 	  if(sound_active)
 	   WriteSoundSilence(10);
 	  else
 	   SDL_Delay(10);
 
-	  LockDebuggerMutex(1);
+	  LockGameMutex(1);
 }
 
 int GameLoop(void *arg)
@@ -631,7 +637,7 @@ int GameLoop(void *arg)
 	{
          int16 *sound;
          int32 ssize;
-         static int fskip=0;
+         int fskip;
         
 	 /* If we requested a new video mode, wait until it's set before calling the emulation code again.
 	 */
@@ -649,6 +655,7 @@ int GameLoop(void *arg)
 	 } while(InFrameAdvance && !NeedFrameAdvance);
 
          fskip = ThrottleCheckFS();
+
 	 if(pending_snapshot || pending_save_state || pending_save_movie || NeedFrameAdvance)
 	  fskip = 0;
 
@@ -659,9 +666,9 @@ int GameLoop(void *arg)
 
 	 int ThisBackBuffer = VTBackBuffer;
 
-	 LockDebuggerMutex(1);
+	 LockGameMutex(1);
          MDFNI_Emulate((uint32 *)VTBuffer[VTBackBuffer], (MDFN_Rect *)VTLineWidths[VTBackBuffer], &sound, &ssize, fskip, CurGameSpeed);
-	 LockDebuggerMutex(0);
+	 LockGameMutex(0);
 	 FPS_IncVirtual();
 	 if(!fskip)
 	  FPS_IncDrawn();
@@ -723,12 +730,6 @@ static volatile SDL_Event gtevents[16384];
 static volatile int gte_read = 0;
 static volatile int gte_write = 0;
 
-
-#ifdef NETWORK
-extern int MDFNDnetplay;
-#endif
-
-
 /* This function may also be called by the main thread if a game is not loaded. */
 static void GameThread_HandleEvents(void)
 {
@@ -740,15 +741,8 @@ static void GameThread_HandleEvents(void)
   if(EventHook)
    EventHook((const SDL_Event *)event);
 
-  switch(event->type)
-  {
-   case SDL_USEREVENT:
-	switch(event->user.code)
-	{
-	 case CEVT_NETPLAYTEXT: if(CurGame && MDFNDnetplay) { MDFNI_NetplayText((uint8 *)event->user.data1); } free(event->user.data1); break;
-	 case CEVT_NETPLAYCONNECT: if(CurGame) { if(*(int *)event->user.data1) MDFND_NetworkConnect(); else MDFND_NetworkClose(); free(event->user.data1); } break;
-	}
-  }
+  NetplayEventHook_GT((const SDL_Event *)event);
+
   gte_read = (gte_read + 1) & 16383;
  }
  SDL_mutexV(EVMutex);
@@ -760,6 +754,15 @@ void PauseGameLoop(bool p)
 }
 
 
+void SendCEvent(unsigned int code, void *data1, void *data2)
+{
+ SDL_Event evt;
+ evt.user.type = SDL_USEREVENT;
+ evt.user.code = code;
+ evt.user.data1 = data1;
+ evt.user.data2 = data2;
+ SDL_PushEvent(&evt);
+}
 
 void SDL_MDFN_ShowCursor(int toggle)
 {
@@ -775,12 +778,14 @@ void SDL_MDFN_ShowCursor(int toggle)
 }
 
 static bool krepeat = 0;
-
 void PumpWrap(void)
 {
  SDL_Event event;
+ bool NITI;
 
- if(Debugger_IsActive() || Netplay_GetTextView() || IsConsoleCheatConfigActive())
+ NITI = Netplay_IsTextInput();
+
+ if(Debugger_IsActive() || NITI || IsConsoleCheatConfigActive() || Help_IsActive())
  {
   if(!krepeat)
    SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
@@ -797,10 +802,11 @@ void PumpWrap(void)
  {
   if(Debugger_IsActive())
    Debugger_Event(&event);
-  else if(Netplay_GetTextView())
-   NetplayEventHook(&event);
-  else if(IsConsoleCheatConfigActive())
-  CheatEventHook(&event);
+  else 
+   if(IsConsoleCheatConfigActive())
+    CheatEventHook(&event);
+
+  NetplayEventHook(&event);
 
   /* This is a very ugly hack for some joystick hats that don't behave very well. */
   if(event.type == SDL_JOYHATMOTION)
@@ -819,6 +825,8 @@ void PumpWrap(void)
     if(docon) continue;
    }
   } // && event.jhat.
+  //if(event.type == SDL_JOYAXISMOTION) printf("Which: %d, axis: %d, value: %d\n", event.jaxis.which, event.jaxis.axis, event.jaxis.value);
+
   /* Handle the event, and THEN hand it over to the GUI. Order is important due to global variable mayhem(CEVT_TOGGLEFS. */
   switch(event.type)
   {
@@ -828,15 +836,29 @@ void PumpWrap(void)
    case SDL_USEREVENT:
 		switch(event.user.code)
 		{
-		 case CEVT_NETPLAYTOGGLEVIEW: Netplay_ToggleTextView(); break;
+		 case CEVT_SET_STATE_STATUS: MT_SetStateStatus((StateStatusStruct *)event.user.data1); break;
+                 case CEVT_SET_MOVIE_STATUS: MT_SetMovieStatus((StateStatusStruct *)event.user.data1); break;
+		 case CEVT_WANT_EXIT:
+		     if(!Netplay_TryTextExit())
+		     {
+		      SDL_Event evt;
+		      evt.quit.type = SDL_QUIT;
+		      SDL_PushEvent(&evt);
+		     }
+		     break;
+	         case CEVT_SET_GRAB_INPUT:
+                         SDL_WM_GrabInput(*(int *)event.user.data1 ? SDL_GRAB_ON : SDL_GRAB_OFF);
+                         free(event.user.data1);
+                         break;
 		 case CEVT_TOGGLEFS: NeedVideoChange = 1; break;
 		 case CEVT_VIDEOSYNC: NeedVideoChange = -1; break;
 		 case CEVT_PRINTERROR: MDFND_PrintError((char *)event.user.data1); free(event.user.data1); break;
 		 case CEVT_PRINTMESSAGE: MDFND_Message((char *)event.user.data1); free(event.user.data1); break;
-		 case CEVT_DISPLAYNETPLAYTEXT: NetplayText_InMainThread((uint8 *)event.user.data1); free(event.user.data1); break;
 		 case CEVT_SHOWCURSOR: SDL_ShowCursor(*(int *)event.user.data1); free(event.user.data1); break;
+	  	 case CEVT_DISP_MESSAGE: VideoShowMessage((UTF8*)event.user.data1); break;
+		 default: memcpy((void *)&gtevents[gte_write], &event, sizeof(SDL_Event)); gte_write = (gte_write + 1) & 16383; break;
 		}
-		//break; // Make sure SDL_USEREVENT events propagate to the game thread.
+		break;
    default: memcpy((void *)&gtevents[gte_write], &event, sizeof(SDL_Event)); gte_write = (gte_write + 1) & 16383; break;
   }
  }
@@ -867,12 +889,39 @@ void SetJoyReadMode(int mode)	// 0 for events, 1 for manual function calling to
 }
 
 
+bool MT_FromRemote_SoundSync(void)
+{
+ GameThreadRun = 0;
+ SDL_WaitThread(GameThread, NULL);
+
+ KillSound();
+ sound_active = 0;
+
+ if(MDFN_GetSettingB("sound"))
+  sound_active = InitSound(CurGame);
+
+ GameThreadRun = 1;
+ GameThread = SDL_CreateThread(GameLoop, NULL);
+}
+
+bool MT_FromRemote_VideoSync(void)
+{
+          KillVideo(0);
+
+          memset(VTBuffer[0], 0, 256 * 256 * sizeof(uint32));
+          memset(VTBuffer[1], 0, 256 * 256 * sizeof(uint32));
+
+          if(!InitVideo(CurGame))
+	   return(0);
+	  return(1);
+}
+
 static uint64 tfreq;
 static uint64 desiredfps;
 
 void RefreshThrottleFPS(int multiplier)
 {
-        desiredfps=((uint64)CurGame->fps * multiplier)>>8;
+        desiredfps = ((uint64)CurGame->fps * multiplier) >> 8;
 	tfreq=10000000;
         tfreq<<=16;    /* Adjustment for fps */
         CurGameSpeed = multiplier;
@@ -908,14 +957,16 @@ int main(int argc, char *argv[])
 	 return(-1);
 	}
 
-	// SDL_GL_LoadLibrary returns 0 on success, -1 on failure
-	if(SDL_GL_LoadLibrary(NULL) == 0)
-         sdlhaveogl = 1;
-	else
-	 sdlhaveogl = 0;
+	InitSTDIOInterface();
+
+	for(unsigned int x = 0; x < sizeof(DriverSettings) / sizeof(MDFNSetting); x++)
+	 NeoDriverSettings.push_back(DriverSettings[x]);
+
+	MakeInputSettings(NeoDriverSettings);
 
 	MainThreadID = SDL_ThreadID();
-        if(!(ret=MDFNI_Initialize(DrBaseDirectory, DriverSettings)))
+
+        if(!(ret=MDFNI_Initialize(DrBaseDirectory, NeoDriverSettings)))
          return(-1);
 
         SDL_EnableUNICODE(1);
@@ -925,18 +976,28 @@ int main(int argc, char *argv[])
         #endif
 
 	CreateDirs();
-	MakeInputConfigStruct();
 	MakeMednafenArgsStruct();
-        LoadInputMapConfig();
 
 	if(!DoArgs(argc,argv, &needie))
 	{
-	 SaveInputMapConfig();
 	 MDFNI_Kill();
 	 DeleteInternalArgs();
+	 KillOtherInputSettings();
 	 return(-1);
 	}
 
+        if(!getenv("__GL_SYNC_TO_VBLANK"))
+	{
+ 	 if(MDFN_GetSettingB("glvsync"))
+	 {
+	  #if HAVE_PUTENV
+	  putenv("__GL_SYNC_TO_VBLANK=1");
+	  #elif HAVE_SETENV
+	  setenv("__GL_SYNC_TO_VBLANK", "1", 0);
+	  #endif
+	 }
+	}
+
 	/* Now the fun begins! */
 	/* Run the video and event pumping in the main thread, and create a 
 	   secondary thread to run the game in(and do sound output, since we use
@@ -947,9 +1008,10 @@ int main(int argc, char *argv[])
 
 	VTMutex = SDL_CreateMutex();
         EVMutex = SDL_CreateMutex();
-	DebuggerMutex = SDL_CreateMutex();
+	GameMutex = SDL_CreateMutex();
 
 	VTReady = NULL;
+	VTLWReady = NULL;
 
 	NeedVideoChange = -1;
 
@@ -966,17 +1028,20 @@ int main(int argc, char *argv[])
          VTBuffer[1] = (uint32 *)malloc(CurGame->pitch * 256);
 	 VTLineWidths[0] = (MDFN_Rect *)calloc(256, sizeof(MDFN_Rect));
 	 VTLineWidths[1] = (MDFN_Rect *)calloc(256, sizeof(MDFN_Rect));
-	 VTOSDBuffer[0] = (uint32 *)calloc(512 * 240, sizeof(uint32));
-	 VTOSDBuffer[1] = (uint32 *)calloc(512 * 240, sizeof(uint32));
          NeedVideoChange = -1;
         }
 	else
 	 NeedExitNow = 1;
 
 	FPS_Init();
+	#ifdef WANT_DEBUGGER
+	MemDebugger_Init();
+	#endif
 
 	while(!NeedExitNow)
 	{
+	 CheckForSTDIOMessages();
+
 	 SDL_mutexP(VTMutex);	/* Lock mutex */
 
 	 if(JoyModeChange)
@@ -1008,11 +1073,11 @@ int main(int argc, char *argv[])
 	  }
 	  else
 	  {
-	   MDFNI_SetSetting("fs", !MDFN_GetSettingB("fs"));
+	   MDFNI_SetSettingB("fs", !MDFN_GetSettingB("fs"));
 
 	   if(!InitVideo(CurGame))
 	   {
-            MDFNI_SetSetting("fs", !MDFN_GetSettingB("fs"));
+            MDFNI_SetSettingB("fs", !MDFN_GetSettingB("fs"));
 	    InitVideo(CurGame);
 	   }
 	  }
@@ -1021,9 +1086,7 @@ int main(int argc, char *argv[])
 
 	 if(VTReady)
 	 {
-	  BlitScreen((uint32 *)VTReady, (MDFN_Rect *)&VTDisplayRect, (MDFN_Rect*)VTLWReady, (uint32 *)VTOSDReady);
-	  if(VTOSDReady)
-	   memset((void *)VTOSDReady, 0, 512 * 224 * sizeof(uint32));
+	  BlitScreen((uint32 *)VTReady, (MDFN_Rect *)&VTDisplayRect, (MDFN_Rect*)VTLWReady);
 	  VTReady = NULL;
 	 } 
 	 PumpWrap();
@@ -1036,40 +1099,34 @@ int main(int argc, char *argv[])
 	SDL_DestroyMutex(VTMutex);
         SDL_DestroyMutex(EVMutex);
 
-	if(VTBuffer[0])
+	for(int x = 0; x < 2; x++)
 	{
-	 free(VTBuffer[0]);
-	 VTBuffer[0] = NULL;
-	}
+	 if(VTBuffer[x])
+	 {
+	  free(VTBuffer[x]);
+	  VTBuffer[x] = NULL;
+	 }
 
-	if(VTBuffer[1])
-	{
-	 free(VTBuffer[1]);
-	 VTBuffer[1] = NULL;
+	 if(VTLineWidths[x])
+	 {
+	  free(VTLineWidths[x]);
+	  VTLineWidths[x] = NULL;
+	 }
 	}
 
 	#ifdef HAVE_SIGNAL
 	SetSignals(SIG_IGN);
 	#endif
 
-	SaveInputMapConfig();
         MDFNI_Kill();
-
 	SDL_Quit();
 
 	DeleteInternalArgs();
+	KillOtherInputSettings();
 
         return(0);
 }
 
-static void DriverKill(void)
-{
- if(inited&2)
-  KillJoysticks();
- KillSound();
- inited=0;
-}
-
 static uint32 last_btime = 0;
 static uint64 ttime,ltime=0;
 static int skipcount = 0;
@@ -1090,24 +1147,31 @@ static int ThrottleCheckFS(void)
   if(!sound_active && !NoWaiting && !nothrottle && GameThreadRun)
   {
    int64 delay;
-   delay=(tfreq/desiredfps)-(ttime-ltime);
-   if(delay>0)
-    SDL_Delay(delay/10000);
+   delay = (((tfreq/desiredfps)-(ttime-ltime)) / 10000) - 1;
+
+   if(delay >= 0)
+    SDL_Delay(delay);
+
    goto waiter;
   }
  }
 
- extern int MDFNDnetplay;
  if(!MDFNDnetplay)
   if(((ttime-ltime) >= (1.5*tfreq/desiredfps)))
   {
+   //MDFN_DispMessage((UTF8*)"%8d %8d %8d, %8d", ttime, ltime, ttime-ltime, tfreq / desiredfps);
    if(skipcount < 4 || (CurGameSpeed > 1 && skipcount < CurGameSpeed))     // Only skip four frames in a row at maximum.
    {
     skipcount ++;
     needskip = 1;
    } else skipcount = 0;
-   if(!sound_active) // && (ttime-ltime) >= ((uint64)3.0*tfreq/desiredfps))    // Only adjust base time if sound is disabled.
-    ltime=ttime;
+   if(!sound_active)    // Only adjust base time if sound is disabled.
+   {
+    if((ttime-ltime) >= ((uint64)3.0*tfreq/desiredfps))
+     ltime=ttime;
+    else
+     ltime += tfreq / desiredfps;
+   }
   }
   else
    ltime+=tfreq/desiredfps;
@@ -1115,6 +1179,20 @@ static int ThrottleCheckFS(void)
  return(needskip);
 }
 
+static int GetSafeWaitTime(void)
+{
+ uint64 curtime = (uint64)SDL_GetTicks() * 10000;
+ int64 delay = 0;
+
+ if((curtime - ltime) < (tfreq / desiredfps ))
+ {
+  delay=((tfreq/desiredfps)-(ttime-ltime)) / 10000;
+
+  if(delay < 0) delay = 0;
+ }
+
+ return(delay);
+}
 
 void MDFND_Update(uint32 *XBuf, int16 *Buffer, int Count)
 {
@@ -1135,10 +1213,9 @@ void MDFND_Update(uint32 *XBuf, int16 *Buffer, int Count)
   {
    ltime = ttime;		// Resynchronize
   }
+
   WriteSound(Buffer, Count);
 
-  #ifdef NETWORK
-  extern int MDFNDnetplay;
   if(MDFNDnetplay && GetWriteSound() >= Count * 1.00) // Cheap code to fix sound buffer underruns due to accumulation of timer error during netplay.
   {
    int16 zbuf[128 * 2];
@@ -1151,22 +1228,19 @@ void MDFND_Update(uint32 *XBuf, int16 *Buffer, int Count)
     t -= 128;
    }
    ltime = ttime;
-   //puts("Ack");
   }
-  #endif
 
  }
 
  MDFND_UpdateInput();
 
- bool osd_drawn = 0;
  if(XBuf)
  {
   if(pending_snapshot)
    MDFNI_SaveSnapshot();
 
   if(pending_save_state || pending_save_movie)
-   LockDebuggerMutex(1);
+   LockGameMutex(1);
 
   if(pending_save_state)
    MDFNI_SaveState(NULL, NULL, XBuf, (MDFN_Rect *)VTLineWidths[VTBackBuffer]);
@@ -1174,20 +1248,14 @@ void MDFND_Update(uint32 *XBuf, int16 *Buffer, int Count)
    MDFNI_SaveMovie(NULL, XBuf, (MDFN_Rect *)VTLineWidths[VTBackBuffer]);
 
   if(pending_save_state || pending_save_movie)
-   LockDebuggerMutex(0);
+   LockGameMutex(0);
 
   pending_save_movie = pending_snapshot = pending_save_state = 0;
   MDFN_Rect toorect;
   toorect.x = 0;
   toorect.y = 0;
-  toorect.w = 256;
-  toorect.h = 224;
-  osd_drawn = MDFNI_DrawOverlay(XBuf, (uint32 *)VTOSDBuffer[VTBackBuffer], &toorect, 512);
-  if(Netplay_GetTextView() || IsConsoleCheatConfigActive() || Debugger_IsActive())
-   osd_drawn = 1;
-
-  if(FPS_IsActive())
-   osd_drawn = 1;
+  toorect.w = 384;
+  toorect.h = 336;
 
   /* If it's been >= 100ms since the last blit, assume that the blit
      thread is being time-slice starved, and let it run.  This is especially necessary
@@ -1199,22 +1267,36 @@ void MDFND_Update(uint32 *XBuf, int16 *Buffer, int Count)
    //puts("Eep");
    while(VTReady && GameThreadRun) SDL_Delay(1);
   }
+
+  //if(VTReady)
+  //{
+  // int delay_time = GetSafeWaitTime();
+  // delay_time--;
+  //
+  // if(delay_time > 0)
+  // {
+  //  MDFN_DispMessage((UTF8*)"%d", delay_time);
+  //  SDL_Delay(delay_time);
+  // }
+  //}
+  
   if(!VTReady)
   {
    skipcount = 0;
    memcpy((void *)&VTDisplayRect, &CurGame->DisplayRect, sizeof(MDFN_Rect));
-   VTOSDReady = osd_drawn ? VTOSDBuffer[VTBackBuffer] : NULL;
+
    VTLWReady = VTLineWidths[VTBackBuffer];
+
    VTReady = VTBuffer[VTBackBuffer];
+
    VTBackBuffer ^= 1;
    last_btime = SDL_GetTicks();
    FPS_IncBlitted();
   }
  }
- else if(Netplay_GetTextView() || IsConsoleCheatConfigActive() && !VTReady)
+ else if(IsConsoleCheatConfigActive() && !VTReady)
  {
   VTBackBuffer ^= 1;
-  VTOSDReady = VTOSDBuffer[VTBackBuffer];
   VTLWReady = VTLineWidths[VTBackBuffer];
   VTReady = VTBuffer[VTBackBuffer];
   VTBackBuffer ^= 1;
@@ -1226,3 +1308,20 @@ uint32 MDFND_GetTime(void)
 {
  return(SDL_GetTicks());
 }
+
+
+void MDFND_DispMessage(UTF8 *text)
+{
+ SendCEvent(CEVT_DISP_MESSAGE, text, NULL);
+}
+
+void MDFND_SetStateStatus(StateStatusStruct *status)
+{
+ SendCEvent(CEVT_SET_STATE_STATUS, status, NULL);
+}
+
+void MDFND_SetMovieStatus(StateStatusStruct *status)
+{
+ SendCEvent(CEVT_SET_MOVIE_STATUS, status, NULL);
+}
+
