@@ -23,6 +23,7 @@
 #include "../driver.h"
 #include "../mednafen.h"
 #include "../settings.h"
+#include "../endian.h"
 #include "config.h"
 #include "args.h"
 
@@ -40,15 +41,29 @@
 #define CEVT_PRINTERROR	3
 #define CEVT_PRINTMESSAGE	4
 #define CEVT_VIDEOSYNC	5
-#define CEVT_NETPLAYCONNECT	7
-#define CEVT_NETPLAYTEXT	8
-#define CEVT_NETPLAYGUISET	9
-#define CEVT_DISPLAYNETPLAYTEXT 0x0a
-#define CEVT_NETPLAYTOGGLEVIEW 	0x0b
 #define CEVT_SHOWCURSOR		0x0c
-
 #define CEVT_CHEATTOGGLEVIEW	0x10
 
+
+#define CEVT_DISP_MESSAGE	0x11
+
+#define CEVT_SET_GRAB_INPUT	0x20
+
+#define CEVT_SET_STATE_STATUS	0x40
+#define CEVT_SET_MOVIE_STATUS	0x41
+
+#define CEVT_WANT_EXIT		0x80 // Emulator exit or GUI exit or bust!
+
+
+#define CEVT_NP_TEXT_TO_SERVER	0x100
+#define CEVT_NP_DISPLAY_TEXT	0x101
+#define CEVT_NP_TOGGLE_VIEWABLE	0x102
+#define CEVT_NP_TOGGLE_TT	0x103
+#define CEVT_NP_CONNECT		0x104
+#define CEVT_NP_SETNICK		0x105
+
+void SendCEvent(unsigned int code, void *data1, void *data2);
+
 void PauseGameLoop(bool p);
 
 void SDL_MDFN_ShowCursor(int toggle);
@@ -72,11 +87,14 @@ void DoRunNormal(void);
 void DoFrameAdvance(void);
 
 
-void LockDebuggerMutex(bool lock);
+void LockGameMutex(bool lock);
 void DebuggerFudge(void);
 
 extern volatile int GameThreadRun;
 
 extern int sdlhaveogl;
 
+bool MT_FromRemote_VideoSync(void);
+bool MT_FromRemote_SoundSync(void);
+
 #endif
