@@ -46,6 +46,16 @@ glTexSubImage2D_Func p_glTexSubImage2D;
 glFinish_Func p_glFinish;
 glOrtho_Func p_glOrtho;
 glPixelTransferf_Func p_glPixelTransferf;
+glColorMask_Func p_glColorMask;
+glTexEnvf_Func p_glTexEnvf;
+glGetIntegerv_Func p_glGetIntegerv;
+glTexGend_Func p_glTexGend;
+glDrawPixels_Func p_glDrawPixels;
+glRasterPos2i_Func p_glRasterPos2i;
+glPixelZoom_Func p_glPixelZoom;
+glGetTexLevelParameteriv_Func p_glGetTexLevelParameteriv;
+glAccum_Func p_glAccum;
+glClearAccum_Func p_glClearAccum;
 
 #if MDFN_WANT_OPENGL_SHADERS
 glCreateShaderObjectARB_Func p_glCreateShaderObjectARB;
@@ -68,8 +78,10 @@ glDeleteObjectARB_Func p_glDeleteObjectARB;
 glDetachObjectARB_Func p_glDetachObjectARB;
 #endif
 
+static uint32 MaxTextureSize;
+
 static SDL_Surface *gl_screen;
-static GLuint textures[3]={0, 0, 0}; // emulated fb, scanlines, osd
+static GLuint textures[4]={0, 0, 0, 0}; // emulated fb, scanlines, osd, raw(netplay)
 static GLuint rgb_mask;
 
 static int using_scanlines = 0;
@@ -78,90 +90,77 @@ static float twitchy;
 
 extern uint32 uppow2(uint32);
 
-static unsigned int needglclear = 0;
-static bool OSDTextureCreated;
+static uint32 OSDLastWidth, OSDLastHeight;
 
 #if MDFN_WANT_OPENGL_SHADERS
 static bool UsingShader;
 #endif
 
-void BlitOpenGLOSD(uint32 *OSDBuffer, const SDL_Rect *OSDRect)
+void BlitOpenGLRaw(SDL_Surface *surface, const SDL_Rect *rect, const SDL_Rect *dest_rect)
 {
-  int multiplierx = gl_screen->w / 256;
-  int multipliery = gl_screen->h / 224;
-  int multiplier;
+ unsigned int tmpwidth = uppow2(rect->w);
+ unsigned int tmpheight = uppow2(rect->h);
 
-  multiplier = multiplierx;
-  if(multipliery < multiplier)
-   multiplier = multipliery;
+ if(tmpwidth > MaxTextureSize || tmpheight > MaxTextureSize)
+ {
+  SDL_Rect neo_rect;
+  SDL_Rect neo_dest_rect;
 
-  p_glBindTexture(GL_TEXTURE_2D, textures[2]);
-  p_glPixelStorei(GL_UNPACK_ROW_LENGTH, 512);
-
-  if(!OSDTextureCreated)
+  for(uint32 xseg = 0; xseg < rect->w; xseg += MaxTextureSize)
   {
-   p_glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 512, 256, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
-   OSDTextureCreated = 1;
+   for(uint32 yseg = 0; yseg < rect->h; yseg += MaxTextureSize)
+   {
+    neo_rect.x = rect->x + xseg;
+    neo_rect.w = rect->w - xseg;
+    if(neo_rect.w > MaxTextureSize)
+     neo_rect.w = MaxTextureSize;
+    neo_rect.y = rect->y + yseg;
+    neo_rect.h = rect->h - yseg;
+    if(neo_rect.h > MaxTextureSize)
+     neo_rect.h = MaxTextureSize;
+
+
+    neo_dest_rect.x = dest_rect->x + xseg * dest_rect->w / rect->w;
+    neo_dest_rect.y = dest_rect->y + yseg * dest_rect->h / rect->h;
+    neo_dest_rect.w = neo_rect.w * dest_rect->w / rect->w;
+    neo_dest_rect.h = neo_rect.h * dest_rect->h / rect->h;
+    BlitOpenGLRaw(surface, &neo_rect, &neo_dest_rect);
+   }
   }
 
-  p_glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, OSDRect->w, 224, GL_RGBA, GL_UNSIGNED_BYTE, OSDBuffer);
-
+ }
+ else
+ {
   p_glEnable(GL_BLEND);
   p_glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
-  if((multiplier&1) && OSDRect->w == 512)
-  {
-   multiplier &= ~1;
-  }
+  p_glBindTexture(GL_TEXTURE_2D, textures[3]);
+  p_glPixelStorei(GL_UNPACK_ROW_LENGTH, surface->pitch >> 2);
 
-  if(!multiplier || ((double)gl_screen->w / (256 * multiplier) >= 1.75) || ((double)gl_screen->h / (224 * multiplier) >= 1.75) )
-  {
-   p_glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
-   p_glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
+  p_glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, tmpwidth, tmpheight, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
+  p_glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, rect->w, rect->h, GL_RGBA, GL_UNSIGNED_BYTE, (uint32 *)surface->pixels + rect->x + rect->y * (surface->pitch >> 2));
 
-   p_glBegin(GL_QUADS);
-   p_glTexCoord2f(0.0f, 1.0f * 224 / 256);  // Bottom left of our picture.
-   p_glVertex2f(0, gl_screen->h);
+  p_glBegin(GL_QUADS);
 
-   p_glTexCoord2f((float)OSDRect->w / 512, 1.0f * 224 / 256); // Bottom right of our picture.
-   p_glVertex2f(gl_screen->w, gl_screen->h);
+  p_glTexCoord2f(0.0f, 1.0f * rect->h / tmpheight);  // Bottom left of our picture.
+  p_glVertex2f(dest_rect->x, dest_rect->y + dest_rect->h);
 
-   p_glTexCoord2f((float)OSDRect->w / 512, 0.0f);    // Top right of our picture.
-   p_glVertex2f(gl_screen->w,  0);
+  p_glTexCoord2f((float)rect->w / tmpwidth, 1.0f * rect->h / tmpheight); // Bottom right of our picture.
+  p_glVertex2f(dest_rect->x + dest_rect->w, dest_rect->y + dest_rect->h);
 
-   p_glTexCoord2f(0.0f, 0.0f);     // Top left of our picture.
-   p_glVertex2f(0, 0);
-  }
-  else
-  {
-   int dest_left = (gl_screen->w - (256 * multiplier)) / 2;
-   int dest_right = dest_left + 256 * multiplier;
-   int dest_top = (gl_screen->h - (224 * multiplier)) / 2;
-   int dest_bottom = dest_top + 224 * multiplier;
+  p_glTexCoord2f((float)rect->w / tmpwidth, 0.0f);    // Top right of our picture.
+  p_glVertex2f(dest_rect->x + dest_rect->w,  dest_rect->y);
 
-   p_glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
-   p_glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
+  p_glTexCoord2f(0.0f, 0.0f);     // Top left of our picture.
+  p_glVertex2f(dest_rect->x, dest_rect->y);
 
-   p_glBegin(GL_QUADS);
-   p_glTexCoord2f(0.0f, 1.0f * 224 / 256);  // Bottom left of our picture.
-   p_glVertex2f(dest_left, dest_bottom);
-
-   p_glTexCoord2f((float)OSDRect->w / 512, 1.0f * 224 / 256); // Bottom right of our picture.
-   p_glVertex2f(dest_right, dest_bottom);
-
-   p_glTexCoord2f((float)OSDRect->w / 512, 0.0f);    // Top right of our picture.
-   p_glVertex2f(dest_right,  dest_top);
-
-   p_glTexCoord2f(0.0f, 0.0f);     // Top left of our picture.
-   p_glVertex2f(dest_left, dest_top);
-  }
   p_glEnd();
-  p_glDisable(GL_BLEND);
 
-  needglclear = 2;
+  p_glDisable(GL_BLEND);
+ }
 }
 
-void BlitOpenGL(SDL_Surface *src_surface, const SDL_Rect *src_rect, const SDL_Rect *dest_rect, const SDL_Rect *original_src_rect)
+void BlitOpenGL(SDL_Surface *src_surface, const SDL_Rect *src_rect, const SDL_Rect *dest_rect, const SDL_Rect *original_src_rect, bool alpha_blend)
 {
  int left = src_rect->x;
  int right = src_rect->x + src_rect->w;	// First nonincluded pixel
@@ -194,57 +193,77 @@ void BlitOpenGL(SDL_Surface *src_surface, const SDL_Rect *src_rect, const SDL_Re
   ShaderBegin(src_surface, src_rect, tmpwidth, tmpheight);
  #endif
 
+ if(alpha_blend)
+ {
+  p_glEnable(GL_BLEND);
+  p_glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+ }
+
  p_glPixelStorei(GL_UNPACK_ROW_LENGTH, src_surface->pitch >> 2);
  p_glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, right, bottom, GL_RGBA, GL_UNSIGNED_BYTE, src_pixies);
 
+ bool DoneAgain = 0;
+
+ DoAgain:
  p_glBegin(GL_QUADS);
- if(CurGame)
+ if(CurGame->rotated != MDFN_ROTATE0)
  {
-  if(CurGame->rotated)
+  if(CurGame->rotated == MDFN_ROTATE90)
   {
-   if(CurGame->rotated == 1)
-   {
-    p_glTexCoord2f(0, 0);
-     p_glVertex2f(dest_rect->x, dest_rect->y + dest_rect->h);
+   p_glTexCoord2f(0, 0);
+    p_glVertex2f(dest_rect->x, dest_rect->y + dest_rect->h);
 
-    p_glTexCoord2f(0, (float)src_rect->h / tmpheight);
-     p_glVertex2f(dest_rect->x + dest_rect->w, dest_rect->y + dest_rect->h);
+   p_glTexCoord2f(0, (float)src_rect->h / tmpheight);
+    p_glVertex2f(dest_rect->x + dest_rect->w, dest_rect->y + dest_rect->h);
 
-    p_glTexCoord2f(1.0f * right / tmpwidth, (float)src_rect->h / tmpheight);
-     p_glVertex2f(dest_rect->x + dest_rect->w,  dest_rect->y);
+   p_glTexCoord2f(1.0f * right / tmpwidth, (float)src_rect->h / tmpheight);
+    p_glVertex2f(dest_rect->x + dest_rect->w,  dest_rect->y);
 
-    p_glTexCoord2f(1.0f * right / tmpwidth, 0);
-     p_glVertex2f(dest_rect->x,  dest_rect->y);
-   }
-   else
-   {
-    p_glTexCoord2f(1.0f * right / tmpwidth, (float)src_rect->h / tmpheight);
-     p_glVertex2f(dest_rect->x, dest_rect->y + dest_rect->h);
-    p_glTexCoord2f(1.0f * right / tmpwidth, 0);
-     p_glVertex2f(dest_rect->x + dest_rect->w, dest_rect->y + dest_rect->h);
-    p_glTexCoord2f(0, 0);
-     p_glVertex2f(dest_rect->x + dest_rect->w,  dest_rect->y);
-    p_glTexCoord2f(0, (float)src_rect->h / tmpheight);
-     p_glVertex2f(dest_rect->x,  dest_rect->y);
-   }
+   p_glTexCoord2f(1.0f * right / tmpwidth, 0);
+    p_glVertex2f(dest_rect->x,  dest_rect->y);
   }
   else
   {
-   p_glTexCoord2f(0, (1.0f * src_rect->h) / tmpheight);
-    p_glVertex2f(dest_rect->x, twitchy + dest_rect->y + dest_rect->h);
-
-   p_glTexCoord2f(1.0f * right / tmpwidth, (1.0f * src_rect->h) / tmpheight);
-    p_glVertex2f(dest_rect->x + dest_rect->w, twitchy + dest_rect->y + dest_rect->h);
-
+   p_glTexCoord2f(1.0f * right / tmpwidth, (float)src_rect->h / tmpheight);
+    p_glVertex2f(dest_rect->x, dest_rect->y + dest_rect->h);
    p_glTexCoord2f(1.0f * right / tmpwidth, 0);
-    p_glVertex2f(dest_rect->x + dest_rect->w, twitchy + dest_rect->y);
-
+    p_glVertex2f(dest_rect->x + dest_rect->w, dest_rect->y + dest_rect->h);
    p_glTexCoord2f(0, 0);
-    p_glVertex2f(dest_rect->x, twitchy + dest_rect->y);
+    p_glVertex2f(dest_rect->x + dest_rect->w,  dest_rect->y);
+   p_glTexCoord2f(0, (float)src_rect->h / tmpheight);
+    p_glVertex2f(dest_rect->x,  dest_rect->y);
   }
  }
+ else
+ {
+  p_glTexCoord2f(0, (1.0f * src_rect->h) / tmpheight);
+   p_glVertex2f(dest_rect->x, twitchy + dest_rect->y + dest_rect->h);
+   p_glTexCoord2f(1.0f * right / tmpwidth, (1.0f * src_rect->h) / tmpheight);
+  p_glVertex2f(dest_rect->x + dest_rect->w, twitchy + dest_rect->y + dest_rect->h);
+  p_glTexCoord2f(1.0f * right / tmpwidth, 0);
+   p_glVertex2f(dest_rect->x + dest_rect->w, twitchy + dest_rect->y);
+
+  p_glTexCoord2f(0, 0);
+   p_glVertex2f(dest_rect->x, twitchy + dest_rect->y);
+ }
+
  p_glEnd();
 
+ if(alpha_blend)
+ {
+  if(!DoneAgain)
+  {
+   DoneAgain = TRUE;
+   p_glBlendFunc(GL_ONE_MINUS_SRC_ALPHA, GL_ONE);
+
+   goto DoAgain;
+  }
+  else
+  {
+   p_glDisable(GL_BLEND);
+  }
+ }
+
  #if MDFN_WANT_OPENGL_SHADERS
  if(UsingShader)
   ShaderEnd();
@@ -274,18 +293,19 @@ void BlitOpenGL(SDL_Surface *src_surface, const SDL_Rect *src_rect, const SDL_Re
   p_glEnd();
   p_glDisable(GL_BLEND);
  }
+
+ //if(1)
+ //{
+ // p_glAccum(GL_MULT, 0.99);
+ // p_glAccum(GL_ACCUM, 1 - 0.99);
+ // p_glAccum(GL_RETURN, 1.0);
+ //}
 }
 
 void FlipOpenGL(void)
 {
- p_glFinish();
  PumpWrap();
  SDL_GL_SwapBuffers();
- if(needglclear)
- {
-  needglclear--;
-  p_glClear(GL_COLOR_BUFFER_BIT);
- }
 }
 
 void KillOpenGL(void)
@@ -321,22 +341,31 @@ int InitOpenGL(int ipolate, int scanlines, std::string pixshader, SDL_Surface *s
  LFG(glLoadIdentity);
  LFG(glClear);
  LFG(glMatrixMode);
- LFG(glDisable);
-
  LFG(glDisable);
  LFG(glPixelStorei);
  LFG(glTexSubImage2D);
  LFG(glFinish);
  LFG(glOrtho);
  LFG(glPixelTransferf);
+ LFG(glColorMask);
+ LFG(glTexEnvf);
+ LFG(glGetIntegerv);
+ LFG(glTexGend);
+ LFG(glRasterPos2i);
+ LFG(glDrawPixels);
+ LFG(glPixelZoom);
+ LFG(glAccum);
+ LFG(glClearAccum);
+ LFG(glGetTexLevelParameteriv);
 
  gl_screen = screen;
 
+ MDFN_printf(_("OpenGL Implementation: %s %s %s\n"), p_glGetString(GL_VENDOR), p_glGetString(GL_RENDERER), p_glGetString(GL_VERSION));
  extensions=(const char*)p_glGetString(GL_EXTENSIONS);
 
  p_glViewport(0, 0, screen->w, screen->h);
 
- p_glGenTextures(3, &textures[0]);
+ p_glGenTextures(4, &textures[0]);
  p_glGenTextures(1, &rgb_mask);
  using_scanlines = 0;
 
@@ -426,6 +455,12 @@ int InitOpenGL(int ipolate, int scanlines, std::string pixshader, SDL_Surface *s
   p_glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, 64, 256 * 2, 0, GL_RGBA,GL_UNSIGNED_BYTE,buf);
   free(buf);
  }
+ p_glBindTexture(GL_TEXTURE_2D, textures[3]);
+ p_glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
+ p_glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
+ p_glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
+ p_glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
+
 
  p_glBindTexture(GL_TEXTURE_2D, textures[0]);
      
@@ -436,12 +471,6 @@ int InitOpenGL(int ipolate, int scanlines, std::string pixshader, SDL_Surface *s
 
  p_glBindTexture(GL_TEXTURE_2D, textures[2]);
 
- // Hmm...on my card, if I create the texture here(even when I specify non-NULL pixel data),
- // there is an annoying flash of white when the OSD texture is shown, so delay it
- // until right before the glTexImage2D() call.
- OSDTextureCreated = 0; 
- //p_glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 256, 256, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
-
  p_glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
  p_glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
 
@@ -449,8 +478,6 @@ int InitOpenGL(int ipolate, int scanlines, std::string pixshader, SDL_Surface *s
  p_glClearColor(0.0f, 0.0f, 0.0f, 0.0f);	// Background color to black.
  p_glMatrixMode(GL_MODELVIEW);
 
- //p_glViewport(0, 0, screen->w, screen->h);
- //p_glClear(GL_COLOR_BUFFER_BIT);
  p_glLoadIdentity();
  p_glFinish();
 
@@ -488,17 +515,50 @@ int InitOpenGL(int ipolate, int scanlines, std::string pixshader, SDL_Surface *s
  last_w = 0;
  last_h = 0;
 
+ OSDLastWidth = OSDLastHeight = 0;
+
+
+ MDFN_printf(_("Checking maximum texture size...\n"));
+ MDFN_indent(1);
+ p_glBindTexture(GL_TEXTURE_2D, textures[0]);
+ // Assume maximum texture width is the same as maximum texture height to greatly simplify things
+ MaxTextureSize = 32768;
+ 
+ while(MaxTextureSize)
+ {
+  GLint width_test = 0;
+
+  p_glTexImage2D(GL_PROXY_TEXTURE_2D, 0, GL_RGBA, MaxTextureSize, MaxTextureSize, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
+  p_glGetTexLevelParameteriv(GL_PROXY_TEXTURE_2D, 0, GL_TEXTURE_WIDTH, &width_test);
+
+  if((unsigned int)width_test == MaxTextureSize)
+   break;
+
+  MaxTextureSize >>= 1;
+ }
+ MDFN_printf(_("Apparently it is at least: %d x %d\n"), MaxTextureSize, MaxTextureSize);
+
+ if(MaxTextureSize < 256)
+ {
+  MDFN_printf(_("Warning:  Maximum texture size is reported as being less than 256, but we can't handle that.\n"));
+  MaxTextureSize = 256;
+ }
+
+ MDFN_indent(-1);
+
  return(1);
 }
 
-void ClearGLSurfaces(SDL_Surface *screen)
+void ClearBackBufferOpenGL(SDL_Surface *screen)
 {
- int x;
- for(x=0;x<2;x++)
- {
-  p_glClear(GL_COLOR_BUFFER_BIT | GL_ACCUM_BUFFER_BIT);
-  p_glFinish();
-  SDL_GL_SwapBuffers();
- }
+ //if(1)
+ //{
+ // p_glClearAccum(0.0, 0.0, 0.0, 1.0);
+ // p_glClear(GL_COLOR_BUFFER_BIT | GL_ACCUM_BUFFER_BIT);
+ //}
+ //else
+ //{
+  p_glClear(GL_COLOR_BUFFER_BIT);
+ //}
 }
 
