@@ -33,6 +33,8 @@
 #include "nnx.h"
 #include "debugger.h"
 #include "fps.h"
+#include "help.h"
+#include "video-state.h"
 
 typedef struct
 {
@@ -52,32 +54,30 @@ static int _fullscreen;
 
 typedef struct
 {
-	char *name;
+	const char *name;
 	int id;
 	int xscale;
 	int yscale;
-	int input_bpp;
-	int output_bpp;
 } ScalerDefinition;
 
 static ScalerDefinition Scalers[] = 
 {
 
-	{"hq2x", NTVB_HQ2X, 2, 2, 32, 32 },
-	{"hq3x", NTVB_HQ3X, 3, 3, 32, 32 },
-	{"hq4x", NTVB_HQ4X, 4, 4, 32, 32 },
+	{"hq2x", NTVB_HQ2X, 2, 2 },
+	{"hq3x", NTVB_HQ3X, 3, 3 },
+	{"hq4x", NTVB_HQ4X, 4, 4 },
 
-	{"scale2x", NTVB_SCALE2X, 2, 2, 0, 0 },
-	{"scale3x", NTVB_SCALE3X, 3, 3, 0, 0 },
-	{"scale4x", NTVB_SCALE4X, 4, 4, 0, 0 },
+	{"scale2x", NTVB_SCALE2X, 2, 2 },
+	{"scale3x", NTVB_SCALE3X, 3, 3 },
+	{"scale4x", NTVB_SCALE4X, 4, 4 },
 
-	{"nn2x", NTVB_NN2X, 2, 2, 32, 32},
-        {"nn3x", NTVB_NN3X, 3, 3, 32, 32},
-        {"nn4x", NTVB_NN4X, 4, 4, 32, 32},
+	{"nn2x", NTVB_NN2X, 2, 2 },
+        {"nn3x", NTVB_NN3X, 3, 3 },
+        {"nn4x", NTVB_NN4X, 4, 4 },
 
-	{"nny2x", NTVB_NNY2X, 1, 2, 32, 32 },
-	{"nny3x", NTVB_NNY3X, 1, 3, 32, 32 },
-	{"nny4x", NTVB_NNY4X, 1, 4, 32, 32 },
+	{"nny2x", NTVB_NNY2X, 1, 2 },
+	{"nny3x", NTVB_NNY3X, 1, 3 },
+	{"nny4x", NTVB_NNY4X, 1, 4 },
 	{ 0 }
 };
 
@@ -87,15 +87,28 @@ static int cur_xres, cur_yres, cur_flags;
 
 static ScalerDefinition *CurrentScaler = NULL;
 
-static SDL_Surface *OSDSurface;
-
 static SDL_Surface *source_surface;
 static SDL_Surface *screen = NULL;
 static SDL_Surface *IconSurface=NULL;
 
 static SDL_Rect src_rect;
 static SDL_Rect dest_rect;
-static SDL_Rect OSDRect;
+
+static SDL_Surface *DebuggerSurface;
+static SDL_Rect DebuggerRect;
+
+static SDL_Surface *NetSurface;
+static SDL_Rect NetRect;
+
+static SDL_Surface *CheatSurface;
+static SDL_Rect CheatRect;
+
+static SDL_Surface *HelpSurface;
+static SDL_Rect HelpRect;
+
+static SDL_Surface *SMSurface;
+static SDL_Rect SMRect, SMDRect;
+
 
 static int curbpp;
 static int inited=0;
@@ -104,45 +117,18 @@ static double exs,eys;
 static int exres, eyres;
 static int evideoip;
 
-void ClearVideoSurfaces(void)
+static int NeedClear = 0;
+
+void ClearBackBuffer(void)
 {
- SDL_Surface *scr = screen;
-
- if(!scr) 
-  return;
-
  if(cur_flags & SDL_OPENGL)
  {
-  ClearGLSurfaces(scr);
-  return;
+  ClearBackBufferOpenGL(screen);
  }
-
- int b = 0;
- int y;
-
- if(scr->flags & SDL_DOUBLEBUF)
-  b++;
-
- do
+ else
  {
-  if(SDL_MUSTLOCK(scr))
-   if(SDL_LockSurface(scr))
-    return;
-
-  for(y=0;y<scr->h;y++)
-   memset((uint8 *)scr->pixels + scr->pitch * y, 0, scr->w);
-
-  if(SDL_MUSTLOCK(scr))
-   SDL_UnlockSurface(scr);
-
-  SDL_UpdateRect(scr, 0, 0, scr->w, scr->h);
-
-  PumpWrap();
-
-  if(scr->flags&SDL_DOUBLEBUF)
-   SDL_Flip(scr);
- } while(b--);
-
+  SDL_FillRect(screen, NULL, 0);
+ }
 }
 
 /* Return 1 if video was killed, 0 otherwise(video wasn't initialized). */
@@ -161,11 +147,28 @@ int KillVideo(int subsys)
   source_surface = NULL;
  }
 
- if(OSDSurface)
+ if(DebuggerSurface)
  {
-  OSDSurface->pixels = NULL;
-  SDL_FreeSurface(OSDSurface);
-  OSDSurface = NULL;
+  SDL_FreeSurface(DebuggerSurface);
+  DebuggerSurface = NULL;
+ }
+
+ if(SMSurface)
+ {
+  SDL_FreeSurface(SMSurface);
+  SMSurface = NULL;
+ }
+
+ if(CheatSurface)
+ {
+  SDL_FreeSurface(CheatSurface);
+  CheatSurface = NULL;
+ }
+
+ if(HelpSurface)
+ {
+  SDL_FreeSurface(HelpSurface);
+  HelpSurface = NULL;
  }
 
  if(inited&1)
@@ -299,23 +302,28 @@ int InitVideo(MDFNGI *gi)
  MDFNI_printf(_("Initializing video...\n"));
  MDFN_indent(1);
 
- std::string special_string = MDFN_GetSettingS(std::string(std::string(gi->shortname) + "." + std::string("special")).c_str());
+ std::string sn = std::string(gi->shortname);
+
+ if(gi->GameType == GMT_PLAYER)
+  sn = "player";
+
+ std::string special_string = MDFN_GetSettingS(std::string(sn + "." + std::string("special")).c_str());
 
  _fullscreen = MDFN_GetSettingB("fs");
- _video.xres = MDFN_GetSettingUI(std::string(std::string(gi->shortname) + "." + std::string("xres")).c_str());
- _video.yres = MDFN_GetSettingUI(std::string(std::string(gi->shortname) + "." + std::string("yres")).c_str());
- _video.xscale = MDFN_GetSettingF(std::string(std::string(gi->shortname) + "." + std::string("xscale")).c_str());
- _video.yscale = MDFN_GetSettingF(std::string(std::string(gi->shortname) + "." + std::string("yscale")).c_str());
- _video.xscalefs = MDFN_GetSettingF(std::string(std::string(gi->shortname) + "." + std::string("xscalefs")).c_str());
- _video.yscalefs = MDFN_GetSettingF(std::string(std::string(gi->shortname) + "." + std::string("yscalefs")).c_str());
- _video.videoip = MDFN_GetSettingB(std::string(std::string(gi->shortname) + "." + std::string("videoip")).c_str());
- _video.stretch = MDFN_GetSettingB(std::string(std::string(gi->shortname) + "." + std::string("stretch")).c_str());
- _video.scanlines = MDFN_GetSettingUI(std::string(std::string(gi->shortname) + "." + std::string("scanlines")).c_str());
+ _video.xres = MDFN_GetSettingUI(std::string(sn + "." + std::string("xres")).c_str());
+ _video.yres = MDFN_GetSettingUI(std::string(sn + "." + std::string("yres")).c_str());
+ _video.xscale = MDFN_GetSettingF(std::string(sn + "." + std::string("xscale")).c_str());
+ _video.yscale = MDFN_GetSettingF(std::string(sn + "." + std::string("yscale")).c_str());
+ _video.xscalefs = MDFN_GetSettingF(std::string(sn + "." + std::string("xscalefs")).c_str());
+ _video.yscalefs = MDFN_GetSettingF(std::string(sn + "." + std::string("yscalefs")).c_str());
+ _video.videoip = MDFN_GetSettingB(std::string(sn + "." + std::string("videoip")).c_str());
+ _video.stretch = MDFN_GetSettingB(std::string(sn + "." + std::string("stretch")).c_str());
+ _video.scanlines = MDFN_GetSettingUI(std::string(sn + "." + std::string("scanlines")).c_str());
 
  _video.special = GetSpecialScalerID(special_string);
 
  #ifdef MDFN_WANT_OPENGL_SHADERS
- _video.pixshader = MDFN_GetSettingS(std::string(std::string(gi->shortname) + "." + std::string("pixshader")).c_str());
+ _video.pixshader = MDFN_GetSettingS(std::string(sn + "." + std::string("pixshader")).c_str());
  #else
  _video.pixshader = "none";
  #endif
@@ -342,21 +350,40 @@ int InitVideo(MDFNGI *gi)
 
  unsigned int vdriver = MDFN_GetSettingUI("vdriver");
 
- if(MDFN_GetSettingUI("doublebuf"))
+ if(vdriver == 0)
  {
-  if(vdriver == 0)
-   SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
+  if(!sdlhaveogl)
+  {
+   // SDL_GL_LoadLibrary returns 0 on success, -1 on failure
+   if(SDL_GL_LoadLibrary(NULL) == 0)
+    sdlhaveogl = 1;
+   else
+    sdlhaveogl = 0;
+  }
+
+  if(sdlhaveogl)
+   flags |= SDL_OPENGL;
   else
-   flags |= SDL_DOUBLEBUF;
+  {
+   MDFN_PrintError(_("Could not load OpenGL library, disabling OpenGL usage!"));
+   vdriver = 0;
+  }
  }
 
  if(vdriver == 0)
  {
-  if(sdlhaveogl)
-   flags |= SDL_OPENGL;
-  else
-   MDFN_PrintError(_("Could not load OpenGL library, disabling OpenGL usage!"));
+  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1 );
+
+  if(MDFN_GetSettingB("glvsync"))
+  {
+   #if SDL_VERSION_ATLEAST(1, 2, 10)
+   SDL_GL_SetAttribute(SDL_GL_SWAP_CONTROL, 1);
+   #endif
+  }
  }
+ else
+  flags |= SDL_DOUBLEBUF;
+
  exres = _video.xres;
  eyres = _video.yres;
  exs = _fullscreen ? _video.xscalefs : _video.xscale;
@@ -465,12 +492,19 @@ int InitVideo(MDFNGI *gi)
  else
   SDL_WM_SetCaption("Mednafen","Mednafen");
 
- #ifdef LSB_FIRST
- IconSurface=SDL_CreateRGBSurfaceFrom((void *)mednafen_playicon.pixel_data,32,32,32,32*4,0xFF,0xFF00,0xFF0000,0xFF000000);
+ #ifdef WIN32
+  #ifdef LSB_FIRST
+  IconSurface=SDL_CreateRGBSurfaceFrom((void *)mednafen_playicon.pixel_data,32,32,32,32*4,0xFF,0xFF00,0xFF0000,0xFF000000);
+  #else
+  IconSurface=SDL_CreateRGBSurfaceFrom((void *)mednafen_playicon.pixel_data,32,32,32,32*4,0xFF000000,0xFF0000,0xFF00,0xFF);
+  #endif
  #else
- IconSurface=SDL_CreateRGBSurfaceFrom((void *)mednafen_playicon.pixel_data,32,32,32,32*4,0xFF000000,0xFF0000,0xFF00,0xFF);
+  #ifdef LSB_FIRST
+  IconSurface=SDL_CreateRGBSurfaceFrom((void *)mednafen_playicon128.pixel_data,128,128,32,128*4,0xFF,0xFF00,0xFF0000,0xFF000000);
+  #else
+  IconSurface=SDL_CreateRGBSurfaceFrom((void *)mednafen_playicon128.pixel_data,128,128,32,128*4,0xFF000000,0xFF0000,0xFF00,0xFF);
+  #endif
  #endif
-
  SDL_WM_SetIcon(IconSurface,0);
 
  if(cur_flags & SDL_OPENGL)
@@ -536,20 +570,117 @@ int InitVideo(MDFNGI *gi)
    bs = 16;
    as = 24;
   }
- source_surface = SDL_CreateRGBSurfaceFrom(NULL, (VideoGI->system == GISYS_PCE) ? 512 : 256, 256, 32, gi->pitch, 0xFF << rs, 0xFF << gs, 0xFF << bs, 0xFF << as);
- OSDSurface = SDL_CreateRGBSurfaceFrom(NULL, 512, 224, 32, 512 * sizeof(uint32), 0xFF << rs, 0xFF << gs, 0xFF << bs, 0xFF << as);
- //OSDRect.w = 256;
- OSDRect.h = 224;
- OSDRect.x = 0;
- OSDRect.y = 0;
+ source_surface = SDL_CreateRGBSurfaceFrom(NULL, 256, 256, 32, gi->pitch, 0xFF << rs, 0xFF << gs, 0xFF << bs, 0xFF << as);
+
+ NetSurface = SDL_CreateRGBSurface(SDL_SWSURFACE | SDL_SRCALPHA, screen->w, 18 * 5, 32, 0xFF << real_rs, 0xFF << real_gs, 0xFF << real_bs, 0xFF << real_as);
+ SDL_SetColorKey(NetSurface, SDL_SRCCOLORKEY, 0);
+ SDL_SetAlpha(NetSurface, SDL_SRCALPHA, 0);
+
+ NetRect.w = screen->w;
+ NetRect.h = 18 * 5;
+ NetRect.x = 0;
+ NetRect.y = 0;
+
+
+ {
+  int xmu = 1;
+  int ymu = 1;
+
+  if(screen->w >= 768)
+   xmu = screen->w / 384;
+  if(screen->h >= 576)
+   ymu = screen->h / 288;
+
+  SMRect.h = 18 + 2;
+  SMRect.x = 0;
+  SMRect.y = 0;
+  SMRect.w = screen->w;
+
+  SMDRect.w = SMRect.w * xmu;
+  SMDRect.h = SMRect.h * ymu;
+  SMDRect.x = (screen->w - SMDRect.w) / 2;
+  SMDRect.y = screen->h - SMDRect.h;
+
+  if(SMDRect.x < 0)
+  {
+   SMRect.w += SMDRect.x * 2 / xmu;
+   SMDRect.w = SMRect.w * xmu;
+   SMDRect.x = 0;
+  }
+  SMSurface = SDL_CreateRGBSurface(SDL_SWSURFACE | SDL_SRCALPHA, SMRect.w, SMRect.h, 32, 0xFF << real_rs, 0xFF << real_gs, 0xFF << real_bs, 0xFF << real_as);
+  SDL_SetColorKey(SMSurface, SDL_SRCCOLORKEY, 0);
+ }
 
  MDFNI_SetPixelFormat(rs, gs, bs, as);
 
- ClearVideoSurfaces();
+ for(int i = 0; i < 2; i++)
+ {
+  ClearBackBuffer();
+
+  if(cur_flags & SDL_OPENGL)
+   FlipOpenGL();
+  else
+   SDL_Flip(screen);
+ }
+
  return 1;
 }
 
-static void SubBlit(void)
+static uint32 howlong = 0;
+static UTF8 *CurrentMessage = NULL;
+
+void VideoShowMessage(UTF8 *text)
+{
+ if(text)
+  howlong = MDFND_GetTime() + 2500;
+ else
+  howlong = 0;
+ CurrentMessage = text;
+}
+
+#define MK_COLOR_A(tmp_surface, r,g,b,a) ( ((a)<<tmp_surface->format->Ashift) | ((r)<<tmp_surface->format->Rshift) | ((g) << tmp_surface->format->Gshift) | ((b) << tmp_surface->format->Bshift))
+
+void BlitRaw(SDL_Surface *src, SDL_Rect *src_rect, SDL_Rect *dest_rect)
+{
+ if(cur_flags & SDL_OPENGL)
+  BlitOpenGLRaw(src, src_rect, dest_rect);
+ else 
+ {
+  BlitNonGL(src, src_rect, src_rect, dest_rect, screen, 0, 0);
+
+  //SDL_BlitSurface(src, src_rect, screen, dest_rect);
+ }
+ NeedClear = 2;
+}
+
+
+static bool BlitInternalMessage(void)
+{
+ if(howlong < MDFND_GetTime())
+ {
+  if(CurrentMessage)
+  {
+   free(CurrentMessage);
+   CurrentMessage = NULL;
+  }
+  return(0);
+ }
+
+ if(CurrentMessage)
+ {
+  SDL_FillRect(SMSurface, NULL, MK_COLOR_A(SMSurface, 0x00, 0x00, 0x00, 0xC0));
+  DrawTextTransShadow((uint32 *)((uint8 *)SMSurface->pixels + SMSurface->pitch * 1), SMSurface->pitch, SMRect.w, CurrentMessage, 
+	MK_COLOR_A(SMSurface, 0xFF, 0xFF, 0xFF, 0xFF), MK_COLOR_A(SMSurface, 0x00, 0x00, 0x00, 0xFF), TRUE);
+  free(CurrentMessage);
+  CurrentMessage = NULL;
+ }
+
+ BlitRaw(SMSurface, &SMRect, &SMDRect);
+ return(1);
+}
+
+
+static void SubBlit(bool alpha_blend)
 {
    if(CurrentScaler)
    {
@@ -598,11 +729,9 @@ static void SubBlit(void)
       unsigned int srs = bah_surface->format->Rshift;
       unsigned int sgs = bah_surface->format->Gshift;
       unsigned int sbs = bah_surface->format->Bshift;
-      unsigned int sas = bah_surface->format->Ashift;
       unsigned int drs = real_rs;
       unsigned int dgs = real_gs;
       unsigned int dbs = real_bs;
-      unsigned int das = real_as;
 
       for(int y = 0; y < boohoo_rect.h; y++)
       {
@@ -613,64 +742,41 @@ static void SubBlit(void)
        }
        lineptr += bah_surface->pitch >> 2;
       }
-      // Also fix the OSD buffer if we're in OpenGL mode
-      // SDL will take care of it in nonOpenGL mode
-      // because we use SDL_BlitSurface()
-      lineptr = (uint32 *)OSDSurface->pixels;
-      if((cur_flags & SDL_OPENGL) && OSDSurface->pixels)
-       for(int y = 0; y < 224; y++)
-       {
-        for(int x = 0; x < 256; x++)
-        {
-         uint32 pixel = lineptr[x];
-         lineptr[x] = (((pixel >> srs) & 0xFF) << drs) | (((pixel >> sgs) & 0xFF) << dgs) | (((pixel >> sbs) & 0xFF) << dbs) | (((pixel >> sas) & 0xFF) << das);
-        }
-        lineptr += 256;
-       }
-      }
+     }
     }
+
     if(cur_flags & SDL_OPENGL)
-     BlitOpenGL(bah_surface, &boohoo_rect, &dest_rect, &src_rect);
+     BlitOpenGL(bah_surface, &boohoo_rect, &dest_rect, &src_rect, alpha_blend);
     else
-     BlitNonGL(bah_surface, &boohoo_rect, &src_rect, &dest_rect, screen, _video.scanlines);
+     BlitNonGL(bah_surface, &boohoo_rect, &src_rect, &dest_rect, screen, _video.scanlines, CurGame->rotated);
+
     SDL_FreeSurface(bah_surface);
    }
    else // No special scaler:
    {
     if(cur_flags & SDL_OPENGL)
-     BlitOpenGL(source_surface, &src_rect, &dest_rect, &src_rect);
+     BlitOpenGL(source_surface, &src_rect, &dest_rect, &src_rect, alpha_blend);
     else
-     BlitNonGL(source_surface, &src_rect, &src_rect, &dest_rect, screen, _video.scanlines);
+     BlitNonGL(source_surface, &src_rect, &src_rect, &dest_rect, screen, _video.scanlines, CurGame->rotated);
    }
 }
 
-void BlitScreen(uint32 *XBuf, MDFN_Rect *DisplayRect, MDFN_Rect *LineWidths, uint32 *OSDBuffer)
+void BlitScreen(uint32 *XBuf, MDFN_Rect *DisplayRect, MDFN_Rect *LineWidths)
 {
  if(!screen) return;
 
+ if(NeedClear)
+ {
+  NeedClear--;
+  ClearBackBuffer();
+ }
+
  GenerateRects(*DisplayRect);
  source_surface->pixels = XBuf;
- OSDSurface->pixels = OSDBuffer;
-
- if(OSDBuffer)
- {
-  OSDRect.w = 256;
-  if(Netplay_GetTextView() || IsConsoleCheatConfigActive() || Debugger_IsActive())
-   OSDRect.w = 512;
-
-  FPS_Draw(OSDSurface, &OSDRect);
-
-  if(Debugger_IsActive())
-   Debugger_Draw(OSDSurface, &OSDRect);
-  else if(Netplay_GetTextView())
-   DrawNetplayTextBuffer(OSDSurface, &OSDRect);
-  else if(IsConsoleCheatConfigActive())
-   DrawCheatConsole(OSDSurface, &OSDRect);
- }
 
  if(LineWidths[0].w == ~0) // Skip multi line widths code?
  {
-  SubBlit();
+  SubBlit(0);
  }
  else
  {
@@ -696,7 +802,7 @@ void BlitScreen(uint32 *XBuf, MDFN_Rect *DisplayRect, MDFN_Rect *LineWidths, uin
     dest_rect.h = src_rect.h * dest_rect.h / src_rect_save.h;
 
     // Blit here!
-    SubBlit();
+    SubBlit(0);
 
     src_rect = src_rect_save;
     dest_rect = dest_rect_save;
@@ -707,39 +813,136 @@ void BlitScreen(uint32 *XBuf, MDFN_Rect *DisplayRect, MDFN_Rect *LineWidths, uin
   }
  }
 
- if(OSDBuffer)
+ unsigned int debw, debh;
+
+ if(Debugger_IsActive(&debw, &debh))
  {
-  if(cur_flags & SDL_OPENGL)
+  if(!DebuggerSurface)
   {
-   BlitOpenGLOSD(OSDBuffer, &OSDRect);
+   DebuggerSurface = SDL_CreateRGBSurface(SDL_SWSURFACE | SDL_SRCALPHA, 640, 480, 32, 0xFF << real_rs, 0xFF << real_gs, 0xFF << real_bs, 0xFF << real_as);
   }
-  else
+  DebuggerRect.w = debw;
+  DebuggerRect.h = debh;
+  DebuggerRect.x = 0;
+  DebuggerRect.y = 0;
+
+  SDL_Rect zederect;
+
+  int xm = screen->w / DebuggerRect.w;
+  int ym = screen->h / DebuggerRect.h;
+
+  if(xm < 1) xm = 1;
+  if(ym < 1) ym = 1;
+
+  //if(xm > ym) xm = ym;
+  //if(ym > xm) ym = xm;
+
+  // Allow it to be compacted horizontally, but don't stretch it out, as it's hard(IMHO) to read.
+  if(xm > ym) xm = ym;
+  if(ym > (2 * xm)) ym = 2 * xm;
+
+  zederect.w = DebuggerRect.w * xm;
+  zederect.h = DebuggerRect.h * ym;
+
+  zederect.x = (screen->w - zederect.w) / 2;
+  zederect.y = (screen->h - zederect.h) / 2;
+
+  Debugger_Draw(DebuggerSurface, &DebuggerRect, &zederect);
+
+  BlitRaw(DebuggerSurface, &DebuggerRect, &zederect);
+ }
+
+ if(Help_IsActive())
+ {
+  if(!HelpSurface)
+  {
+   HelpRect.w = screen->w;
+   HelpRect.h = screen->h;
+
+   if(HelpRect.w >= 512)
+   {
+    HelpRect.w /= (HelpRect.w / 512);
+    if(HelpRect.w > 512)
+     HelpRect.w = 512;
+   }
+
+   if(HelpRect.h >= 384)
+   {
+    HelpRect.h /= (HelpRect.h / 384);
+    if(HelpRect.h > 384)
+     HelpRect.h = 384;
+   }
+
+   HelpSurface = SDL_CreateRGBSurface(SDL_SWSURFACE | SDL_SRCALPHA, HelpRect.w, HelpRect.h, 32, 0xFF << real_rs, 0xFF << real_gs, 0xFF << real_bs, 0xFF << real_as);
+   SDL_SetColorKey(HelpSurface, SDL_SRCCOLORKEY, 0);
+   SDL_SetAlpha(HelpSurface, SDL_SRCALPHA, 0);
+   Help_Draw(HelpSurface, &HelpRect);
+  }
+  SDL_Rect zederect;
+
+  zederect.w = HelpRect.w * (screen->w / HelpRect.w);
+  zederect.h = HelpRect.h * (screen->h / HelpRect.h);
+
+  zederect.x = (screen->w - zederect.w) / 2;
+  zederect.y = (screen->h - zederect.h) / 2;
+  BlitRaw(HelpSurface, &HelpRect, &zederect);
+ }
+ else if(HelpSurface)
+ {
+  SDL_FreeSurface(HelpSurface);
+  HelpSurface = NULL;
+ }
+
+ DrawSaveStates(screen, exs, eys, real_rs, real_gs, real_bs, real_as);
+
+ if(IsConsoleCheatConfigActive())
+ {
+  if(!CheatSurface)
+  {
+   CheatRect.w = screen->w;
+   CheatRect.h = screen->h;
+
+   CheatSurface = SDL_CreateRGBSurface(SDL_SWSURFACE | SDL_SRCALPHA, CheatRect.w, CheatRect.h, 32, 0xFF << real_rs, 0xFF << real_gs, 0xFF << real_bs, 0xFF << real_as);
+   SDL_SetColorKey(CheatSurface, SDL_SRCCOLORKEY, 0);
+   SDL_SetAlpha(CheatSurface, SDL_SRCALPHA, 0);
+  }
+  SDL_Rect zederect = CheatRect;
+  DrawCheatConsole(CheatSurface, &CheatRect);
+  BlitRaw(CheatSurface, &CheatRect, &zederect);
+ }
+ else if(CheatSurface)
+ {
+  SDL_FreeSurface(CheatSurface);
+  CheatSurface = NULL;
+ }
+
+ if(Netplay_GetTextView())
+ {
+  if(SDL_MUSTLOCK(NetSurface))
+   SDL_LockSurface(NetSurface);
+
+  DrawNetplayTextBuffer(NetSurface, &NetRect);
+
   {
    SDL_Rect zederect;
 
    zederect.x = 0;
-   zederect.y = 0;
-   zederect.w = OSDRect.w;
-   zederect.h = OSDRect.h;
+   zederect.y = screen->h - NetRect.h;
+   zederect.w = NetRect.w;
+   zederect.h = NetRect.h;
 
-   if(zederect.w > screen->w) zederect.w = screen->w;
-   if(zederect.h > screen->h) zederect.h = screen->h;
-
-   zederect.x = (screen->w - zederect.w) / 2;
-   zederect.y = (screen->h - zederect.h) / 2;
-
-   SDL_SetColorKey(OSDSurface, SDL_SRCCOLORKEY, 0);
-   SDL_SetAlpha(OSDSurface, SDL_SRCALPHA, 0); //128);
-   SDL_BlitSurface(OSDSurface, &OSDRect, screen, &zederect);
+   BlitRaw(NetSurface, &NetRect, &zederect);
   }
+  if(SDL_MUSTLOCK(NetSurface))
+   SDL_UnlockSurface(NetSurface);
  }
+
+ BlitInternalMessage();
+
+ FPS_Draw(screen, real_rs, real_gs, real_bs, real_as);
+
  if(!(cur_flags & SDL_OPENGL))
- {
-  if(cur_flags & SDL_DOUBLEBUF)
-   SDL_Flip(screen);
-  else
-   SDL_UpdateRect(screen, dest_rect.x, dest_rect.y, dest_rect.w, dest_rect.h);
- }
+  SDL_Flip(screen);
  else
   FlipOpenGL();
 }
