@@ -4,11 +4,10 @@ typedef struct {
 	uint8 *data;
 	int64 size;
 	int64 location;
-	void *mmap;
 	char *ext;
 } MDFNFILE;
 
-MDFNFILE *MDFN_fopen(const char *path, const char *ipsfn, char *mode, char *ext);
+MDFNFILE *MDFN_fopen(const char *path, const char *ipsfn, const char *mode, const char *ext);
 int MDFN_fclose(MDFNFILE*);
 uint64 MDFN_fread(void *ptr, size_t size, size_t nmemb, MDFNFILE*);
 uint64 MDFN_fwrite(void *ptr, size_t size, size_t nmemb, MDFNFILE*);
