@@ -34,13 +34,12 @@
 #include "GBAinline.h"
 #include "Globals.h"
 #include "Gfx.h"
-#include "EEprom.h"
-#include "Flash.h"
+#include "eeprom.h"
+#include "flash.h"
 #include "Sound.h"
-#include "Sram.h"
+#include "sram.h"
 #include "bios.h"
 #include "elf.h"
-#include "Util.h"
 #include "Port.h"
 
 #include "arm.h"
@@ -71,11 +70,12 @@ int dummyAddress = 0;
 bool8 cpuBreakLoop = false;
 int cpuNextEvent = 0;
 
-int gbaSaveType = 0; // used to remember the save type on reset
-bool8 intState = false;
-bool8 stopState = false;
-bool8 holdState = false;
-int holdType = 0;
+static int gbaSaveType = 0; // used to remember the save type on reset
+static bool8 intState = false;
+static bool8 stopState = false;
+static bool8 holdState = false;
+static int holdType = 0;
+
 bool8 cpuSramEnabled = true;
 bool8 cpuFlashEnabled = true;
 bool8 cpuEEPROMEnabled = true;
@@ -84,46 +84,20 @@ bool8 cpuEEPROMSensorEnabled = false;
 uint32 cpuPrefetch[2];
 
 int cpuTotalTicks = 0;
-
-uint8 freezeWorkRAM[0x40000];
-uint8 freezeInternalRAM[0x8000];
-int lcdTicks = (useBios && !skipBios) ? 1008 : 208;
+static int lcdTicks;
 uint8 timerOnOffDelay = 0;
-uint16 timer0Value = 0;
-bool8 timer0On = false;
-int timer0Ticks = 0;
-int timer0Reload = 0;
-int timer0ClockReload  = 0;
-uint16 timer1Value = 0;
-bool8 timer1On = false;
-int timer1Ticks = 0;
-int timer1Reload = 0;
-int timer1ClockReload  = 0;
-uint16 timer2Value = 0;
-bool8 timer2On = false;
-int timer2Ticks = 0;
-int timer2Reload = 0;
-int timer2ClockReload  = 0;
-uint16 timer3Value = 0;
-bool8 timer3On = false;
-int timer3Ticks = 0;
-int timer3Reload = 0;
-int timer3ClockReload  = 0;
+
+GBATimer timers[4];
+
 uint32 dmaSource[4] = {0};
 uint32 dmaDest[4] = {0};
 void (*cpuSaveGameFunc)(uint32,uint8) = flashSaveDecide;
 void (*renderLine)() = mode0RenderLine;
 bool8 fxOn = false;
 bool8 windowOn = false;
-char buffer[1024];
-FILE *out = NULL;
-int count = 0;
 
-int capture = 0;
-int capturePrevious = 0;
-int captureNumber = 0;
-
-const int TIMER_TICKS[4] = {
+static const int TIMER_TICKS[4] = 
+{
   0,
   6,
   8,
@@ -163,199 +137,30 @@ uint8 biosProtected[4];
 bool8 cpuBiosSwapped = false;
 #endif
 
-uint32 myROM[] = {
-0xEA000006,
-0xEA000093,
-0xEA000006,
-0x00000000,
-0x00000000,
-0x00000000,
-0xEA000088,
-0x00000000,
-0xE3A00302,
-0xE1A0F000,
-0xE92D5800,
-0xE55EC002,
-0xE28FB03C,
-0xE79BC10C,
-0xE14FB000,
-0xE92D0800,
-0xE20BB080,
-0xE38BB01F,
-0xE129F00B,
-0xE92D4004,
-0xE1A0E00F,
-0xE12FFF1C,
-0xE8BD4004,
-0xE3A0C0D3,
-0xE129F00C,
-0xE8BD0800,
-0xE169F00B,
-0xE8BD5800,
-0xE1B0F00E,
-0x0000009C,
-0x0000009C,
-0x0000009C,
-0x0000009C,
-0x000001F8,
-0x000001F0,
-0x000000AC,
-0x000000A0,
-0x000000FC,
-0x00000168,
-0xE12FFF1E,
-0xE1A03000,
-0xE1A00001,
-0xE1A01003,
-0xE2113102,
-0x42611000,
-0xE033C040,
-0x22600000,
-0xE1B02001,
-0xE15200A0,
-0x91A02082,
-0x3AFFFFFC,
-0xE1500002,
-0xE0A33003,
-0x20400002,
-0xE1320001,
-0x11A020A2,
-0x1AFFFFF9,
-0xE1A01000,
-0xE1A00003,
-0xE1B0C08C,
-0x22600000,
-0x42611000,
-0xE12FFF1E,
-0xE92D0010,
-0xE1A0C000,
-0xE3A01001,
-0xE1500001,
-0x81A000A0,
-0x81A01081,
-0x8AFFFFFB,
-0xE1A0000C,
-0xE1A04001,
-0xE3A03000,
-0xE1A02001,
-0xE15200A0,
-0x91A02082,
-0x3AFFFFFC,
-0xE1500002,
-0xE0A33003,
-0x20400002,
-0xE1320001,
-0x11A020A2,
-0x1AFFFFF9,
-0xE0811003,
-0xE1B010A1,
-0xE1510004,
-0x3AFFFFEE,
-0xE1A00004,
-0xE8BD0010,
-0xE12FFF1E,
-0xE0010090,
-0xE1A01741,
-0xE2611000,
-0xE3A030A9,
-0xE0030391,
-0xE1A03743,
-0xE2833E39,
-0xE0030391,
-0xE1A03743,
-0xE2833C09,
-0xE283301C,
-0xE0030391,
-0xE1A03743,
-0xE2833C0F,
-0xE28330B6,
-0xE0030391,
-0xE1A03743,
-0xE2833C16,
-0xE28330AA,
-0xE0030391,
-0xE1A03743,
-0xE2833A02,
-0xE2833081,
-0xE0030391,
-0xE1A03743,
-0xE2833C36,
-0xE2833051,
-0xE0030391,
-0xE1A03743,
-0xE2833CA2,
-0xE28330F9,
-0xE0000093,
-0xE1A00840,
-0xE12FFF1E,
-0xE3A00001,
-0xE3A01001,
-0xE92D4010,
-0xE3A03000,
-0xE3A04001,
-0xE3500000,
-0x1B000004,
-0xE5CC3301,
-0xEB000002,
-0x0AFFFFFC,
-0xE8BD4010,
-0xE12FFF1E,
-0xE3A0C301,
-0xE5CC3208,
-0xE15C20B8,
-0xE0110002,
-0x10222000,
-0x114C20B8,
-0xE5CC4208,
-0xE12FFF1E,
-0xE92D500F,
-0xE3A00301,
-0xE1A0E00F,
-0xE510F004,
-0xE8BD500F,
-0xE25EF004,
-0xE59FD044,
-0xE92D5000,
-0xE14FC000,
-0xE10FE000,
-0xE92D5000,
-0xE3A0C302,
-0xE5DCE09C,
-0xE35E00A5,
-0x1A000004,
-0x05DCE0B4,
-0x021EE080,
-0xE28FE004,
-0x159FF018,
-0x059FF018,
-0xE59FD018,
-0xE8BD5000,
-0xE169F00C,
-0xE8BD5000,
-0xE25EF004,
-0x03007FF0,
-0x09FE2000,
-0x09FFC000,
-0x03007FE0
+static uint32 myROM[] = 
+{
+ #include "myrom.h"
 };
 
+static uint32 *systemColorMap32 = NULL; // 65536
+static uint8 *CustomColorMap = NULL; // 32768 * 3
 static int romSize = 0x2000000;
 
-inline int CPUUpdateTicks()
+static INLINE int CPUUpdateTicks()
 {
   int cpuLoopTicks = lcdTicks;
   
-  if(timer0On && (timer0Ticks < cpuLoopTicks)) {
-    cpuLoopTicks = timer0Ticks;
+  if(timers[0].On && (timers[0].Ticks < cpuLoopTicks)) {
+    cpuLoopTicks = timers[0].Ticks;
   }
-  if(timer1On && !(TM1CNT & 4) && (timer1Ticks < cpuLoopTicks)) {
-    cpuLoopTicks = timer1Ticks;
+  if(timers[1].On && !(timers[1].CNT & 4) && (timers[1].Ticks < cpuLoopTicks)) {
+    cpuLoopTicks = timers[1].Ticks;
   }
-  if(timer2On && !(TM2CNT & 4) && (timer2Ticks < cpuLoopTicks)) {
-    cpuLoopTicks = timer2Ticks;
+  if(timers[2].On && !(timers[2].CNT & 4) && (timers[2].Ticks < cpuLoopTicks)) {
+    cpuLoopTicks = timers[2].Ticks;
   }
-  if(timer3On && !(TM3CNT & 4) && (timer3Ticks < cpuLoopTicks)) {
-    cpuLoopTicks = timer3Ticks;
+  if(timers[3].On && !(timers[3].CNT & 4) && (timers[3].Ticks < cpuLoopTicks)) {
+    cpuLoopTicks = timers[3].Ticks;
   }
 
   if (SWITicks) {
@@ -460,7 +265,6 @@ static int StateAction(StateMem *sm, int load, int data_only)
   SFVAR(busPrefetch),
   SFVAR(busPrefetchEnable),
   SFVAR(busPrefetchCount),
-  SFVAR(cpuDmaTicksToUpdate),
   SFVAR(cpuDmaHack),
   SFVAR(cpuDmaLast),
   SFVAR(cpuDmaTicksToUpdate),
@@ -474,15 +278,11 @@ static int StateAction(StateMem *sm, int load, int data_only)
   SFVAR(BG1CNT),
   SFVAR(BG2CNT),
   SFVAR(BG3CNT),
-  SFVAR(BG0HOFS),
 
-  SFVAR(BG0VOFS),
-  SFVAR(BG1HOFS),
-  SFVAR(BG1VOFS),
-  SFVAR(BG2HOFS),
-  SFVAR(BG2VOFS),
-  SFVAR(BG3HOFS),
-  SFVAR(BG3VOFS),
+
+  SFARRAY16(BGHOFS, 4),
+  SFARRAY16(BGVOFS, 4),
+
   SFVAR(BG2PA),
   SFVAR(BG2PB),
   SFVAR(BG2PC),
@@ -518,14 +318,14 @@ static int StateAction(StateMem *sm, int load, int data_only)
   SFARRAY32(DMCNT_L, 4),
   SFARRAY32(DMCNT_H, 4),
 
-  SFVAR(TM0D),
-  SFVAR(TM0CNT),
-  SFVAR(TM1D),
-  SFVAR(TM1CNT),
-  SFVAR(TM2D),
-  SFVAR(TM2CNT),
-  SFVAR(TM3D),
-  SFVAR(TM3CNT),
+  SFVAR(timers[0].D),
+  SFVAR(timers[0].CNT),
+  SFVAR(timers[1].D),
+  SFVAR(timers[1].CNT),
+  SFVAR(timers[2].D),
+  SFVAR(timers[2].CNT),
+  SFVAR(timers[3].D),
+  SFVAR(timers[3].CNT),
 
   SFVAR(P1),
   SFVAR(IE),
@@ -536,22 +336,25 @@ static int StateAction(StateMem *sm, int load, int data_only)
   SFVAR(holdType),
   SFVAR(lcdTicks),
 
-  SFVAR(timer0On),
-  SFVAR(timer0Ticks),
-  SFVAR(timer0Reload),
-  SFVAR(timer0ClockReload),
-  SFVAR(timer1On),
-  SFVAR(timer1Ticks),
-  SFVAR(timer1Reload),
-  SFVAR(timer1ClockReload),
-  SFVAR(timer2On),
-  SFVAR(timer2Ticks),
-  SFVAR(timer2Reload),
-  SFVAR(timer2ClockReload),
-  SFVAR(timer3On),
-  SFVAR(timer3Ticks),
-  SFVAR(timer3Reload),
-  SFVAR(timer3ClockReload),
+  SFVAR(timers[0].On),
+  SFVAR(timers[0].Ticks),
+  SFVAR(timers[0].Reload),
+  SFVAR(timers[0].ClockReload),
+
+  SFVAR(timers[1].On),
+  SFVAR(timers[1].Ticks),
+  SFVAR(timers[1].Reload),
+  SFVAR(timers[1].ClockReload),
+
+  SFVAR(timers[2].On),
+  SFVAR(timers[2].Ticks),
+  SFVAR(timers[2].Reload),
+  SFVAR(timers[2].ClockReload),
+
+  SFVAR(timers[3].On),
+  SFVAR(timers[3].Ticks),
+  SFVAR(timers[3].Reload),
+  SFVAR(timers[3].ClockReload),
 
   SFARRAY32(dmaSource, 4),
   SFARRAY32(dmaDest, 4),
@@ -618,8 +421,10 @@ static int StateAction(StateMem *sm, int load, int data_only)
   CPUUpdateWindow1();
 
   gbaSaveType = 0;
-        saveType = 0;
-  switch(saveType) {
+  saveType = 0;
+
+  switch(saveType) 
+  {
   case 0:
     cpuSaveGameFunc = flashSaveDecide;
     break;
@@ -641,6 +446,7 @@ static int StateAction(StateMem *sm, int load, int data_only)
 
   if(eepromInUse)
     gbaSaveType = 3;
+
   if(armState) {
     ARM_PREFETCH;
   } else {
@@ -651,31 +457,6 @@ static int StateAction(StateMem *sm, int load, int data_only)
  return(1);
 }
 
-
-bool8 CPUExportEepromFile(const char *fileName)
-{
-  if(eepromInUse) {
-    FILE *file = fopen(fileName, "wb");
-    
-    if(!file) {
-      MDFN_PrintError(_("Error creating file %s"), fileName);
-      return false;
-    }
-
-    for(int i = 0; i < eepromSize;) {
-      for(int j = 0; j < 8; j++) {
-        if(fwrite(&eepromData[i+7-j], 1, 1, file) != 1) {
-          fclose(file);
-          return false;
-        }
-      }
-      i += 8;
-    }
-    fclose(file);
-  }
-  return true;
-}
-
 bool8 CPUWriteBatteryFile(const char *fileName)
 {
   if(gbaSaveType == 0) {
@@ -723,169 +504,6 @@ bool8 CPUWriteBatteryFile(const char *fileName)
   return true;
 }
 
-bool8 CPUReadGSASnapshot(const char *fileName)
-{
-  int i;
-  FILE *file = fopen(fileName, "rb");
-    
-  if(!file) {
-    MDFN_PrintError(_("Cannot open file %s"), fileName);
-    return false;
-  }
-  
-  // check file size to know what we should read
-  fseek(file, 0, SEEK_END);
-
-  // long size = ftell(file);
-  fseek(file, 0x0, SEEK_SET);
-  fread(&i, 1, 4, file);
-  fseek(file, i, SEEK_CUR); // Skip SharkPortSave
-  fseek(file, 4, SEEK_CUR); // skip some sort of flag
-  fread(&i, 1, 4, file); // name length
-  fseek(file, i, SEEK_CUR); // skip name
-  fread(&i, 1, 4, file); // desc length
-  fseek(file, i, SEEK_CUR); // skip desc
-  fread(&i, 1, 4, file); // notes length
-  fseek(file, i, SEEK_CUR); // skip notes
-  int saveSize;
-  fread(&saveSize, 1, 4, file); // read length
-  saveSize -= 0x1c; // remove header size
-  char buffer[17];
-  char buffer2[17];
-  fread(buffer, 1, 16, file);
-  buffer[16] = 0;
-  for(i = 0; i < 16; i++)
-    if(buffer[i] < 32)
-      buffer[i] = 32;
-  memcpy(buffer2, &rom[0xa0], 16);
-  buffer2[16] = 0;
-  for(i = 0; i < 16; i++)
-    if(buffer2[i] < 32)
-      buffer2[i] = 32;  
-  if(memcmp(buffer, buffer2, 16)) {
-    MDFN_PrintError(_("Cannot import snapshot for %s. Current game is %s"),
-                  buffer,
-                  buffer2);
-    fclose(file);
-    return false;
-  }
-  fseek(file, 12, SEEK_CUR); // skip some flags
-  if(saveSize >= 65536) {
-    if(fread(flashSaveMemory, 1, saveSize, file) != (size_t)saveSize) {
-      fclose(file);
-      return false;
-    }
-  } else {
-    MDFN_PrintError(_("Unsupported snapshot file %s"), fileName);
-    fclose(file);
-    return false;
-  }
-  fclose(file);
-  CPUReset();
-  return true;
-}
-
-bool8 CPUWriteGSASnapshot(const char *fileName, 
-                         const char *title, 
-                         const char *desc, 
-                         const char *notes)
-{
-  FILE *file = fopen(fileName, "wb");
-    
-  if(!file) {
-    MDFN_PrintError(_("Cannot open file %s"), fileName);
-    return false;
-  }
-
-  uint8 buffer[17];
-
-  utilPutDword(buffer, 0x0d); // SharkPortSave length
-  fwrite(buffer, 1, 4, file);
-  fwrite("SharkPortSave", 1, 0x0d, file);
-  utilPutDword(buffer, 0x000f0000);
-  fwrite(buffer, 1, 4, file); // save type 0x000f0000 = GBA save
-  utilPutDword(buffer, strlen(title));
-  fwrite(buffer, 1, 4, file); // title length
-  fwrite(title, 1, strlen(title), file);
-  utilPutDword(buffer, strlen(desc));
-  fwrite(buffer, 1, 4, file); // desc length
-  fwrite(desc, 1, strlen(desc), file);
-  utilPutDword(buffer, strlen(notes));
-  fwrite(buffer, 1, 4, file); // notes length
-  fwrite(notes, 1, strlen(notes), file);
-  int saveSize = 0x10000;
-  if(gbaSaveType == 2)
-    saveSize = flashSize;
-  int totalSize = saveSize + 0x1c;
-
-  utilPutDword(buffer, totalSize); // length of remainder of save - CRC
-  fwrite(buffer, 1, 4, file);
-
-  char temp[0x2001c];
-  memset(temp, 0, 28);
-  memcpy(temp, &rom[0xa0], 16); // copy internal name
-  temp[0x10] = rom[0xbe]; // reserved area (old checksum)
-  temp[0x11] = rom[0xbf]; // reserved area (old checksum)
-  temp[0x12] = rom[0xbd]; // complement check
-  temp[0x13] = rom[0xb0]; // maker
-  temp[0x14] = 1; // 1 save ?
-  memcpy(&temp[0x1c], flashSaveMemory, saveSize); // copy save
-  fwrite(temp, 1, totalSize, file); // write save + header
-  uint32 crc = 0;
-  
-  for(int i = 0; i < totalSize; i++) {
-    crc += ((uint32)temp[i] << (crc % 0x18));
-  }
-  
-  utilPutDword(buffer, crc);
-  fwrite(buffer, 1, 4, file); // CRC?
-  
-  fclose(file);
-  return true;
-}
-
-bool8 CPUImportEepromFile(const char *fileName)
-{
-  FILE *file = fopen(fileName, "rb");
-    
-  if(!file)
-    return false;
-  
-  // check file size to know what we should read
-  fseek(file, 0, SEEK_END);
-
-  long size = ftell(file);
-  fseek(file, 0, SEEK_SET);
-  if(size == 512 || size == 0x2000) {
-    if(fread(eepromData, 1, size, file) != (size_t)size) {
-      fclose(file);
-      return false;
-    }
-    for(int i = 0; i < size;) {
-      uint8 tmp = eepromData[i];
-      eepromData[i] = eepromData[7-i];
-      eepromData[7-i] = tmp;
-      i++;
-      tmp = eepromData[i];
-      eepromData[i] = eepromData[7-i];
-      eepromData[7-i] = tmp;
-      i++;
-      tmp = eepromData[i];
-      eepromData[i] = eepromData[7-i];
-      eepromData[7-i] = tmp;
-      i++;      
-      tmp = eepromData[i];
-      eepromData[i] = eepromData[7-i];
-      eepromData[7-i] = tmp;
-      i++;      
-      i += 4;
-    }
-  } else
-    return false;
-  fclose(file);
-  return true;
-}
-
 bool8 CPUReadBatteryFile(const char *fileName)
 {
   FILE *file = fopen(fileName, "rb");
@@ -1003,6 +621,12 @@ void CPUCleanUp()
     ioMem = NULL;
   }
   
+  if(systemColorMap32)
+  {
+   free(systemColorMap32);
+   systemColorMap32 = NULL;
+  }
+
   elfCleanUp();
 }
 
@@ -1054,6 +678,31 @@ static void gsf_load_func(void *data, uint32 len)
  //printf("%d %08x %08x %08x\n", len, entry_point, gsf_offset, size);
 }
 
+static void RedoColorMap(int rshift, int gshift, int bshift)
+{
+ int x;
+
+ if(CustomColorMap)
+ {
+  for(x = 0; x < 65536; x++)
+  {
+   int r = CustomColorMap[(x & 0x7FFF) * 3 + 0];
+   int g = CustomColorMap[(x & 0x7FFF) * 3 + 1];
+   int b = CustomColorMap[(x & 0x7FFF) * 3 + 2];
+   systemColorMap32[x] = (r << rshift) | (g << gshift) | (b << bshift);
+  }
+ }
+ else
+ {
+  rshift += 3;
+  gshift += 3;
+  bshift += 3;
+
+  for(x=0;x<65536;x++)
+   systemColorMap32[x] = ((x & 0x1f) << rshift) | (((x & 0x3e0)>>5) << gshift) | (((x & 0x7c00)>>10) << bshift);
+ }
+}
+
 static int Load(const char *name, MDFNFILE *fp)
 {
   int size = 0x2000000;
@@ -1072,7 +721,7 @@ static int Load(const char *name, MDFNFILE *fp)
    }
    static UTF8 *sn;
    sn = (UTF8*)pi->title;
-   MDFNMP_Init(1, (UTF8*)pi->game, (UTF8*)pi->artist, (UTF8*)pi->copyright, &sn);
+   Player_Init(1, (UTF8*)pi->game, (UTF8*)pi->artist, (UTF8*)pi->copyright, &sn);
    //saveType = 2;
   }
   else
@@ -1154,10 +803,15 @@ static int Load(const char *name, MDFNFILE *fp)
     CPUCleanUp();
     return 0;
   }
+  if(!(systemColorMap32 = (uint32*)MDFN_malloc(65536 * sizeof(uint32), _("GBA Color Map"))))
+  {
+   CPUCleanUp();
+   return(0);
+  }
 
   CPUUpdateRenderBuffers(true);
 
-  MDFNGameInfo->pitch = 256 * sizeof(uint32);
+  MDFNGameInfo->GameSetMD5Valid = FALSE;
 
   if(MDFN_GetSettingB("gba.forcemono"))
   {
@@ -1182,12 +836,35 @@ static int Load(const char *name, MDFNFILE *fp)
   CPUInit(NULL, 0);
   CPUReset();
 
-  flashInit();
+  GBA_Flash_Init();
   eepromInit();
 
   if(!pi)
    CPUReadBatteryFile(MDFN_MakeFName(MDFNMKF_SAV, 0, "sav").c_str());
 
+  std::string colormap_fn = MDFN_GetSettingS("gba.colormap");
+
+  if(colormap_fn != "" && colormap_fn != "0" && colormap_fn != "none")
+  {
+   MDFN_printf(_("Loading custom color map from \"%s\"...\n"),  colormap_fn.c_str());
+   gzFile gp = gzopen(colormap_fn.c_str(), "rb");
+   if(!fp)
+    MDFN_printf("Error\n");
+   else if((CustomColorMap = (uint8*)MDFN_malloc(32768 * 3, _("custom color map"))))
+   {
+    if(gzread(gp, CustomColorMap, 32768 * 3) != 32768 * 3)
+    {
+     MDFN_printf("Error\n");
+     free(CustomColorMap);
+     CustomColorMap = NULL;
+    }
+    else
+    {
+ 
+    }
+   }
+  }
+
   return size;
 }
 
@@ -1982,7 +1659,8 @@ void CPUCheckDMA(int reason, int dmamask)
 
 void CPUUpdateRegister(uint32 address, uint16 value)
 {
-  switch(address) {
+  switch(address) 
+  {
   case 0x00:
     {
       if ((value & 7) >5)
@@ -2044,36 +1722,36 @@ void CPUUpdateRegister(uint32 address, uint16 value)
     UPDATE_REG(0x0E, BG3CNT);
     break;
   case 0x10:
-    BG0HOFS = value & 511;
-    UPDATE_REG(0x10, BG0HOFS);
+    BGHOFS[0] = value & 511;
+    UPDATE_REG(0x10, BGHOFS[0]);
     break;
   case 0x12:
-    BG0VOFS = value & 511;
-    UPDATE_REG(0x12, BG0VOFS);
+    BGVOFS[0] = value & 511;
+    UPDATE_REG(0x12, BGVOFS[0]);
     break;
   case 0x14:
-    BG1HOFS = value & 511;
-    UPDATE_REG(0x14, BG1HOFS);
+    BGHOFS[1] = value & 511;
+    UPDATE_REG(0x14, BGHOFS[1]);
     break;
   case 0x16:
-    BG1VOFS = value & 511;
-    UPDATE_REG(0x16, BG1VOFS);
+    BGVOFS[1] = value & 511;
+    UPDATE_REG(0x16, BGVOFS[1]);
     break;      
   case 0x18:
-    BG2HOFS = value & 511;
-    UPDATE_REG(0x18, BG2HOFS);
+    BGHOFS[2] = value & 511;
+    UPDATE_REG(0x18, BGHOFS[2]);
     break;
   case 0x1A:
-    BG2VOFS = value & 511;
-    UPDATE_REG(0x1A, BG2VOFS);
+    BGVOFS[2] = value & 511;
+    UPDATE_REG(0x1A, BGVOFS[2]);
     break;
   case 0x1C:
-    BG3HOFS = value & 511;
-    UPDATE_REG(0x1C, BG3HOFS);
+    BGHOFS[3] = value & 511;
+    UPDATE_REG(0x1C, BGHOFS[3]);
     break;
   case 0x1E:
-    BG3VOFS = value & 511;
-    UPDATE_REG(0x1E, BG3VOFS);
+    BGVOFS[3] = value & 511;
+    UPDATE_REG(0x1E, BGVOFS[3]);
     break;      
   case 0x20:
     BG2PA = value;
@@ -2366,34 +2044,34 @@ void CPUUpdateRegister(uint32 address, uint16 value)
     }
     break;
  case 0x100:
-    timer0Reload = value;
+    timers[0].Reload = value;
     break;
   case 0x102:
-    timer0Value = value;
+    timers[0].Value = value;
     timerOnOffDelay|=1;
     cpuNextEvent = cpuTotalTicks;
     break;
   case 0x104:
-    timer1Reload = value;
+    timers[1].Reload = value;
     break;
   case 0x106:
-    timer1Value = value;
+    timers[1].Value = value;
     timerOnOffDelay|=2;
     cpuNextEvent = cpuTotalTicks;
     break;
   case 0x108:
-    timer2Reload = value;
+    timers[2].Reload = value;
     break;
   case 0x10A:
-    timer2Value = value;
+    timers[2].Value = value;
     timerOnOffDelay|=4;
     cpuNextEvent = cpuTotalTicks;
     break;
   case 0x10C:
-    timer3Reload = value;
+    timers[3].Reload = value;
     break;
   case 0x10E:
-    timer3Value = value;
+    timers[3].Value = value;
     timerOnOffDelay|=8;
     cpuNextEvent = cpuTotalTicks;
     break;
@@ -2430,28 +2108,17 @@ void CPUUpdateRegister(uint32 address, uint16 value)
     {
       memoryWait[0x0e] = memoryWaitSeq[0x0e] = gamepakRamWaitState[value & 3];
       
-      if(!speedHack) {
-        memoryWait[0x08] = memoryWait[0x09] = gamepakWaitState[(value >> 2) & 3];
-        memoryWaitSeq[0x08] = memoryWaitSeq[0x09] =
-          gamepakWaitState0[(value >> 4) & 1];
+      memoryWait[0x08] = memoryWait[0x09] = gamepakWaitState[(value >> 2) & 3];
+      memoryWaitSeq[0x08] = memoryWaitSeq[0x09] =
+        gamepakWaitState0[(value >> 4) & 1];
         
-        memoryWait[0x0a] = memoryWait[0x0b] = gamepakWaitState[(value >> 5) & 3];
-        memoryWaitSeq[0x0a] = memoryWaitSeq[0x0b] =
-          gamepakWaitState1[(value >> 7) & 1];
+      memoryWait[0x0a] = memoryWait[0x0b] = gamepakWaitState[(value >> 5) & 3];
+      memoryWaitSeq[0x0a] = memoryWaitSeq[0x0b] =
+        gamepakWaitState1[(value >> 7) & 1];
         
-        memoryWait[0x0c] = memoryWait[0x0d] = gamepakWaitState[(value >> 8) & 3];
-        memoryWaitSeq[0x0c] = memoryWaitSeq[0x0d] =
-          gamepakWaitState2[(value >> 10) & 1];
-      } else {
-        memoryWait[0x08] = memoryWait[0x09] = 3;
-        memoryWaitSeq[0x08] = memoryWaitSeq[0x09] = 1;
-        
-        memoryWait[0x0a] = memoryWait[0x0b] = 3;
-        memoryWaitSeq[0x0a] = memoryWaitSeq[0x0b] = 1;
-        
-        memoryWait[0x0c] = memoryWait[0x0d] = 3;
-        memoryWaitSeq[0x0c] = memoryWaitSeq[0x0d] = 1;
-      }
+      memoryWait[0x0c] = memoryWait[0x0d] = gamepakWaitState[(value >> 8) & 3];
+      memoryWaitSeq[0x0c] = memoryWaitSeq[0x0d] =
+        gamepakWaitState2[(value >> 10) & 1];
          
       for(int i = 8; i < 15; i++) {
         memoryWait32[i] = memoryWait[i] + memoryWaitSeq[i] + 1;
@@ -2492,56 +2159,56 @@ void applyTimer ()
 {
   if (timerOnOffDelay & 1)
   {
-    timer0ClockReload = TIMER_TICKS[timer0Value & 3];        
-    if(!timer0On && (timer0Value & 0x80)) {
+    timers[0].ClockReload = TIMER_TICKS[timers[0].Value & 3];        
+    if(!timers[0].On && (timers[0].Value & 0x80)) {
       // reload the counter
-      TM0D = timer0Reload;      
-      timer0Ticks = (0x10000 - TM0D) << timer0ClockReload;
-      UPDATE_REG(0x100, TM0D);
+      timers[0].D = timers[0].Reload;      
+      timers[0].Ticks = (0x10000 - timers[0].D) << timers[0].ClockReload;
+      UPDATE_REG(0x100, timers[0].D);
     }
-    timer0On = timer0Value & 0x80 ? true : false;
-    TM0CNT = timer0Value & 0xC7;
-    UPDATE_REG(0x102, TM0CNT);
+    timers[0].On = timers[0].Value & 0x80 ? true : false;
+    timers[0].CNT = timers[0].Value & 0xC7;
+    UPDATE_REG(0x102, timers[0].CNT);
     //    CPUUpdateTicks();
   }
   if (timerOnOffDelay & 2)
   {
-    timer1ClockReload = TIMER_TICKS[timer1Value & 3];        
-    if(!timer1On && (timer1Value & 0x80)) {
+    timers[1].ClockReload = TIMER_TICKS[timers[1].Value & 3];        
+    if(!timers[1].On && (timers[1].Value & 0x80)) {
       // reload the counter
-      TM1D = timer1Reload;      
-      timer1Ticks = (0x10000 - TM1D) << timer1ClockReload;
-      UPDATE_REG(0x104, TM1D);
+      timers[1].D = timers[1].Reload;      
+      timers[1].Ticks = (0x10000 - timers[1].D) << timers[1].ClockReload;
+      UPDATE_REG(0x104, timers[1].D);
     }
-    timer1On = timer1Value & 0x80 ? true : false;
-    TM1CNT = timer1Value & 0xC7;
-    UPDATE_REG(0x106, TM1CNT);
+    timers[1].On = timers[1].Value & 0x80 ? true : false;
+    timers[1].CNT = timers[1].Value & 0xC7;
+    UPDATE_REG(0x106, timers[1].CNT);
   }
   if (timerOnOffDelay & 4)
   {
-    timer2ClockReload = TIMER_TICKS[timer2Value & 3];        
-    if(!timer2On && (timer2Value & 0x80)) {
+    timers[2].ClockReload = TIMER_TICKS[timers[2].Value & 3];        
+    if(!timers[2].On && (timers[2].Value & 0x80)) {
       // reload the counter
-      TM2D = timer2Reload;      
-      timer2Ticks = (0x10000 - TM2D) << timer2ClockReload;
-      UPDATE_REG(0x108, TM2D);
+      timers[2].D = timers[2].Reload;      
+      timers[2].Ticks = (0x10000 - timers[2].D) << timers[2].ClockReload;
+      UPDATE_REG(0x108, timers[2].D);
     }
-    timer2On = timer2Value & 0x80 ? true : false;
-    TM2CNT = timer2Value & 0xC7;
-    UPDATE_REG(0x10A, TM2CNT);
+    timers[2].On = timers[2].Value & 0x80 ? true : false;
+    timers[2].CNT = timers[2].Value & 0xC7;
+    UPDATE_REG(0x10A, timers[2].CNT);
   }
   if (timerOnOffDelay & 8)
   {
-    timer3ClockReload = TIMER_TICKS[timer3Value & 3];        
-    if(!timer3On && (timer3Value & 0x80)) {
+    timers[3].ClockReload = TIMER_TICKS[timers[3].Value & 3];        
+    if(!timers[3].On && (timers[3].Value & 0x80)) {
       // reload the counter
-      TM3D = timer3Reload;      
-      timer3Ticks = (0x10000 - TM3D) << timer3ClockReload;
-      UPDATE_REG(0x10C, TM3D);
+      timers[3].D = timers[3].Reload;      
+      timers[3].Ticks = (0x10000 - timers[3].D) << timers[3].ClockReload;
+      UPDATE_REG(0x10C, timers[3].D);
     }
-    timer3On = timer3Value & 0x80 ? true : false;
-    TM3CNT = timer3Value & 0xC7;
-    UPDATE_REG(0x10E, TM3CNT);
+    timers[3].On = timers[3].Value & 0x80 ? true : false;
+    timers[3].CNT = timers[3].Value & 0xC7;
+    UPDATE_REG(0x10E, timers[3].CNT);
   }
   cpuNextEvent = CPUUpdateTicks();
   timerOnOffDelay = 0;
@@ -2610,15 +2277,18 @@ void CPUWriteByte(uint32 address, uint8 b)
       internalRAM[address & 0x7fff] = b;
     break;
   case 4:
-    if(address < 0x4000400) {
-      switch(address & 0x3FF) {
+    if(address < 0x4000400) 
+    {
+     switch(address & 0x3FF) 
+     {
       case 0x301:
 	if(b == 0x80)
 	  stopState = true;
 	holdState = 1;
 	holdType = -1;
-  cpuNextEvent = cpuTotalTicks;
+	cpuNextEvent = cpuTotalTicks;
 	break;
+
       case 0x60:
       case 0x61:
       case 0x62:
@@ -2846,14 +2516,13 @@ void CPUReset()
   BG1CNT   = 0x0000;
   BG2CNT   = 0x0000;
   BG3CNT   = 0x0000;
-  BG0HOFS  = 0x0000;
-  BG0VOFS  = 0x0000;
-  BG1HOFS  = 0x0000;
-  BG1VOFS  = 0x0000;
-  BG2HOFS  = 0x0000;
-  BG2VOFS  = 0x0000;
-  BG3HOFS  = 0x0000;
-  BG3VOFS  = 0x0000;
+
+  for(int i = 0; i < 4; i++)
+  {
+   BGHOFS[i] = 0;
+   BGVOFS[i] = 0;
+  }
+
   BG2PA    = 0x0100;
   BG2PB    = 0x0000;
   BG2PC    = 0x0000;
@@ -2880,38 +2549,16 @@ void CPUReset()
   BLDMOD   = 0x0000;
   COLEV    = 0x0000;
   COLY     = 0x0000;
-  DMSAD_L[0] = 0x0000;
-  DMSAD_H[0] = 0x0000;
-  DMDAD_L[0] = 0x0000;
-  DMDAD_H[0] = 0x0000;
-  DMCNT_L[0] = 0x0000;
-  DMCNT_H[0] = 0x0000;
-  DMSAD_L[1] = 0x0000;
-  DMSAD_H[1] = 0x0000;
-  DMDAD_L[1] = 0x0000;
-  DMDAD_H[1] = 0x0000;
-  DMCNT_L[1] = 0x0000;
-  DMCNT_H[1] = 0x0000;
-  DMSAD_L[2] = 0x0000;
-  DMSAD_H[2] = 0x0000;
-  DMDAD_L[2] = 0x0000;
-  DMDAD_H[2] = 0x0000;
-  DMCNT_L[2] = 0x0000;
-  DMCNT_H[2] = 0x0000;
-  DMSAD_L[3] = 0x0000;
-  DMSAD_H[3] = 0x0000;
-  DMDAD_L[3] = 0x0000;
-  DMDAD_H[3] = 0x0000;
-  DMCNT_L[3] = 0x0000;
-  DMCNT_H[3] = 0x0000;
-  TM0D     = 0x0000;
-  TM0CNT   = 0x0000;
-  TM1D     = 0x0000;
-  TM1CNT   = 0x0000;
-  TM2D     = 0x0000;
-  TM2CNT   = 0x0000;
-  TM3D     = 0x0000;
-  TM3CNT   = 0x0000;
+
+  for(int i = 0; i < 4; i++)
+   DMSAD_L[i] = DMSAD_H[i] = 0;
+
+  for(int i = 0; i < 4; i++)
+  {
+   timers[i].D     = 0;
+   timers[i].CNT   = 0;
+  }
+
   P1       = 0x03FF;
   IE       = 0x0000;
   IF       = 0x0000;
@@ -2973,22 +2620,15 @@ void CPUReset()
   biosProtected[3] = 0xe1;
   
   lcdTicks = (useBios && !skipBios) ? 1008 : 208;
-  timer0On = false;
-  timer0Ticks = 0;
-  timer0Reload = 0;
-  timer0ClockReload  = 0;
-  timer1On = false;
-  timer1Ticks = 0;
-  timer1Reload = 0;
-  timer1ClockReload  = 0;
-  timer2On = false;
-  timer2Ticks = 0;
-  timer2Reload = 0;
-  timer2ClockReload  = 0;
-  timer3On = false;
-  timer3Ticks = 0;
-  timer3Reload = 0;
-  timer3ClockReload  = 0;
+
+  for(int i = 0; i < 4; i++)
+  {
+   timers[i].On = false;
+   timers[i].Ticks = 0;
+   timers[i].Reload = 0;
+   timers[i].ClockReload  = 0;
+  }
+
   dmaSource[0] = 0;
   dmaDest[0] = 0;
   dmaSource[1] = 0;
@@ -3037,7 +2677,7 @@ void CPUReset()
   map[14].mask = 0xFFFF;
 
   eepromReset();
-  flashReset();
+  GBA_Flash_Reset();
   
   soundReset();
 
@@ -3131,25 +2771,17 @@ void CPUInterrupt()
   biosProtected[3] = 0xe5;
 }
 
-static uint32 systemColorMap32[65536];
 static void SetPixelFormat(int rshift, int gshift, int bshift)
 {
- int x;
-
- rshift += 3;
- gshift += 3;
- bshift += 3;
-
- for(x=0;x<65536;x++)
-  systemColorMap32[x] = ((x & 0x1f) << rshift) | (((x & 0x3e0)>>5) << gshift) | (((x & 0x7c00)>>10) << bshift);
+ RedoColorMap(rshift, gshift, bshift);
 }
 
 int32 soundTS = 0;
-static uint16 *padq;
+static uint8 *padq;
 
-void MDFNGBA_SetInput(int port, int type, void *ptr, int attrib)
+void MDFNGBA_SetInput(int port, const char *type, void *ptr)
 {
- padq = (uint16*)ptr;
+ padq = (uint8*)ptr;
 }
 
 static int frameready;
@@ -3318,105 +2950,105 @@ void CPULoop(int ticks)
       }
 
       if(!stopState) {
-        if(timer0On) {
-          timer0Ticks -= clockTicks;
-          if(timer0Ticks <= 0) {
-            timer0Ticks += (0x10000 - timer0Reload) << timer0ClockReload;
+        if(timers[0].On) {
+          timers[0].Ticks -= clockTicks;
+          if(timers[0].Ticks <= 0) {
+            timers[0].Ticks += (0x10000 - timers[0].Reload) << timers[0].ClockReload;
             timerOverflow |= 1;
             soundTimerOverflow(0);
-            if(TM0CNT & 0x40) {
+            if(timers[0].CNT & 0x40) {
               IF |= 0x08;
               UPDATE_REG(0x202, IF);
             }
           }
-          TM0D = 0xFFFF - (timer0Ticks >> timer0ClockReload);
-          UPDATE_REG(0x100, TM0D);            
+          timers[0].D = 0xFFFF - (timers[0].Ticks >> timers[0].ClockReload);
+          UPDATE_REG(0x100, timers[0].D);            
         }
         
-        if(timer1On) {
-          if(TM1CNT & 4) {
+        if(timers[1].On) {
+          if(timers[1].CNT & 4) {
             if(timerOverflow & 1) {
-              TM1D++;
-              if(TM1D == 0) {
-                TM1D += timer1Reload;
+              timers[1].D++;
+              if(timers[1].D == 0) {
+                timers[1].D += timers[1].Reload;
                 timerOverflow |= 2;
                 soundTimerOverflow(1);
-                if(TM1CNT & 0x40) {
+                if(timers[1].CNT & 0x40) {
                   IF |= 0x10;
                   UPDATE_REG(0x202, IF);
                 }
               }
-              UPDATE_REG(0x104, TM1D);
+              UPDATE_REG(0x104, timers[1].D);
             }
           } else {
-            timer1Ticks -= clockTicks;
-            if(timer1Ticks <= 0) {
-              timer1Ticks += (0x10000 - timer1Reload) << timer1ClockReload;
+            timers[1].Ticks -= clockTicks;
+            if(timers[1].Ticks <= 0) {
+              timers[1].Ticks += (0x10000 - timers[1].Reload) << timers[1].ClockReload;
               timerOverflow |= 2;           
               soundTimerOverflow(1);
-              if(TM1CNT & 0x40) {
+              if(timers[1].CNT & 0x40) {
                 IF |= 0x10;
                 UPDATE_REG(0x202, IF);
               }
             }
-            TM1D = 0xFFFF - (timer1Ticks >> timer1ClockReload);
-            UPDATE_REG(0x104, TM1D); 
+            timers[1].D = 0xFFFF - (timers[1].Ticks >> timers[1].ClockReload);
+            UPDATE_REG(0x104, timers[1].D); 
           }
         }
         
-        if(timer2On) {
-          if(TM2CNT & 4) {
+        if(timers[2].On) {
+          if(timers[2].CNT & 4) {
             if(timerOverflow & 2) {
-              TM2D++;
-              if(TM2D == 0) {
-                TM2D += timer2Reload;
+              timers[2].D++;
+              if(timers[2].D == 0) {
+                timers[2].D += timers[2].Reload;
                 timerOverflow |= 4;
-                if(TM2CNT & 0x40) {
+                if(timers[2].CNT & 0x40) {
                   IF |= 0x20;
                   UPDATE_REG(0x202, IF);
                 }
               }
-              UPDATE_REG(0x108, TM2D);
+              UPDATE_REG(0x108, timers[2].D);
             }
           } else {
-            timer2Ticks -= clockTicks;
-            if(timer2Ticks <= 0) {
-              timer2Ticks += (0x10000 - timer2Reload) << timer2ClockReload;
+            timers[2].Ticks -= clockTicks;
+            if(timers[2].Ticks <= 0) {
+              timers[2].Ticks += (0x10000 - timers[2].Reload) << timers[2].ClockReload;
               timerOverflow |= 4;           
-              if(TM2CNT & 0x40) {
+              if(timers[2].CNT & 0x40) {
                 IF |= 0x20;
                 UPDATE_REG(0x202, IF);
               }
             }
-            TM2D = 0xFFFF - (timer2Ticks >> timer2ClockReload);
-            UPDATE_REG(0x108, TM2D); 
+            timers[2].D = 0xFFFF - (timers[2].Ticks >> timers[2].ClockReload);
+            UPDATE_REG(0x108, timers[2].D); 
           }
         }
         
-        if(timer3On) {
-          if(TM3CNT & 4) {
+        if(timers[3].On) {
+          if(timers[3].CNT & 4) {
             if(timerOverflow & 4) {
-              TM3D++;
-              if(TM3D == 0) {
-                TM3D += timer3Reload;
-                if(TM3CNT & 0x40) {
+              timers[3].D++;
+              if(timers[3].D == 0) {
+                timers[3].D += timers[3].Reload;
+                if(timers[3].CNT & 0x40) {
                   IF |= 0x40;
                   UPDATE_REG(0x202, IF);
                 }
               }
-              UPDATE_REG(0x10C, TM3D);
+              UPDATE_REG(0x10C, timers[3].D);
             }
           } else {
-              timer3Ticks -= clockTicks;
-            if(timer3Ticks <= 0) {
-              timer3Ticks += (0x10000 - timer3Reload) << timer3ClockReload;         
-              if(TM3CNT & 0x40) {
+              timers[3].Ticks -= clockTicks;
+            if(timers[3].Ticks <= 0) {
+              timers[3].Ticks += (0x10000 - timers[3].Reload) << timers[3].ClockReload;         
+              if(timers[3].CNT & 0x40) {
                 IF |= 0x40;
                 UPDATE_REG(0x202, IF);
               }
             }
-            TM3D = 0xFFFF - (timer3Ticks >> timer3ClockReload);
-            UPDATE_REG(0x10C, TM3D); 
+            timers[3].D = 0xFFFF - (timers[3].Ticks >> timers[3].ClockReload);
+            UPDATE_REG(0x10C, timers[3].D); 
           }
         }
       }
@@ -3513,8 +3145,7 @@ static void DoRewind(void)
 
 static void Emulate(uint32 *pXBuf, MDFN_Rect *LineWidths, int16 **SoundBuf, int32 *SoundBufSize, int skip)
 {
- padbufblah = *padq;
- MDFNMOV_AddJoy(Joy_StateRegs);
+ padbufblah = padq[0] | (padq[1] << 8);
 
  int didrew = MDFN_StateEvil(needrew);
  needrew = 0;
@@ -3534,7 +3165,7 @@ static void Emulate(uint32 *pXBuf, MDFN_Rect *LineWidths, int16 **SoundBuf, int3
 
  *SoundBuf = MDFNGBASOUND_Flush(SoundBufSize, didrew);
  if(pi)
-  MDFNMP_Draw(pXBuf, 0, *SoundBuf, *SoundBufSize);
+  Player_Draw(pXBuf, 0, *SoundBuf, *SoundBufSize);
 }
 
 static bool ToggleLayer(int which)
@@ -3561,16 +3192,54 @@ static void DoSimpleCommand(int cmd)
 
 static MDFNSetting GBASettings[] =
 {
- { "gba.forcemono", "Force monophonic sound output.", MDFNST_BOOL, "0" },
+ { "gba.forcemono", gettext_noop("Force monophonic sound output."), MDFNST_BOOL, "0" },
+ { "gba.colormap", gettext_noop("Load custom color map from specified file."), MDFNST_STRING, "" },
  { NULL }
 };
 
+static const InputDeviceInputInfoStruct IDII[] =
+{
+ { "a", "A", 7, IDIT_BUTTON_CAN_RAPID, NULL },
+ { "b", "B", 6, IDIT_BUTTON_CAN_RAPID, NULL },
+ { "select", "SELECT", 4, IDIT_BUTTON, NULL },
+ { "start", "START", 5, IDIT_BUTTON, NULL },
+ { "right", "RIGHT →", 3, IDIT_BUTTON, "left" },
+ { "left", "LEFT ←", 2, IDIT_BUTTON, "right" },
+ { "up", "UP ↑", 0, IDIT_BUTTON, "down" },
+ { "down", "DOWN ↓", 1, IDIT_BUTTON, "up" },
+ { "shoulder_r", "SHOULDER R", 9, IDIT_BUTTON, NULL },
+ { "shoulder_l", "SHOULDER L", 8, IDIT_BUTTON, NULL },
+};
+
+static InputDeviceInfoStruct InputDeviceInfo[] =
+{
+ {
+  "gamepad",
+  "Gamepad",
+  NULL,
+  sizeof(IDII) / sizeof(InputDeviceInputInfoStruct),
+  IDII,
+ }
+};
+
+static const InputPortInfoStruct PortInfo[] = 
+{ 
+ { 0, "builtin", "Built-In", sizeof(InputDeviceInfo) / sizeof(InputDeviceInfoStruct), InputDeviceInfo } 
+};
+
+static InputInfoStruct InputInfo = 
+{
+ sizeof(PortInfo) / sizeof(InputPortInfoStruct),
+ PortInfo
+};
 
 MDFNGI EmulatedGBA =
 {
- GISYS_GBA,
  "gba",
+ #ifdef WANT_DEBUGGER
  NULL, 
+ #endif
+ &InputInfo,
  Load,
  NULL,
  CloseGame,
@@ -3584,16 +3253,19 @@ MDFNGI EmulatedGBA =
  Emulate,
  SetPixelFormat,
  MDFNGBA_SetInput,
+ NULL,
+ NULL,
+ NULL,
  MDFNGBA_SetSoundMultiplier,
  MDFNGBA_SetSoundVolume,
  MDFNGBA_Sound,
  DoSimpleCommand,
- NULL, // No netplay yet.
  GBASettings,
  (uint32)((double)4194304 / 70224 * 65536 * 256),
  NULL,
  240,
  160,
+ 240, // Save state preview width
  256 * sizeof(uint32),
  { 0, 0, 240, 160 },
 };
