@@ -94,31 +94,31 @@ void CSusie::Reset(void)
 
 	// Reset ALL variables
 
-	mTMPADR.Word=0;
-	mTILTACUM.Word=0;
-	mHOFF.Word=0;
-	mVOFF.Word=0;
-	mVIDBAS.Word=0;
-	mCOLLBAS.Word=0;
-	mVIDADR.Word=0;
-	mCOLLADR.Word=0;
-	mSCBNEXT.Word=0;
-	mSPRDLINE.Word=0;
-	mHPOSSTRT.Word=0;
-	mVPOSSTRT.Word=0;
-	mSPRHSIZ.Word=0;
-	mSPRVSIZ.Word=0;
-	mSTRETCH.Word=0;
-	mTILT.Word=0;
-	mSPRDOFF.Word=0;
-	mSPRVPOS.Word=0;
-	mCOLLOFF.Word=0;
-	mVSIZACUM.Word=0;
-	mHSIZACUM.Word=0;
-	mHSIZOFF.Word=0x007f;
-	mVSIZOFF.Word=0x007f;
-	mSCBADR.Word=0;
-	mPROCADR.Word=0;
+	mTMPADR.Val16=0;
+	mTILTACUM.Val16=0;
+	mHOFF.Val16=0;
+	mVOFF.Val16=0;
+	mVIDBAS.Val16=0;
+	mCOLLBAS.Val16=0;
+	mVIDADR.Val16=0;
+	mCOLLADR.Val16=0;
+	mSCBNEXT.Val16=0;
+	mSPRDLINE.Val16=0;
+	mHPOSSTRT.Val16=0;
+	mVPOSSTRT.Val16=0;
+	mSPRHSIZ.Val16=0;
+	mSPRVSIZ.Val16=0;
+	mSTRETCH.Val16=0;
+	mTILT.Val16=0;
+	mSPRDOFF.Val16=0;
+	mSPRVPOS.Val16=0;
+	mCOLLOFF.Val16=0;
+	mVSIZACUM.Val16=0;
+	mHSIZACUM.Val16=0;
+	mHSIZOFF.Val16=0x007f;
+	mVSIZOFF.Val16=0x007f;
+	mSCBADR.Val16=0;
+	mPROCADR.Val16=0;
 
 	// Must be initialised to this due to
 	// stun runner math initialisation bug
@@ -288,8 +288,8 @@ uint32 CSusie::PaintSprites(void)
 	TRACE_SUSIE0("**************************************************************");
 	TRACE_SUSIE0("                                                              ");
 
-	TRACE_SUSIE1("PaintSprites() VIDBAS  $%04x",mVIDBAS.Word);
-	TRACE_SUSIE1("PaintSprites() COLLBAS $%04x",mCOLLBAS.Word);
+	TRACE_SUSIE1("PaintSprites() VIDBAS  $%04x",mVIDBAS.Val16);
+	TRACE_SUSIE1("PaintSprites() COLLBAS $%04x",mCOLLBAS.Val16);
 	TRACE_SUSIE1("PaintSprites() SPRSYS  $%02x",Peek(SPRSYS));
 
 	if(!mSUZYBUSEN || !mSPRGO)
@@ -309,7 +309,7 @@ uint32 CSusie::PaintSprites(void)
 
 		// And thus it is documented that only the top byte of SCBNEXT is used.
 		// Its mentioned under the bits that are broke section in the bluebook
-		if(!(mSCBNEXT.Word&0xff00))
+		if(!(mSCBNEXT.Val16&0xff00))
 		{
 			TRACE_SUSIE0("PaintSprites() mSCBNEXT==0 - FINISHED");
 			mSPRSYS_Status=0;	// Engine has finished
@@ -321,19 +321,19 @@ uint32 CSusie::PaintSprites(void)
 			mSPRSYS_Status=1;
 		}
 
-		mTMPADR.Word=mSCBNEXT.Word;	// Copy SCB pointer
-		mSCBADR.Word=mSCBNEXT.Word;	// Copy SCB pointer
-		TRACE_SUSIE1("PaintSprites() SCBADDR $%04x",mSCBADR.Word);
+		mTMPADR.Val16=mSCBNEXT.Val16;	// Copy SCB pointer
+		mSCBADR.Val16=mSCBNEXT.Val16;	// Copy SCB pointer
+		TRACE_SUSIE1("PaintSprites() SCBADDR $%04x",mSCBADR.Val16);
 
-		data=RAM_PEEK(mTMPADR.Word);			// Fetch control 0
+		data=RAM_PEEK(mTMPADR.Val16);			// Fetch control 0
 		TRACE_SUSIE1("PaintSprites() SPRCTL0 $%02x",data);
 		mSPRCTL0_Type=data&0x0007;
 		mSPRCTL0_Vflip=data&0x0010;
 		mSPRCTL0_Hflip=data&0x0020;
 		mSPRCTL0_PixelBits=((data&0x00c0)>>6)+1;
-		mTMPADR.Word+=1;
+		mTMPADR.Val16+=1;
 
-		data=RAM_PEEK(mTMPADR.Word);			// Fetch control 1
+		data=RAM_PEEK(mTMPADR.Val16);			// Fetch control 1
 		TRACE_SUSIE1("PaintSprites() SPRCTL1 $%02x",data);
 		mSPRCTL1_StartLeft=data&0x0001;
 		mSPRCTL1_StartUp=data&0x0002;
@@ -342,17 +342,17 @@ uint32 CSusie::PaintSprites(void)
 		mSPRCTL1_ReloadDepth=(data&0x0030)>>4;
 		mSPRCTL1_Sizing=data&0x0040;	
 		mSPRCTL1_Literal=data&0x0080;
-		mTMPADR.Word+=1;
+		mTMPADR.Val16+=1;
 
-		data=RAM_PEEK(mTMPADR.Word);			// Collision num
+		data=RAM_PEEK(mTMPADR.Val16);			// Collision num
 		TRACE_SUSIE1("PaintSprites() SPRCOLL $%02x",data);
 		mSPRCOLL_Number=data&0x000f;
 		mSPRCOLL_Collide=data&0x0020;
-		mTMPADR.Word+=1;
+		mTMPADR.Val16+=1;
 
-		mSCBNEXT.Word=RAM_PEEKW(mTMPADR.Word);	// Next SCB
-		TRACE_SUSIE1("PaintSprites() SCBNEXT $%04x",mSCBNEXT.Word);
-		mTMPADR.Word+=2;
+		mSCBNEXT.Val16=RAM_PEEKW(mTMPADR.Val16);	// Next SCB
+		TRACE_SUSIE1("PaintSprites() SCBNEXT $%04x",mSCBNEXT.Val16);
+		mTMPADR.Val16+=2;
 
 		cycles_used+=5*SPR_RDWR_CYC;
 
@@ -361,7 +361,7 @@ uint32 CSusie::PaintSprites(void)
 // Although Tom Schenck says this is correct, it doesnt appear to be
 //		if(!mSPRCOLL_Collide && !mSPRSYS_NoCollide)
 //		{
-//			mCollision=RAM_PEEK((mSCBADR.Word+mCOLLOFF.Word)&0xffff);
+//			mCollision=RAM_PEEK((mSCBADR.Val16+mCOLLOFF.Val16)&0xffff);
 //			mCollision&=0x0f;
 //		}
 		mCollision=0;
@@ -371,17 +371,17 @@ uint32 CSusie::PaintSprites(void)
 		if(!mSPRCTL1_SkipSprite)
 		{
 
-			mSPRDLINE.Word=RAM_PEEKW(mTMPADR.Word);	// Sprite pack data
-			TRACE_SUSIE1("PaintSprites() SPRDLINE $%04x",mSPRDLINE.Word);
-			mTMPADR.Word+=2;
+			mSPRDLINE.Val16=RAM_PEEKW(mTMPADR.Val16);	// Sprite pack data
+			TRACE_SUSIE1("PaintSprites() SPRDLINE $%04x",mSPRDLINE.Val16);
+			mTMPADR.Val16+=2;
 
-			mHPOSSTRT.Word=RAM_PEEKW(mTMPADR.Word);	// Sprite horizontal start position
-			TRACE_SUSIE1("PaintSprites() HPOSSTRT $%04x",mHPOSSTRT.Word);
-			mTMPADR.Word+=2;
+			mHPOSSTRT.Val16=RAM_PEEKW(mTMPADR.Val16);	// Sprite horizontal start position
+			TRACE_SUSIE1("PaintSprites() HPOSSTRT $%04x",mHPOSSTRT.Val16);
+			mTMPADR.Val16+=2;
 
-			mVPOSSTRT.Word=RAM_PEEKW(mTMPADR.Word);	// Sprite vertical start position
-			TRACE_SUSIE1("PaintSprites() VPOSSTRT $%04x",mVPOSSTRT.Word);
-			mTMPADR.Word+=2;
+			mVPOSSTRT.Val16=RAM_PEEKW(mTMPADR.Val16);	// Sprite vertical start position
+			TRACE_SUSIE1("PaintSprites() VPOSSTRT $%04x",mVPOSSTRT.Val16);
+			mTMPADR.Val16+=2;
 
 			cycles_used+=6*SPR_RDWR_CYC;
 
@@ -398,11 +398,11 @@ uint32 CSusie::PaintSprites(void)
 					TRACE_SUSIE0("PaintSprites() Sizing Enabled");
 					enable_sizing=TRUE;
 
-					mSPRHSIZ.Word=RAM_PEEKW(mTMPADR.Word);	// Sprite Horizontal size
-					mTMPADR.Word+=2;
+					mSPRHSIZ.Val16=RAM_PEEKW(mTMPADR.Val16);	// Sprite Horizontal size
+					mTMPADR.Val16+=2;
 
-					mSPRVSIZ.Word=RAM_PEEKW(mTMPADR.Word);	// Sprite Verticalal size
-					mTMPADR.Word+=2;
+					mSPRVSIZ.Val16=RAM_PEEKW(mTMPADR.Val16);	// Sprite Verticalal size
+					mTMPADR.Val16+=2;
 
 					cycles_used+=4*SPR_RDWR_CYC;
 					break;
@@ -413,14 +413,14 @@ uint32 CSusie::PaintSprites(void)
 					enable_sizing=TRUE;
 					enable_stretch=TRUE;
 
-					mSPRHSIZ.Word=RAM_PEEKW(mTMPADR.Word);	// Sprite Horizontal size
-					mTMPADR.Word+=2;
+					mSPRHSIZ.Val16=RAM_PEEKW(mTMPADR.Val16);	// Sprite Horizontal size
+					mTMPADR.Val16+=2;
 
-					mSPRVSIZ.Word=RAM_PEEKW(mTMPADR.Word);	// Sprite Verticalal size
-					mTMPADR.Word+=2;
+					mSPRVSIZ.Val16=RAM_PEEKW(mTMPADR.Val16);	// Sprite Verticalal size
+					mTMPADR.Val16+=2;
 
-					mSTRETCH.Word=RAM_PEEKW(mTMPADR.Word);	// Sprite stretch
-					mTMPADR.Word+=2;
+					mSTRETCH.Val16=RAM_PEEKW(mTMPADR.Val16);	// Sprite stretch
+					mTMPADR.Val16+=2;
 
 					cycles_used+=6*SPR_RDWR_CYC;
 					break;
@@ -433,17 +433,17 @@ uint32 CSusie::PaintSprites(void)
 					enable_stretch=TRUE;
 					enable_tilt=TRUE;
 
-					mSPRHSIZ.Word=RAM_PEEKW(mTMPADR.Word);	// Sprite Horizontal size
-					mTMPADR.Word+=2;
+					mSPRHSIZ.Val16=RAM_PEEKW(mTMPADR.Val16);	// Sprite Horizontal size
+					mTMPADR.Val16+=2;
 
-					mSPRVSIZ.Word=RAM_PEEKW(mTMPADR.Word);	// Sprite Verticalal size
-					mTMPADR.Word+=2;
+					mSPRVSIZ.Val16=RAM_PEEKW(mTMPADR.Val16);	// Sprite Verticalal size
+					mTMPADR.Val16+=2;
 
-					mSTRETCH.Word=RAM_PEEKW(mTMPADR.Word);	// Sprite stretch
-					mTMPADR.Word+=2;
+					mSTRETCH.Val16=RAM_PEEKW(mTMPADR.Val16);	// Sprite stretch
+					mTMPADR.Val16+=2;
 
-					mTILT.Word=RAM_PEEKW(mTMPADR.Word);		// Sprite tilt
-					mTMPADR.Word+=2;
+					mTILT.Val16=RAM_PEEKW(mTMPADR.Val16);		// Sprite tilt
+					mTMPADR.Val16+=2;
 
 					cycles_used+=8*SPR_RDWR_CYC;
 					break;
@@ -452,10 +452,10 @@ uint32 CSusie::PaintSprites(void)
 					break;
 			}
 
-			TRACE_SUSIE1("PaintSprites() SPRHSIZ $%04x",mSPRHSIZ.Word);
-			TRACE_SUSIE1("PaintSprites() SPRVSIZ $%04x",mSPRVSIZ.Word);
-			TRACE_SUSIE1("PaintSprites() STRETCH $%04x",mSTRETCH.Word);
-			TRACE_SUSIE1("PaintSprites() TILT    $%04x",mTILT.Word);
+			TRACE_SUSIE1("PaintSprites() SPRHSIZ $%04x",mSPRHSIZ.Val16);
+			TRACE_SUSIE1("PaintSprites() SPRVSIZ $%04x",mSPRVSIZ.Val16);
+			TRACE_SUSIE1("PaintSprites() STRETCH $%04x",mSTRETCH.Val16);
+			TRACE_SUSIE1("PaintSprites() TILT    $%04x",mTILT.Val16);
 
 
 			// Optional Palette reload
@@ -465,9 +465,9 @@ uint32 CSusie::PaintSprites(void)
 				TRACE_SUSIE0("PaintSprites() Palette reloaded");
 				for(int loop=0;loop<8;loop++)
 				{
-					uint8 data=RAM_PEEK(mTMPADR.Word++);
-					mPenIndex[loop*2]=(data>>4)&0x0f;
-					mPenIndex[(loop*2)+1]=data&0x0f;
+					uint8 data_tmp = RAM_PEEK(mTMPADR.Val16++);
+					mPenIndex[loop*2]=(data_tmp>>4)&0x0f;
+					mPenIndex[(loop*2)+1]=data_tmp&0x0f;
 				}
 				// Increment cycle count for the reads
 				cycles_used+=8*SPR_RDWR_CYC;
@@ -480,10 +480,10 @@ uint32 CSusie::PaintSprites(void)
 
 			// Setup screen start end variables
 
-			int screen_h_start=(int16)mHOFF.Word;
-			int screen_h_end=(int16)mHOFF.Word+SCREEN_WIDTH;
-			int screen_v_start=(int16)mVOFF.Word;
-			int screen_v_end=(int16)mVOFF.Word+SCREEN_HEIGHT;
+			int screen_h_start=(int16)mHOFF.Val16;
+			int screen_h_end=(int16)mHOFF.Val16+SCREEN_WIDTH;
+			int screen_v_start=(int16)mVOFF.Val16;
+			int screen_v_end=(int16)mVOFF.Val16+SCREEN_HEIGHT;
 
 			int world_h_mid=screen_h_start+0x8000+(SCREEN_WIDTH/2);
 			int world_v_mid=screen_v_start+0x8000+(SCREEN_HEIGHT/2);
@@ -508,8 +508,8 @@ uint32 CSusie::PaintSprites(void)
 
 			// Check ref is inside screen area
 
-			//if((int16)mHPOSSTRT.Word<screen_h_start || (int16)mHPOSSTRT.Word>=screen_h_end ||
-			//	(int16)mVPOSSTRT.Word<screen_v_start || (int16)mVPOSSTRT.Word>=screen_v_end) superclip=TRUE;
+			//if((int16)mHPOSSTRT.Val16<screen_h_start || (int16)mHPOSSTRT.Val16>=screen_h_end ||
+			//	(int16)mVPOSSTRT.Val16<screen_v_start || (int16)mVPOSSTRT.Val16>=screen_v_end) superclip=TRUE;
 
 			TRACE_SUSIE1("PaintSprites() Superclip=%d",superclip);
 
@@ -531,8 +531,8 @@ uint32 CSusie::PaintSprites(void)
 			{
 				TRACE_SUSIE1("PaintSprites() -------- Rendering Quadrant %03d --------",quadrant);
 
-				int sprite_v=mVPOSSTRT.Word;
-				int sprite_h=mHPOSSTRT.Word;
+				int sprite_v=mVPOSSTRT.Val16;
+				int sprite_h=mHPOSSTRT.Val16;
 
 				bool8 render=FALSE;
 
@@ -549,8 +549,8 @@ uint32 CSusie::PaintSprites(void)
 
 				TRACE_SUSIE2("PaintSprites() Hflip=%d Vflip=%d",mSPRCTL0_Hflip,mSPRCTL0_Vflip);
 				TRACE_SUSIE2("PaintSprites() Hsign=%d   Vsign=%d",hsign,vsign);
-				TRACE_SUSIE2("PaintSprites() Hpos =%04x Vpos =%04x",mHPOSSTRT.Word,mVPOSSTRT.Word);
-				TRACE_SUSIE2("PaintSprites() Hsizoff =%04x Vsizoff =%04x",mHSIZOFF.Word,mVSIZOFF.Word);
+				TRACE_SUSIE2("PaintSprites() Hpos =%04x Vpos =%04x",mHPOSSTRT.Val16,mVPOSSTRT.Val16);
+				TRACE_SUSIE2("PaintSprites() Hsizoff =%04x Vsizoff =%04x",mHSIZOFF.Val16,mVSIZOFF.Val16);
 
 				// Two different rendering algorithms used, on-screen & superclip
 				// when on screen we draw in x until off screen then skip to next
@@ -584,7 +584,7 @@ uint32 CSusie::PaintSprites(void)
 					if(mSPRCTL0_Hflip) modquad=hquadflip[modquad];
 
 					// This is causing Eurosoccer to fail!!
-					//if(enable_tilt && mTILT.Word&0x8000) modquad=hquadflip[modquad];
+					//if(enable_tilt && mTILT.Val16&0x8000) modquad=hquadflip[modquad];
 					//if(quadrant == 0 && sprite_v == 219 && sprite_h == 890)
 					//printf("%d:%d %d %d\n", quadrant, modquad, sprite_h, sprite_v);
 
@@ -623,13 +623,13 @@ uint32 CSusie::PaintSprites(void)
 				if(render)
 				{
 					// Set the vertical position & offset
-					voff=(int16)mVPOSSTRT.Word-screen_v_start;
+					voff=(int16)mVPOSSTRT.Val16-screen_v_start;
 
 					// Zero the stretch,tilt & acum values
-					mTILTACUM.Word=0;
+					mTILTACUM.Val16=0;
 
 					// Perform the SIZOFF
-					if(vsign==1) mVSIZACUM.Word=mVSIZOFF.Word; else mVSIZACUM.Word=0;
+					if(vsign==1) mVSIZACUM.Val16=mVSIZOFF.Val16; else mVSIZACUM.Val16=0;
 
 					// Take the sign of the first quad (0) as the basic
 					// sign, all other quads drawing in the other direction
@@ -642,21 +642,21 @@ uint32 CSusie::PaintSprites(void)
 					for(;;)
 					{
 						// Vertical scaling is done here
-						mVSIZACUM.Word+=mSPRVSIZ.Word;
-						pixel_height=mVSIZACUM.Byte.High;
-						mVSIZACUM.Byte.High=0;
+						mVSIZACUM.Val16+=mSPRVSIZ.Val16;
+						pixel_height=mVSIZACUM.Union8.High;
+						mVSIZACUM.Union8.High=0;
 
 						// Update the next data line pointer and initialise our line
-						mSPRDOFF.Word=(uint16)LineInit(0);
+						mSPRDOFF.Val16=(uint16)LineInit(0);
 
 						// If 1 == next quad, ==0 end of sprite, anyways its END OF LINE
-						if(mSPRDOFF.Word==1)		// End of quad
+						if(mSPRDOFF.Val16==1)		// End of quad
 						{
-							mSPRDLINE.Word+=mSPRDOFF.Word;
+							mSPRDLINE.Val16+=mSPRDOFF.Val16;
 							break;
 						}
 
-						if(mSPRDOFF.Word==0)		// End of sprite
+						if(mSPRDOFF.Val16==0)		// End of sprite
 						{
 							loop=4;		// Halt the quad loop
 							break;
@@ -673,12 +673,12 @@ uint32 CSusie::PaintSprites(void)
 							if(voff>=0 && voff<SCREEN_HEIGHT)
 							{
 								// Work out the horizontal pixel start position, start + tilt
-								mHPOSSTRT.Word+=((int16)mTILTACUM.Word>>8);
-								mTILTACUM.Byte.High=0;
-								hoff=(int)((int16)mHPOSSTRT.Word)-screen_h_start;
+								mHPOSSTRT.Val16+=((int16)mTILTACUM.Val16>>8);
+								mTILTACUM.Union8.High=0;
+								hoff=(int)((int16)mHPOSSTRT.Val16)-screen_h_start;
 
 					 			// Zero/Force the horizontal scaling accumulator
-								if(hsign==1) mHSIZACUM.Word=mHSIZOFF.Word; else mHSIZACUM.Word=0;
+								if(hsign==1) mHSIZACUM.Val16=mHSIZOFF.Val16; else mHSIZACUM.Val16=0;
 
 								// Take the sign of the first quad (0) as the basic
 								// sign, all other quads drawing in the other direction
@@ -696,9 +696,9 @@ uint32 CSusie::PaintSprites(void)
 								while((pixel=LineGetPixel())!=LINE_END)
 								{
 									// This is allowed to update every pixel
-									mHSIZACUM.Word+=mSPRHSIZ.Word;
-									pixel_width=mHSIZACUM.Byte.High;
-									mHSIZACUM.Byte.High=0;
+									mHSIZACUM.Val16+=mSPRHSIZ.Val16;
+									pixel_width=mHSIZACUM.Union8.High;
+									mHSIZACUM.Union8.High=0;
 
 									for(hloop=0;hloop<pixel_width;hloop++)
 									{
@@ -722,21 +722,21 @@ uint32 CSusie::PaintSprites(void)
 							// For every destination line we can modify SPRHSIZ & SPRVSIZ & TILTACUM
 							if(enable_stretch)
 							{
-								mSPRHSIZ.Word+=mSTRETCH.Word;
-//								if(mSPRSYS_VStretch) mSPRVSIZ.Word+=mSTRETCH.Word;
+								mSPRHSIZ.Val16+=mSTRETCH.Val16;
+//								if(mSPRSYS_VStretch) mSPRVSIZ.Val16+=mSTRETCH.Val16;
 							}
 							if(enable_tilt)
 							{
 								// Manipulate the tilt stuff
-								mTILTACUM.Word+=mTILT.Word;
+								mTILTACUM.Val16+=mTILT.Val16;
 							}
 						}
 						// According to the docs this increments per dest line 
 						// but only gets set when the source line is read
-						if(mSPRSYS_VStretch) mSPRVSIZ.Word+=mSTRETCH.Word*pixel_height;
+						if(mSPRSYS_VStretch) mSPRVSIZ.Val16+=mSTRETCH.Val16*pixel_height;
 
 						// Update the line start for our next run thru the loop
-						mSPRDLINE.Word+=mSPRDOFF.Word;
+						mSPRDLINE.Val16+=mSPRDOFF.Val16;
 					}
 				}
 				else
@@ -746,15 +746,15 @@ uint32 CSusie::PaintSprites(void)
 					{
 						// Read the start of line offset
 
-						mSPRDOFF.Word=(uint16)LineInit(0);
+						mSPRDOFF.Val16=(uint16)LineInit(0);
 
 						// We dont want to process data so mSPRDLINE is useless to us
-						mSPRDLINE.Word+=mSPRDOFF.Word;
+						mSPRDLINE.Val16+=mSPRDOFF.Val16;
 
 						// If 1 == next quad, ==0 end of sprite, anyways its END OF LINE
 
-						if(mSPRDOFF.Word==1) break;	// End of quad
-						if(mSPRDOFF.Word==0)		// End of sprite
+						if(mSPRDOFF.Val16==1) break;	// End of quad
+						if(mSPRDOFF.Val16==0)		// End of sprite
 						{
 							loop=4;		// Halt the quad loop
 							break;
@@ -780,9 +780,9 @@ uint32 CSusie::PaintSprites(void)
 					case sprite_boundary_shadow:
 					case sprite_shadow:
 						{
-							uint16 coldep=mSCBADR.Word+mCOLLOFF.Word;
+							uint16 coldep=mSCBADR.Val16+mCOLLOFF.Val16;
 							RAM_POKE(coldep,(uint8)mCollision);
-							TRACE_SUSIE2("PaintSprites() COLLOFF=$%04x SCBADR=$%04x",mCOLLOFF.Word,mSCBADR.Word);
+							TRACE_SUSIE2("PaintSprites() COLLOFF=$%04x SCBADR=$%04x",mCOLLOFF.Val16,mSCBADR.Val16);
 							TRACE_SUSIE2("PaintSprites() Wrote $%02x to SCB collision depositary at $%04x",(uint8)mCollision,coldep);
 						}
 						break;
@@ -793,7 +793,7 @@ uint32 CSusie::PaintSprites(void)
 
 			if(mEVERON)
 			{
-				uint16 coldep=mSCBADR.Word+mCOLLOFF.Word;
+				uint16 coldep=mSCBADR.Val16+mCOLLOFF.Val16;
 				uint8 coldat=RAM_PEEK(coldep);
 				if(!everonscreen) coldat|=0x80; else coldat&=0x7f;
 				RAM_POKE(coldep,coldat);
@@ -947,9 +947,9 @@ inline uint32 CSusie::LineGetBits(uint32 bits)
                 // This assumes data comes into LSB and out of MSB
 //              mLineShiftReg&=0x000000ff;      // Has no effect
                 mLineShiftReg<<=24;
-                mLineShiftReg|=RAM_PEEK(mTMPADR.Word++)<<16;
-                mLineShiftReg|=RAM_PEEK(mTMPADR.Word++)<<8;
-                mLineShiftReg|=RAM_PEEK(mTMPADR.Word++);
+                mLineShiftReg|=RAM_PEEK(mTMPADR.Val16++)<<16;
+                mLineShiftReg|=RAM_PEEK(mTMPADR.Val16++)<<8;
+                mLineShiftReg|=RAM_PEEK(mTMPADR.Val16++);
 
                 mLineShiftRegCount+=24;
 
@@ -1228,8 +1228,8 @@ uint32 CSusie::LineInit(uint32 voff)
 		voff=0;
 	}
 
-	mLineBaseAddress=mVIDBAS.Word+(voff*(SCREEN_WIDTH/2));
-	mLineCollisionAddress=mCOLLBAS.Word+(voff*(SCREEN_WIDTH/2));
+	mLineBaseAddress=mVIDBAS.Val16+(voff*(SCREEN_WIDTH/2));
+	mLineCollisionAddress=mCOLLBAS.Val16+(voff*(SCREEN_WIDTH/2));
 //	TRACE_SUSIE1("LineInit() mLineBaseAddress=$%04x",mLineBaseAddress);
 //	TRACE_SUSIE1("LineInit() mLineCollisionAddress=$%04x",mLineCollisionAddress);
 
@@ -1315,219 +1315,219 @@ void CSusie::Poke(uint32 addr,uint8 data)
 	switch(addr&0xff)
 	{
 		case (TMPADRL&0xff):
-			mTMPADR.Byte.Low=data;
-			mTMPADR.Byte.High=0;
+			mTMPADR.Union8.Low=data;
+			mTMPADR.Union8.High=0;
 			TRACE_SUSIE2("Poke(TMPADRL,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (TMPADRH&0xff):
-			mTMPADR.Byte.High=data;
+			mTMPADR.Union8.High=data;
 			TRACE_SUSIE2("Poke(TMPADRH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (TILTACUML&0xff):
-			mTILTACUM.Byte.Low=data;
-			mTILTACUM.Byte.High=0;
+			mTILTACUM.Union8.Low=data;
+			mTILTACUM.Union8.High=0;
 			TRACE_SUSIE2("Poke(TILTACUML,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (TILTACUMH&0xff):
-			mTILTACUM.Byte.High=data;
+			mTILTACUM.Union8.High=data;
 			TRACE_SUSIE2("Poke(TILTACUMH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (HOFFL&0xff):
-			mHOFF.Byte.Low=data;
-			mHOFF.Byte.High=0;
+			mHOFF.Union8.Low=data;
+			mHOFF.Union8.High=0;
 			TRACE_SUSIE2("Poke(HOFFL,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (HOFFH&0xff):
-			mHOFF.Byte.High=data;
+			mHOFF.Union8.High=data;
 			TRACE_SUSIE2("Poke(HOFFH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (VOFFL&0xff):
-			mVOFF.Byte.Low=data;
-			mVOFF.Byte.High=0;
+			mVOFF.Union8.Low=data;
+			mVOFF.Union8.High=0;
 			TRACE_SUSIE2("Poke(VOFFL,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (VOFFH&0xff):
-			mVOFF.Byte.High=data;
+			mVOFF.Union8.High=data;
 			TRACE_SUSIE2("Poke(VOFFH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (VIDBASL&0xff):
-			mVIDBAS.Byte.Low=data;
-			mVIDBAS.Byte.High=0;
+			mVIDBAS.Union8.Low=data;
+			mVIDBAS.Union8.High=0;
 			TRACE_SUSIE2("Poke(VIDBASL,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (VIDBASH&0xff):
-			mVIDBAS.Byte.High=data;
+			mVIDBAS.Union8.High=data;
 			TRACE_SUSIE2("Poke(VIDBASH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (COLLBASL&0xff):
-			mCOLLBAS.Byte.Low=data;
-			mCOLLBAS.Byte.High=0;
+			mCOLLBAS.Union8.Low=data;
+			mCOLLBAS.Union8.High=0;
 			TRACE_SUSIE2("Poke(COLLBASL,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (COLLBASH&0xff):
-			mCOLLBAS.Byte.High=data;
+			mCOLLBAS.Union8.High=data;
 			TRACE_SUSIE2("Poke(COLLBASH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (VIDADRL&0xff):
-			mVIDADR.Byte.Low=data;
-			mVIDADR.Byte.High=0;
+			mVIDADR.Union8.Low=data;
+			mVIDADR.Union8.High=0;
 			TRACE_SUSIE2("Poke(VIDADRL,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (VIDADRH&0xff):
-			mVIDADR.Byte.High=data;
+			mVIDADR.Union8.High=data;
 			TRACE_SUSIE2("Poke(VIDADRH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (COLLADRL&0xff):
-			mCOLLADR.Byte.Low=data;
-			mCOLLADR.Byte.High=0;
+			mCOLLADR.Union8.Low=data;
+			mCOLLADR.Union8.High=0;
 			TRACE_SUSIE2("Poke(COLLADRL,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (COLLADRH&0xff):
-			mCOLLADR.Byte.High=data;
+			mCOLLADR.Union8.High=data;
 			TRACE_SUSIE2("Poke(COLLADRH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (SCBNEXTL&0xff):
-			mSCBNEXT.Byte.Low=data;
-			mSCBNEXT.Byte.High=0;
+			mSCBNEXT.Union8.Low=data;
+			mSCBNEXT.Union8.High=0;
 			TRACE_SUSIE2("Poke(SCBNEXTL,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (SCBNEXTH&0xff):
-			mSCBNEXT.Byte.High=data;
+			mSCBNEXT.Union8.High=data;
 			TRACE_SUSIE2("Poke(SCBNEXTH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (SPRDLINEL&0xff):
-			mSPRDLINE.Byte.Low=data;
-			mSPRDLINE.Byte.High=0;
+			mSPRDLINE.Union8.Low=data;
+			mSPRDLINE.Union8.High=0;
 			TRACE_SUSIE2("Poke(SPRDLINEL,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (SPRDLINEH&0xff):
-			mSPRDLINE.Byte.High=data;
+			mSPRDLINE.Union8.High=data;
 			TRACE_SUSIE2("Poke(SPRDLINEH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (HPOSSTRTL&0xff):
-			mHPOSSTRT.Byte.Low=data;
-			mHPOSSTRT.Byte.High=0;
+			mHPOSSTRT.Union8.Low=data;
+			mHPOSSTRT.Union8.High=0;
 			TRACE_SUSIE2("Poke(HPOSSTRTL,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (HPOSSTRTH&0xff):
-			mHPOSSTRT.Byte.High=data;
+			mHPOSSTRT.Union8.High=data;
 			TRACE_SUSIE2("Poke(HPOSSTRTH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (VPOSSTRTL&0xff):
-			mVPOSSTRT.Byte.Low=data;
-			mVPOSSTRT.Byte.High=0;
+			mVPOSSTRT.Union8.Low=data;
+			mVPOSSTRT.Union8.High=0;
 			TRACE_SUSIE2("Poke(VPOSSTRTL,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (VPOSSTRTH&0xff):
-			mVPOSSTRT.Byte.High=data;
+			mVPOSSTRT.Union8.High=data;
 			TRACE_SUSIE2("Poke(VPOSSTRTH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (SPRHSIZL&0xff):
-			mSPRHSIZ.Byte.Low=data;
-			mSPRHSIZ.Byte.High=0;
+			mSPRHSIZ.Union8.Low=data;
+			mSPRHSIZ.Union8.High=0;
 			TRACE_SUSIE2("Poke(SPRHSIZL,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (SPRHSIZH&0xff):
-			mSPRHSIZ.Byte.High=data;
+			mSPRHSIZ.Union8.High=data;
 			TRACE_SUSIE2("Poke(SPRHSIZH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (SPRVSIZL&0xff):
-			mSPRVSIZ.Byte.Low=data;
-			mSPRVSIZ.Byte.High=0;
+			mSPRVSIZ.Union8.Low=data;
+			mSPRVSIZ.Union8.High=0;
 			TRACE_SUSIE2("Poke(SPRVSIZL,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (SPRVSIZH&0xff):
-			mSPRVSIZ.Byte.High=data;
+			mSPRVSIZ.Union8.High=data;
 			TRACE_SUSIE2("Poke(SPRVSIZH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (STRETCHL&0xff):
-			mSTRETCH.Byte.Low=data;
-			mSTRETCH.Byte.High=0;
+			mSTRETCH.Union8.Low=data;
+			mSTRETCH.Union8.High=0;
 			TRACE_SUSIE2("Poke(STRETCHL,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (STRETCHH&0xff):
 			TRACE_SUSIE2("Poke(STRETCHH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
-			mSTRETCH.Byte.High=data;
+			mSTRETCH.Union8.High=data;
 			break;
 		case (TILTL&0xff):
-			mTILT.Byte.Low=data;
-			mTILT.Byte.High=0;
+			mTILT.Union8.Low=data;
+			mTILT.Union8.High=0;
 			TRACE_SUSIE2("Poke(TILTL,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (TILTH&0xff):
-			mTILT.Byte.High=data;
+			mTILT.Union8.High=data;
 			TRACE_SUSIE2("Poke(TILTH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (SPRDOFFL&0xff):
 			TRACE_SUSIE2("Poke(SPRDOFFL,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
-			mSPRDOFF.Byte.Low=data;
-			mSPRDOFF.Byte.High=0;
+			mSPRDOFF.Union8.Low=data;
+			mSPRDOFF.Union8.High=0;
 			break;
 		case (SPRDOFFH&0xff):
 			TRACE_SUSIE2("Poke(SPRDOFFH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
-			mSPRDOFF.Byte.High=data;
+			mSPRDOFF.Union8.High=data;
 			break;
 		case (SPRVPOSL&0xff):
 			TRACE_SUSIE2("Poke(SPRVPOSL,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
-			mSPRVPOS.Byte.Low=data;
-			mSPRVPOS.Byte.High=0;
+			mSPRVPOS.Union8.Low=data;
+			mSPRVPOS.Union8.High=0;
 			break;
 		case (SPRVPOSH&0xff):
-			mSPRVPOS.Byte.High=data;
+			mSPRVPOS.Union8.High=data;
 			TRACE_SUSIE2("Poke(SPRVPOSH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (COLLOFFL&0xff):
-			mCOLLOFF.Byte.Low=data;
-			mCOLLOFF.Byte.High=0;
+			mCOLLOFF.Union8.Low=data;
+			mCOLLOFF.Union8.High=0;
 			TRACE_SUSIE2("Poke(COLLOFFL,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (COLLOFFH&0xff):
-			mCOLLOFF.Byte.High=data;
+			mCOLLOFF.Union8.High=data;
 			TRACE_SUSIE2("Poke(COLLOFFH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (VSIZACUML&0xff):
-			mVSIZACUM.Byte.Low=data;
-			mVSIZACUM.Byte.High=0;
+			mVSIZACUM.Union8.Low=data;
+			mVSIZACUM.Union8.High=0;
 			TRACE_SUSIE2("Poke(VSIZACUML,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (VSIZACUMH&0xff):
-			mVSIZACUM.Byte.High=data;
+			mVSIZACUM.Union8.High=data;
 			TRACE_SUSIE2("Poke(VSIZACUMH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (HSIZOFFL&0xff):
-			mHSIZOFF.Byte.Low=data;
-			mHSIZOFF.Byte.High=0;
+			mHSIZOFF.Union8.Low=data;
+			mHSIZOFF.Union8.High=0;
 			TRACE_SUSIE2("Poke(HSIZOFFL,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (HSIZOFFH&0xff):
-			mHSIZOFF.Byte.High=data;
+			mHSIZOFF.Union8.High=data;
 			TRACE_SUSIE2("Poke(HSIZOFFH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (VSIZOFFL&0xff):
-			mVSIZOFF.Byte.Low=data;
-			mVSIZOFF.Byte.High=0;
+			mVSIZOFF.Union8.Low=data;
+			mVSIZOFF.Union8.High=0;
 			TRACE_SUSIE2("Poke(VSIZOFFL,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (VSIZOFFH&0xff):
-			mVSIZOFF.Byte.High=data;
+			mVSIZOFF.Union8.High=data;
 			TRACE_SUSIE2("Poke(VSIZOFFH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (SCBADRL&0xff):
-			mSCBADR.Byte.Low=data;
-			mSCBADR.Byte.High=0;
+			mSCBADR.Union8.Low=data;
+			mSCBADR.Union8.High=0;
 			TRACE_SUSIE2("Poke(SCBADRL,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (SCBADRH&0xff):
-			mSCBADR.Byte.High=data;
+			mSCBADR.Union8.High=data;
 			TRACE_SUSIE2("Poke(SCBADRH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (PROCADRL&0xff):
-			mPROCADR.Byte.Low=data;
-			mPROCADR.Byte.High=0;
+			mPROCADR.Union8.Low=data;
+			mPROCADR.Union8.High=0;
 			TRACE_SUSIE2("Poke(PROCADRL,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 		case (PROCADRH&0xff):
-			mPROCADR.Byte.High=data;
+			mPROCADR.Union8.High=data;
 			TRACE_SUSIE2("Poke(PROCADRH,%02x) at PC=$%04x",data,mSystem.mCpu->GetPC());
 			break;
 
@@ -1731,242 +1731,242 @@ uint8 CSusie::Peek(uint32 addr)
 	switch(addr&0xff)
 	{
 		case (TMPADRL&0xff):
-			retval=mTMPADR.Byte.Low;
+			retval=mTMPADR.Union8.Low;
 			TRACE_SUSIE2("Peek(TMPADRL)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (TMPADRH&0xff):
-			retval=mTMPADR.Byte.High;
+			retval=mTMPADR.Union8.High;
 			TRACE_SUSIE2("Peek(TMPADRH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (TILTACUML&0xff):
-			retval=mTILTACUM.Byte.Low;
+			retval=mTILTACUM.Union8.Low;
 			TRACE_SUSIE2("Peek(TILTACUML)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (TILTACUMH&0xff):
-			retval=mTILTACUM.Byte.High;
+			retval=mTILTACUM.Union8.High;
 			TRACE_SUSIE2("Peek(TILTACUMH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (HOFFL&0xff):
-			retval=mHOFF.Byte.Low;
+			retval=mHOFF.Union8.Low;
 			TRACE_SUSIE2("Peek(HOFFL)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (HOFFH&0xff):
-			retval=mHOFF.Byte.High;
+			retval=mHOFF.Union8.High;
 			TRACE_SUSIE2("Peek(HOFFH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (VOFFL&0xff):
-			retval=mVOFF.Byte.Low;
+			retval=mVOFF.Union8.Low;
 			TRACE_SUSIE2("Peek(VOFFL)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (VOFFH&0xff):
-			retval=mVOFF.Byte.High;
+			retval=mVOFF.Union8.High;
 			TRACE_SUSIE2("Peek(VOFFH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (VIDBASL&0xff):
-			retval=mVIDBAS.Byte.Low;
+			retval=mVIDBAS.Union8.Low;
 			TRACE_SUSIE2("Peek(VIDBASL)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (VIDBASH&0xff):
-			retval=mVIDBAS.Byte.High;
+			retval=mVIDBAS.Union8.High;
 			TRACE_SUSIE2("Peek(VIDBASH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (COLLBASL&0xff):
-			retval=mCOLLBAS.Byte.Low;
+			retval=mCOLLBAS.Union8.Low;
 			TRACE_SUSIE2("Peek(COLLBASL)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (COLLBASH&0xff):
-			retval=mCOLLBAS.Byte.High;
+			retval=mCOLLBAS.Union8.High;
 			TRACE_SUSIE2("Peek(COLLBASH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (VIDADRL&0xff):
-			retval=mVIDADR.Byte.Low;
+			retval=mVIDADR.Union8.Low;
 			TRACE_SUSIE2("Peek(VIDADRL)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (VIDADRH&0xff):
-			retval=mVIDADR.Byte.High;
+			retval=mVIDADR.Union8.High;
 			TRACE_SUSIE2("Peek(VIDADRH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (COLLADRL&0xff):
-			retval=mCOLLADR.Byte.Low;
+			retval=mCOLLADR.Union8.Low;
 			TRACE_SUSIE2("Peek(COLLADRL)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (COLLADRH&0xff):
-			retval=mCOLLADR.Byte.High;
+			retval=mCOLLADR.Union8.High;
 			TRACE_SUSIE2("Peek(COLLADRH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (SCBNEXTL&0xff):
-			retval=mSCBNEXT.Byte.Low;
+			retval=mSCBNEXT.Union8.Low;
 			TRACE_SUSIE2("Peek(SCBNEXTL)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (SCBNEXTH&0xff):
-			retval=mSCBNEXT.Byte.High;
+			retval=mSCBNEXT.Union8.High;
 			TRACE_SUSIE2("Peek(SCBNEXTH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (SPRDLINEL&0xff):
-			retval=mSPRDLINE.Byte.Low;
+			retval=mSPRDLINE.Union8.Low;
 			TRACE_SUSIE2("Peek(SPRDLINEL)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (SPRDLINEH&0xff):
-			retval=mSPRDLINE.Byte.High;
+			retval=mSPRDLINE.Union8.High;
 			TRACE_SUSIE2("Peek(SPRDLINEH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (HPOSSTRTL&0xff):
-			retval=mHPOSSTRT.Byte.Low;
+			retval=mHPOSSTRT.Union8.Low;
 			TRACE_SUSIE2("Peek(HPOSSTRTL)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (HPOSSTRTH&0xff):
-			retval=mHPOSSTRT.Byte.High;
+			retval=mHPOSSTRT.Union8.High;
 			TRACE_SUSIE2("Peek(HPOSSTRTH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (VPOSSTRTL&0xff):
-			retval=mVPOSSTRT.Byte.Low;
+			retval=mVPOSSTRT.Union8.Low;
 			TRACE_SUSIE2("Peek(VPOSSTRTL)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (VPOSSTRTH&0xff):
-			retval=mVPOSSTRT.Byte.High;
+			retval=mVPOSSTRT.Union8.High;
 			TRACE_SUSIE2("Peek(VPOSSTRTH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (SPRHSIZL&0xff):
-			retval=mSPRHSIZ.Byte.Low;
+			retval=mSPRHSIZ.Union8.Low;
 			TRACE_SUSIE2("Peek(SPRHSIZL)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (SPRHSIZH&0xff):
-			retval=mSPRHSIZ.Byte.High;
+			retval=mSPRHSIZ.Union8.High;
 			TRACE_SUSIE2("Peek(SPRHSIZH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (SPRVSIZL&0xff):
-			retval=mSPRVSIZ.Byte.Low;
+			retval=mSPRVSIZ.Union8.Low;
 			TRACE_SUSIE2("Peek(SPRVSIZL)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (SPRVSIZH&0xff):
-			retval=mSPRVSIZ.Byte.High;
+			retval=mSPRVSIZ.Union8.High;
 			TRACE_SUSIE2("Peek(SPRVSIZH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (STRETCHL&0xff):
-			retval=mSTRETCH.Byte.Low;
+			retval=mSTRETCH.Union8.Low;
 			TRACE_SUSIE2("Peek(STRETCHL)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (STRETCHH&0xff):
-			retval=mSTRETCH.Byte.High;
+			retval=mSTRETCH.Union8.High;
 			TRACE_SUSIE2("Peek(STRETCHH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (TILTL&0xff):
-			retval=mTILT.Byte.Low;
+			retval=mTILT.Union8.Low;
 			TRACE_SUSIE2("Peek(TILTL)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (TILTH&0xff):
-			retval=mTILT.Byte.High;
+			retval=mTILT.Union8.High;
 			TRACE_SUSIE2("Peek(TILTH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (SPRDOFFL&0xff):
-			retval=mSPRDOFF.Byte.Low;
+			retval=mSPRDOFF.Union8.Low;
 			TRACE_SUSIE2("Peek(SPRDOFFL)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (SPRDOFFH&0xff):
-			retval=mSPRDOFF.Byte.High;
+			retval=mSPRDOFF.Union8.High;
 			TRACE_SUSIE2("Peek(SPRDOFFH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (SPRVPOSL&0xff):
-			retval=mSPRVPOS.Byte.Low;
+			retval=mSPRVPOS.Union8.Low;
 			TRACE_SUSIE2("Peek(SPRVPOSL)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (SPRVPOSH&0xff):
-			retval=mSPRVPOS.Byte.High;
+			retval=mSPRVPOS.Union8.High;
 			TRACE_SUSIE2("Peek(SPRVPOSH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (COLLOFFL&0xff):
-			retval=mCOLLOFF.Byte.Low;
+			retval=mCOLLOFF.Union8.Low;
 			TRACE_SUSIE2("Peek(COLLOFFL)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (COLLOFFH&0xff):
-			retval=mCOLLOFF.Byte.High;
+			retval=mCOLLOFF.Union8.High;
 			TRACE_SUSIE2("Peek(COLLOFFH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (VSIZACUML&0xff):
-			retval=mVSIZACUM.Byte.Low;
+			retval=mVSIZACUM.Union8.Low;
 			TRACE_SUSIE2("Peek(VSIZACUML)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (VSIZACUMH&0xff):
-			retval=mVSIZACUM.Byte.High;
+			retval=mVSIZACUM.Union8.High;
 			TRACE_SUSIE2("Peek(VSIZACUMH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (HSIZOFFL&0xff):
-			retval=mHSIZOFF.Byte.Low;
+			retval=mHSIZOFF.Union8.Low;
 			TRACE_SUSIE2("Peek(HSIZOFFL)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (HSIZOFFH&0xff):
-			retval=mHSIZOFF.Byte.High;
+			retval=mHSIZOFF.Union8.High;
 			TRACE_SUSIE2("Peek(HSIZOFFH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (VSIZOFFL&0xff):
-			retval=mVSIZOFF.Byte.Low;
+			retval=mVSIZOFF.Union8.Low;
 			TRACE_SUSIE2("Peek(VSIZOFFL)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (VSIZOFFH&0xff):
-			retval=mVSIZOFF.Byte.High;
+			retval=mVSIZOFF.Union8.High;
 			TRACE_SUSIE2("Peek(VSIZOFFH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (SCBADRL&0xff):
-			retval=mSCBADR.Byte.Low;
+			retval=mSCBADR.Union8.Low;
 			TRACE_SUSIE2("Peek(SCBADRL)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (SCBADRH&0xff):
-			retval=mSCBADR.Byte.High;
+			retval=mSCBADR.Union8.High;
 			TRACE_SUSIE2("Peek(SCBADRH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (PROCADRL&0xff):
-			retval=mPROCADR.Byte.Low;
+			retval=mPROCADR.Union8.Low;
 			TRACE_SUSIE2("Peek(PROCADRL)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
 		case (PROCADRH&0xff):
-			retval=mPROCADR.Byte.High;
+			retval=mPROCADR.Union8.High;
 			TRACE_SUSIE2("Peek(PROCADRH)=$%02x at PC=$%04x",retval,mSystem.mCpu->GetPC());
 			return retval;
 			break;
