@@ -19,42 +19,32 @@
  */
 
 #include <stdlib.h>
+#include <errno.h>
 
 #include "types.h"
 #include "mednafen.h"
 #include "memory.h"
 #include "general.h"
 
-void *MDFN_gmalloc(uint32 size)
-{
- void *ret;
- ret=malloc(size);
- if(!ret)  
- {
-  MDFN_PrintError(_("Error allocating memory!  Doing a hard exit."));
-  exit(1);
- }
- return ret;
-}
+//static int testie = 4;
 
-void *MDFN_malloc(uint32 size)
+void *MDFN_malloc_real(uint32 size, const char *purpose, const char *_file, const int _line)
 {
  void *ret;
- ret=malloc(size);
+
+ ret = malloc(size);
+ //testie--;
+ //if(testie <= 0) ret = 0;
  if(!ret)
  {
-  MDFN_PrintError(_("Error allocating memory!"));
+  MDFN_PrintError(_("Error allocating %u bytes for \"%s\" in %s(%d)!"), size, purpose, _file, _line);
   return(0);
  }
  return ret;
 }
 
-void MDFN_free(void *ptr)		// Might do something with this and MDFN_malloc later...
+void MDFN_free(void *ptr)
 {
  free(ptr);
 }
 
-void MDFN_gfree(void *ptr)
-{
- free(ptr);
-}
