@@ -28,7 +28,7 @@
 #include "md5.h"
 #include "mempatcher.h"
 
-static uint8 **RAMPtrs;
+static uint8 **RAMPtrs = NULL;
 static uint32 PageSize;
 static uint32 NumPages;
 
@@ -113,6 +113,21 @@ bool MDFNMP_Init(uint32 ps, uint32 numpages)
  return(1);
 }
 
+void MDFNMP_Kill(void)
+{
+ if(CheatComp)
+ {
+  free(CheatComp);
+  CheatComp = NULL;
+ }
+ if(RAMPtrs)
+ {
+  free(RAMPtrs);
+  RAMPtrs = NULL;
+ }
+}
+
+
 void MDFNMP_AddRAM(uint32 size, uint32 A, uint8 *RAM)
 {
  uint32 AB = A / PageSize;
