@@ -362,7 +362,6 @@ static int StateAction(StateMem *sm, int load, int data_only)
 
  if(load)
  {
-  int x;
   SyncPRG();
   FixNTAR();
   FixCRR();
@@ -372,7 +371,7 @@ static int StateAction(StateMem *sm, int load, int data_only)
    for(int x = 0; x < 8; x++)
      PlayIndex[x] <<= TOINDEXBS - TOINDEX;
 
-  for(x=0x40;x<0x80;x++)
+  for(int x=0x40;x<0x80;x++)
    FixCache(x,IRAM[x]);
  }
  return(ret);
@@ -434,6 +433,56 @@ static void N106_Power(CartInfo *info)
 	 FixCache(x,IRAM[x]);
 }
 
+#ifdef WANT_DEBUGGER
+static void GetAddressSpaceBytes(const char *name, uint32 Address, uint32 Length, uint8 *Buffer)
+{
+ if(!strcmp(name, "wram"))
+ {
+  while(Length--)
+  {
+   Address &= 8192 - 1;
+   *Buffer = WRAM[Address];
+   Address++;
+   Buffer++;
+  }
+ }
+ else if(!strcmp(name, "iram"))
+ {
+  while(Length--)
+  {
+   Address &= 0x7F;
+   *Buffer = IRAM[Address];
+   Address++;
+   Buffer++;
+  }
+ }
+}
+
+static void PutAddressSpaceBytes(const char *name, uint32 Address, uint32 Length, uint32 Granularity, bool hl, const uint8 *Buffer)
+{
+ if(!strcmp(name, "wram"))
+ {
+  while(Length--)
+  {
+   Address &= 8192 - 1;
+   WRAM[Address] = *Buffer;
+   Address++;
+   Buffer++;
+  }
+ }
+ else if(!strcmp(name, "iram"))
+ {
+  while(Length--)
+  {
+   Address &= 0x7F;
+   IRAM[Address] = *Buffer;
+   Address++;
+   Buffer++;
+  }
+ }
+}
+#endif
+
 void Mapper19_Init(CartInfo *info)
 {
 	is210=0;
@@ -452,6 +501,10 @@ void Mapper19_Init(CartInfo *info)
 	 info->SaveGame[1]=IRAM;
 	 info->SaveGameLen[1]=128;
 	}
+	#ifdef WANT_DEBUGGER
+	MDFNNES_AddASpace(GetAddressSpaceBytes, PutAddressSpaceBytes, "wram", "Cart WRAM", 13);
+	MDFNNES_AddASpace(GetAddressSpaceBytes, PutAddressSpaceBytes, "iram", "N106 Internal RAM", 7);
+	#endif
 }
 
 static int Mapper210_StateAction(StateMem *sm, int load, int data_only)
