@@ -92,6 +92,19 @@ static INLINE void setpageptr(int s, uint32 A, uint8 *p, int ram)
 }
 
 static uint8 nothing[8192];
+
+uint8 *GetCartPagePtr(uint16 A)
+{
+ int n = A >> 11;
+
+ if(Page[n] == (nothing - (n << 11)))
+ {
+  return(0);
+ }
+ else
+  return(Page[n]);
+}
+
 void ResetCartMapping(void)
 {
  int x;
@@ -433,6 +446,11 @@ void SetupCartMirroring(int m, int hard, uint8 *extra)
  mirrorhard = hard;
 }
 
+bool CartHasHardMirroring(void)
+{
+ return(mirrorhard);
+}
+
 static uint8 *GENIEROM=0;
 
 void FixGenieMap(void);
@@ -445,7 +463,7 @@ void OpenGenie(void)
 
  if(!GENIEROM)
  {
-  if(!(GENIEROM=(uint8 *)MDFN_malloc(4096+1024))) return;
+  if(!(GENIEROM=(uint8 *)MDFN_malloc(4096+1024, _("Game Genie ROM image")))) return;
 
   std::string fn = MDFN_MakeFName(MDFNMKF_GGROM, 0, 0);
 
