@@ -39,6 +39,8 @@ void ResetCartMapping(void);
 void SetupCartPRGMapping(int chip, uint8 *p, uint32 size, int ram);
 void SetupCartCHRMapping(int chip, uint8 *p, uint32 size, int ram);
 void SetupCartMirroring(int m, int hard, uint8 *extra);
+bool CartHasHardMirroring(void);
+uint8 *GetCartPagePtr(uint16 A);
 
 DECLFR(CartBROB);
 DECLFR(CartBR);
