@@ -24,6 +24,7 @@
 #include <string.h>
 #include <string>
 #include <zlib.h>
+#include <math.h>
 
 #include "nes.h"
 #include "x6502.h"
@@ -76,7 +77,7 @@ static int iNES_StateAction(StateMem *sm, int load, int data_only)
   SFEND
  };
 
- if(MDFNGameInfo->nes.type == GIT_VSUNI)
+ if(NESIsVSUni)
   if(!MDFNNES_VSUNIStateAction(sm,load, data_only))
    return(0);
 
@@ -131,6 +132,77 @@ static void iNES_Close(void)
 	iNESFree();
 }
 
+#ifdef WANT_DEBUGGER
+static void iNES_GetAddressSpaceBytes(const char *name, uint32 Address, uint32 Length, uint8 *Buffer)
+{
+ if(!strcmp(name, "prgrom"))
+ {
+  while(Length--)
+  {
+   Address &= (ROM_size << 14) - 1;
+   *Buffer = ROM[Address];
+   Address++;
+   Buffer++;
+  }
+ }
+ else if(!strcmp(name, "chrrom") && VROM_size)
+ {
+  while(Length--)
+  {
+   Address &= (VROM_size << 13) - 1;
+   *Buffer = VROM[Address];
+   Address++;
+   Buffer++;
+  }
+ }
+ else if(!strcmp(name, "chrram") && !VROM_size && CHRRAMSize)
+ {
+  while(Length--)
+  {
+   Address &= CHRRAMSize - 1;
+   *Buffer = VROM[Address];
+   Address++;
+   Buffer++;
+  }
+ }
+}
+
+static void iNES_PutAddressSpaceBytes(const char *name, uint32 Address, uint32 Length, uint32 Granularity, bool hl, const uint8 *Buffer)
+{
+ if(!strcmp(name, "prgrom"))
+ {
+  while(Length--)
+  {
+   Address &= (ROM_size << 14) - 1;
+   ROM[Address] = *Buffer;
+   Address++;
+   Buffer++;
+  }
+ }
+ else if(!strcmp(name, "chrrom") && VROM_size)
+ {
+  while(Length--)
+  {
+   Address &= (VROM_size << 13) - 1;
+   VROM[Address] = *Buffer;
+   Address++;
+   Buffer++;
+  }
+ }
+ else if(!strcmp(name, "chrram") && !VROM_size && CHRRAMSize)
+ {
+  while(Length--)
+  {
+   Address &= CHRRAMSize - 1;
+   VROM[Address] = *Buffer;
+   Address++;
+   Buffer++;
+  }
+ }
+}
+#endif
+
+
 uint32 iNESGameCRC32;
 
 struct CRCMATCH	{
@@ -139,10 +211,10 @@ struct CRCMATCH	{
 };
 
 struct INPSEL {
-	uint32 crc32;
-	int input1;
-	int input2;
-	int inputfc;
+	const uint32 crc32;
+	const char* input1;
+	const char *input2;
+	const char *inputfc;
 };
 
 /* This is mostly for my personal use.  So HA. */
@@ -150,81 +222,82 @@ static void SetInput(void)
 {
  static struct INPSEL moo[]=
 	{
-         {0x3a1694f9,SI_GAMEPAD,SI_GAMEPAD,SIFC_4PLAYER},       /* Nekketsu Kakutou Densetsu */
+         {0x3a1694f9,"gamepad","gamepad","4player"},       /* Nekketsu Kakutou Densetsu */
 
-	 {0xc3c0811d,SI_GAMEPAD,SI_GAMEPAD,SIFC_OEKAKIDS},	/* The two "Oeka Kids" games */
-	 {0x9d048ea4,SI_GAMEPAD,SI_GAMEPAD,SIFC_OEKAKIDS},	/*			     */
+	 {0xc3c0811d,"gamepad","gamepad","oekakids"},	/* The two "Oeka Kids" games */
+	 {0x9d048ea4,"gamepad","gamepad","oekakids"},	/*			     */
 
-	 {0xaf4010ea,SI_GAMEPAD,SI_POWERPADB,-1},	/* World Class Track Meet */
-	 {0xd74b2719,SI_GAMEPAD,SI_POWERPADB,-1},	/* Super Team Games */
-	 {0x61d86167,SI_GAMEPAD,SI_POWERPADB,-1},	/* Street Cop */
-	 {0x6435c095,SI_GAMEPAD,SI_POWERPADB,-1},	    /* Short Order/Eggsplode */
+	 {0xaf4010ea,"gamepad","powerpadb", NULL},	/* World Class Track Meet */
+	 {0xd74b2719,"gamepad","powerpadb", NULL},	/* Super Team Games */
+	 {0x61d86167,"gamepad","powerpadb", NULL},	/* Street Cop */
+	 {0x6435c095,"gamepad","powerpadb", NULL},	    /* Short Order/Eggsplode */
 
 
-	 {0x47232739,SI_GAMEPAD,SI_GAMEPAD,SIFC_TOPRIDER},	/* Top Rider */
+	 {0x48ca0ee1,"gamepad","gamepad", "bworld"},    /* Barcode World */
+	 {0x9f8f200a,"gamepad","gamepad","ftrainera"}, /* Super Mogura Tataki!! - Pokkun Moguraa */
+	 {0x9044550e,"gamepad","gamepad","ftrainera"}, /* Rairai Kyonshizu */
+	 {0x2f128512,"gamepad","gamepad","ftrainera"}, /* Jogging Race */
+	 {0x60ad090a,"gamepad","gamepad","ftrainera"}, /* Athletic World */
 
-	 {0x48ca0ee1,SI_GAMEPAD,SI_GAMEPAD,SIFC_BWORLD},    /* Barcode World */
-	 {0x9f8f200a,SI_GAMEPAD,SI_GAMEPAD,SIFC_FTRAINERA}, /* Super Mogura Tataki!! - Pokkun Moguraa */
-	 {0x9044550e,SI_GAMEPAD,SI_GAMEPAD,SIFC_FTRAINERA}, /* Rairai Kyonshizu */
-	 {0x2f128512,SI_GAMEPAD,SI_GAMEPAD,SIFC_FTRAINERA}, /* Jogging Race */
-	 {0x60ad090a,SI_GAMEPAD,SI_GAMEPAD,SIFC_FTRAINERA}, /* Athletic World */
+	 {0x8a12a7d9,"gamepad","gamepad","ftrainerb"}, /* Totsugeki Fuuun Takeshi Jou */
+	 {0xea90f3e2,"gamepad","gamepad","ftrainerb"}, /* Running Stadium */
+	 {0x370ceb65,"gamepad","gamepad","ftrainerb"}, /* Meiro Dai Sakusen */
+	 // Bad dump? {0x69ffb014,"gamepad","gamepad","ftrainerb"}, /* Fuun Takeshi Jou 2 */
+	 {0x6cca1c1f,"gamepad","gamepad","ftrainerb"}, /* Dai Undoukai */
+	 {0x29de87af,"gamepad","gamepad","ftrainerb"},  /* Aerobics Studio */
+         {0xbba58be5,"gamepad","gamepad","ftrainerb"},  /* Family Trainer: Manhattan Police */
+	 {0xea90f3e2,"gamepad","gamepad","ftrainerb"},  /* Family Trainer:  Running Stadium */
 
-	 {0x8a12a7d9,SI_GAMEPAD,SI_GAMEPAD,SIFC_FTRAINERB}, /* Totsugeki Fuuun Takeshi Jou */
-	 {0xea90f3e2,SI_GAMEPAD,SI_GAMEPAD,SIFC_FTRAINERB}, /* Running Stadium */
-	 {0x370ceb65,SI_GAMEPAD,SI_GAMEPAD,SIFC_FTRAINERB}, /* Meiro Dai Sakusen */
-	 // Bad dump? {0x69ffb014,SI_GAMEPAD,SI_GAMEPAD,SIFC_FTRAINERB}, /* Fuun Takeshi Jou 2 */
-	 {0x6cca1c1f,SI_GAMEPAD,SI_GAMEPAD,SIFC_FTRAINERB}, /* Dai Undoukai */
-	 {0x29de87af,SI_GAMEPAD,SI_GAMEPAD,SIFC_FTRAINERB},  /* Aerobics Studio */
-         {0xbba58be5,SI_GAMEPAD,SI_GAMEPAD,SIFC_FTRAINERB},  /* Family Trainer: Manhattan Police */
-	 {0xea90f3e2,SI_GAMEPAD,SI_GAMEPAD,SIFC_FTRAINERB},  /* Family Trainer:  Running Stadium */
+	 {0xd9f45be9,"gamepad","gamepad","quizking"},  /* Gimme a Break ... */
+	 {0x1545bd13,"gamepad","gamepad","quizking"},  /* Gimme a Break ... 2 */
 
-	 {0xd9f45be9,SI_GAMEPAD,SI_GAMEPAD,SIFC_QUIZKING},  /* Gimme a Break ... */
-	 {0x1545bd13,SI_GAMEPAD,SI_GAMEPAD,SIFC_QUIZKING},  /* Gimme a Break ... 2 */
+	 {0x7b44fb2a,"gamepad","gamepad","mahjong"},  /* Ide Yousuke Meijin no Jissen Mahjong 2 */
+	 {0x9fae4d46,"gamepad","gamepad","mahjong"},  /* Ide Yousuke Meijin no Jissen Mahjong */
 
-	 {0x7b44fb2a,SI_GAMEPAD,SI_GAMEPAD,SIFC_MAHJONG},  /* Ide Yousuke Meijin no Jissen Mahjong 2 */
-	 {0x9fae4d46,SI_GAMEPAD,SI_GAMEPAD,SIFC_MAHJONG},  /* Ide Yousuke Meijin no Jissen Mahjong */
+	 {0x980be936,"gamepad","gamepad","hypershot"}, /* Hyper Olympic */
+	 {0x21f85681,"gamepad","gamepad","hypershot"}, /* Hyper Olympic (Gentei Ban) */
+	 {0x915a53a7,"gamepad","gamepad","hypershot"}, /* Hyper Sports */
+	 {0xad9c63e2,"gamepad",NULL,"shadow"},	/* Space Shadow */
 
-	 {0x980be936,SI_GAMEPAD,SI_GAMEPAD,SIFC_HYPERSHOT}, /* Hyper Olympic */
-	 {0x21f85681,SI_GAMEPAD,SI_GAMEPAD,SIFC_HYPERSHOT}, /* Hyper Olympic (Gentei Ban) */
-	 {0x915a53a7,SI_GAMEPAD,SI_GAMEPAD,SIFC_HYPERSHOT}, /* Hyper Sports */
-	 {0xad9c63e2,SI_GAMEPAD,-1,SIFC_SHADOW},	/* Space Shadow */
+	 {0x4d68cfb1, NULL, "zapper", "none"},	/* Crime Busters */
+	 {0xbbe40dc4,NULL,"zapper", "none"},	/* Baby Boomer */
+	 {0x24598791,NULL,"zapper", "none"},	/* Duck Hunt */
+	 {0xff24d794,NULL,"zapper", "none"},   /* Hogan's Alley */
+	 {0xbeb8ab01,NULL,"zapper", "none"},	/* Gumshoe */
+	 {0xde8fd935,NULL,"zapper", "none"},	/* To the Earth */
+	 {0xedc3662b,NULL,"zapper", "none"},	/* Operation Wolf */
+	 {0x2a6559a1,NULL,"zapper", "none"},	/* Operation Wolf (J) */
 
-	 {0x4d68cfb1, -1, SI_ZAPPER, 0},	/* Crime Busters */
-	 {0xbbe40dc4,-1,SI_ZAPPER,0},	/* Baby Boomer */
-	 {0x24598791,-1,SI_ZAPPER,0},	/* Duck Hunt */
-	 {0xff24d794,-1,SI_ZAPPER,0},   /* Hogan's Alley */
-	 {0xbeb8ab01,-1,SI_ZAPPER,0},	/* Gumshoe */
-	 {0xde8fd935,-1,SI_ZAPPER,0},	/* To the Earth */
-	 {0xedc3662b,-1,SI_ZAPPER,0},	/* Operation Wolf */
-	 {0x2a6559a1,-1,SI_ZAPPER,0},	/* Operation Wolf (J) */
+	 {0x23d17f5e,"gamepad","zapper", "none"},	/* The Lone Ranger */
+	 {0xb8b9aca3,NULL,"zapper", "none"},  /* Wild Gunman */
+	 {0x5112dc21,NULL,"zapper", "none"},  /* Wild Gunman */
+	 {0x4318a2f8,NULL,"zapper", "none"},  /* Barker Bill's Trick Shooting */
+	 {0x5ee6008e,NULL,"zapper", "none"},  /* Mechanized Attack */
+	 {0x3e58a87e,NULL,"zapper", "none"},  /* Freedom Force */
+	 {0x851eb9be,"gamepad","zapper", "none"},	/* Shooting Range */
+	 {0x74bea652,"gamepad","zapper", "none"},	/* Supergun 3-in-1 */
+	 {0x32fb0583,NULL,"arkanoid","none"}, /* Arkanoid(NES) */
+	 {0xd89e5a67,NULL,NULL,"arkanoid"}, /* Arkanoid (J) */
+	 {0x0f141525,NULL,NULL,"arkanoid"}, /* Arkanoid 2(J) */
 
-	 {0x23d17f5e,SI_GAMEPAD,SI_ZAPPER,0},	/* The Lone Ranger */
-	 {0xb8b9aca3,-1,SI_ZAPPER,0},  /* Wild Gunman */
-	 {0x5112dc21,-1,SI_ZAPPER,0},  /* Wild Gunman */
-	 {0x4318a2f8,-1,SI_ZAPPER,0},  /* Barker Bill's Trick Shooting */
-	 {0x5ee6008e,-1,SI_ZAPPER,0},  /* Mechanized Attack */
-	 {0x3e58a87e,-1,SI_ZAPPER,0},  /* Freedom Force */
-	 {0x851eb9be,SI_GAMEPAD,SI_ZAPPER,0},	/* Shooting Range */
-	 {0x74bea652,SI_GAMEPAD,SI_ZAPPER,0},	/* Supergun 3-in-1 */
-	 {0x32fb0583,-1,SI_ARKANOID,0}, /* Arkanoid(NES) */
-	 {0xd89e5a67,-1,-1,SIFC_ARKANOID}, /* Arkanoid (J) */
-	 {0x0f141525,-1,-1,SIFC_ARKANOID}, /* Arkanoid 2(J) */
-
-	 {0x912989dc,-1,-1,SIFC_FKB},	/* Playbox BASIC */
-	 {0xf7606810,-1,-1,SIFC_FKB},	/* Family BASIC 2.0A */
-	 {0x895037bc,-1,-1,SIFC_FKB},	/* Family BASIC 2.1a */
-	 {0xb2530afc,-1,-1,SIFC_FKB},	/* Family BASIC 3.0 */
-	 {0,-1,-1,-1}
+	 {0x912989dc,NULL,NULL,"fkb"},	/* Playbox BASIC */
+	 {0xf7606810,NULL,NULL,"fkb"},	/* Family BASIC 2.0A */
+	 {0x895037bc,NULL,NULL,"fkb"},	/* Family BASIC 2.1a */
+	 {0xb2530afc,NULL,NULL,"fkb"},	/* Family BASIC 3.0 */
+	 {0, NULL, NULL, NULL },
 	};
  int x=0;
 
- while(moo[x].input1>=0 || moo[x].input2>=0 || moo[x].inputfc>=0)
+ while(moo[x].input1 > 0 || moo[x].input2 > 0 || moo[x].inputfc > 0)
  {
   if(moo[x].crc32==iNESGameCRC32)
   {
-   MDFNGameInfo->nes.input[0]=moo[x].input1;
-   MDFNGameInfo->nes.input[1]=moo[x].input2;
-   MDFNGameInfo->nes.inputfc=moo[x].inputfc;
+   MDFNGameInfo->DesiredInput.clear();
+   MDFNGameInfo->DesiredInput.push_back(moo[x].input1);
+   MDFNGameInfo->DesiredInput.push_back(moo[x].input2);
+   MDFNGameInfo->DesiredInput.push_back("gamepad");
+   MDFNGameInfo->DesiredInput.push_back("gamepad");
+   MDFNGameInfo->DesiredInput.push_back(moo[x].inputfc);
    break;
   }
   x++;
@@ -438,7 +511,7 @@ static void CheckHInfo(void)
    MDFN_printf(_("The battery-backed bit should be set.\n"));
   }
   if(tofix&8)
-  {
+	  {
    MDFN_printf(_("This game should not have any CHR ROM.  "));
   }
   MDFN_indent(-1);
@@ -508,21 +581,27 @@ int iNESLoad(const char *name, MDFNFILE *fp, NESGameType *gt)
 
 	if(head.ROM_type&8) Mirroring=2;
 
-        if(!(ROM=(uint8 *)malloc(ROM_size<<14)))
+        if(!(ROM=(uint8 *)MDFN_malloc(ROM_size<<14, _("PRG ROM data"))))
 	{
-	 MDFN_PrintError(_("Error allocating memory for PRG ROM data"));
 	 iNESFree();
 	 return 0;
 	}
-	
+
+	#ifdef WANT_DEBUGGER
+	MDFNNES_AddASpace(iNES_GetAddressSpaceBytes, iNES_PutAddressSpaceBytes, "prgrom", "PRG ROM", (unsigned int)(log(ROM_size << 14) / log(2)));
+	#endif
+
         if (VROM_size) 
-         if(!(VROM=(uint8 *)malloc(VROM_size<<13)))
+	{
+         if(!(VROM=(uint8 *)MDFN_malloc(VROM_size<<13, "CHR ROM data")))
 	 {
 	  iNESFree();
-	  MDFN_PrintError(_("Error allocating memory for CHR ROM data"));
 	  return 0;
 	 }
-
+	 #ifdef WANT_DEBUGGER
+	 MDFNNES_AddASpace(iNES_GetAddressSpaceBytes, iNES_PutAddressSpaceBytes, "chrrom", "CHR ROM", (unsigned int)(log(VROM_size << 13) / log(2)));
+	 #endif
+	}
         memset(ROM,0xFF,ROM_size<<14);
         if(VROM_size) memset(VROM,0xFF,VROM_size<<13);
         if(head.ROM_type&4)     /* Trainer */
@@ -783,6 +862,10 @@ static int NewiNES_Init(int num)
 
     VROM=(uint8 *)malloc(CHRRAMSize);
     SetupCartCHRMapping(0x0,VROM,CHRRAMSize,1);
+
+    #ifdef WANT_DEBUGGER
+    MDFNNES_AddASpace(iNES_GetAddressSpaceBytes, iNES_PutAddressSpaceBytes, "chrram", "CHR RAM", (unsigned int)(log(CHRRAMSize) / log(2)));
+    #endif
    }
    tmp->init(&iNESCart);
    return(1);
