@@ -1,33 +1,37 @@
-/*-----------------------------------------------------------------------------
-	[ADPCM.h]
-		`cobl܂B
+/* Mednafen - Multi-system Emulator
+ *
+ * Copyright notice for this file:
+ *  Copyright (C) 2004 Ki
+ *  Copyright (C) 2007 Mednafen Team
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
 
-		Implement ADPCM functions.
-
-	Copyright (C) 2004 Ki
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
------------------------------------------------------------------------------*/
 #include <stdio.h>		// debug
 #include "pce.h"
 #include "psg.h"
 #include "cdrom.h"
 #include "adpcm.h"
+#include "../scsicd.h"
 
 #define ADPCM_MAXVOLUME			1024
 
 typedef Blip_Synth<blip_good_quality, 16384> ADSynth;
 static ADSynth ADPCMSynth;
 
-static uint8	RAM[0x10000];
+static uint8	*RAM = NULL; //0x10000;
 static uint16	Addr;
 static uint32   ReadAddr;
 static uint16	WriteAddr;
@@ -38,7 +42,6 @@ static uint8 LastCmd;
 static uint32 SampleFreq;
 
 static bool ADPCMLP;
-static uint32 LSampleFreq; // For ADPCM lowpass filter
 
 static uint8 ReadBuffer;
 static int32 ReadPending;
@@ -54,7 +57,7 @@ static bool	bFadeOut = FALSE;
 static int32	ClockCount;
 static int32	FadeCycle;
 
-static uint32 lastts;
+static int32 bigdivacc;
 static int32 bigdiv;
 static int32 smalldiv;
 static int32 last_pcm;
@@ -66,7 +69,7 @@ static void (*_pfnNotification)(uint32 adpcmState);
 	The original decoder is provided by Dave Shadoff,
 	and further optimized by Ki.
 */
-static int32		ad_sample;
+static uint32 ad_sample;
 static int32		ad_ref_index;
 
 static const uint32	_LUT[49*16] =
@@ -182,21 +185,87 @@ static inline int32 saturated_addition(int32 val, int32 add, int32 min, int32 ma
 	return val;
 }
 
-/*-----------------------------------------------------------------------------
-	[decode]
-		ADPCMGR[hꂽf[^𕄍t16rbgI[fBIf[^
-	fR[h܂B
------------------------------------------------------------------------------*/
 static inline int32 decode(uint8 code)
 {
 	uint32 data = _LUT[ad_ref_index * 16 + code];
-	ad_sample = saturated_addition(ad_sample, (int32)data >> 16, -2048, 2047);
+
+	ad_sample += ((int32)data >> 16);
+	ad_sample &= 4095;
 	ad_ref_index = saturated_addition(ad_ref_index, (int16)(data & 0xffff), 0, 48);
-	return ad_sample;
+	return ad_sample - 2048;
 }
 
+#ifdef WANT_DEBUGGER
+uint32 ADPCM_GetRegister(const std::string &name, std::string *special)
+{
+ uint32 value = 0xDEADBEEF;
+
+ if(name == "ADFREQ")
+  value = SampleFreq;
+ else if(name == "ADCUR")
+  value = ad_sample;
+ else if(name == "ADWrAddr")
+  value = WriteAddr;
+ else if(name == "ADWrNibble")
+  value = 0; // fixme
+ else if(name == "ADRdAddr")
+  value = ReadAddr >> 1;
+ else if(name == "ADRdNibble")
+  value = ReadAddr & 1;
+
+ return(value);
+}
+
+void ADPCM_GetAddressSpaceBytes(const char *name, uint32 Address, uint32 Length, uint8 *Buffer)
+{
+ while(Length--)
+ {
+  Address &= 0xFFFF;
+  *Buffer = RAM[Address];
+  Address++;
+  Buffer++;
+ }
+}
+
+void ADPCM_PutAddressSpaceBytes(const char *name, uint32 Address, uint32 Length, uint32 Granularity, bool hl, const uint8 *Buffer)
+{
+ while(Length--)
+ {
+  Address &= 0xFFFF;
+  RAM[Address] = *Buffer;
+  Address++;
+  Buffer++;
+ }
+}
+#endif
+
+static void RedoLPF(int f)
+{
+         if(ADPCMLP)
+         {
+          if(f >= 14)
+          {
+           int rolloff = (int)((((double)32087.5 / (16 - f)) / 2) * 0.70);
+           ADPCMSynth.treble_eq( blip_eq_t::blip_eq_t(-1000, rolloff, FSettings.SndRate));
+          }
+          else
+          {
+           int rolloff = (int)((((double)32087.5 / (16 - f)) / 2) * 0.80);
+           ADPCMSynth.treble_eq( blip_eq_t::blip_eq_t(-1000, rolloff, FSettings.SndRate));
+          }
+         }
+}
+
+
 bool ADPCM_Init()
 {
+	uint32 ADPCMVolumeSetting;
+
+	if(!(RAM = (uint8 *)MDFN_malloc(0x10000, _("PCE ADPCM RAM"))))
+	{
+	 return(0);
+	}
+
 	CurrentVolume = InitialVolume;
 	VolumeStep = InitialVolume / 10;
 	bFadeIn  = FALSE;
@@ -205,41 +274,34 @@ bool ADPCM_Init()
 	FadeCycle = 0;
 	LastCmd = 0;
 
-	ad_sample = 0;
+	ad_sample = 2048;
 	ad_ref_index = 0;
 
-	ADPCMSynth.volume(0.375f); //0.50f); //0.375f); //0.50f); //0.300f);
-	LSampleFreq = 0xFFFFFFFF;
+	ADPCMVolumeSetting = MDFN_GetSettingUI("pce.adpcmvolume");
+	if(ADPCMVolumeSetting != 100)
+	{
+	 MDFN_printf("ADPCM Volume: %d%%\n", ADPCMVolumeSetting);
+	}
+
+	ADPCMSynth.volume(0.42735f * ADPCMVolumeSetting / 100);
 	ADPCMLP = MDFN_GetSettingB("pce.adpcmlp");
+	bigdivacc = (int32)((double)1789772.727272 * 4 * 65536 / 32087.5 * pce_overclocked);
+
+	#ifdef WANT_DEBUGGER
+	PCE_AddASpace(ADPCM_GetAddressSpaceBytes, ADPCM_PutAddressSpaceBytes, "adpcm", "ADPCM RAM", 16);
+	#endif
 
 	return TRUE;
 }
 
 
-/*-----------------------------------------------------------------------------
-	[Deinit]
-		
------------------------------------------------------------------------------*/
-void ADPCM_Deinit()
+void ADPCM_Close()
 {
-	LastCmd = 0;
-	memset(RAM, 0, sizeof(RAM));
-
-	Addr = 0;
-	ReadAddr = 0;
-	WriteAddr = 0;
-	LengthCount = 0;
-	SampleFreq = 0;
-
-	CurrentVolume = InitialVolume;
-	VolumeStep = 0;
-	bFadeIn  = FALSE;
-	bFadeOut = FALSE;
-	ClockCount = 0;
-	FadeCycle = 0;
-
-	ad_sample = 0;
-	ad_ref_index = 0;
+	if(RAM)
+	{
+	 free(RAM);
+	 RAM = NULL;
+	}
 }
 
 
@@ -258,6 +320,7 @@ void ADPCM_Reset()
 	LastCmd		= 0;
 
 	// for adpcm play
+	SampleFreq = 0;
 	CurrentVolume = InitialVolume;
 	VolumeStep = InitialVolume / 10;
 	bFadeIn  = FALSE;
@@ -265,10 +328,12 @@ void ADPCM_Reset()
 	ClockCount = 0;
 	FadeCycle = 0;
 
-	ad_sample = 0;
+	ad_sample = 2048;
 	ad_ref_index = 0;
 	bigdiv = 0;
 	smalldiv = 0;
+
+	RedoLPF(SampleFreq);
 }
 
 
@@ -309,27 +374,26 @@ uint8 ADPCM_ReadBuffer()
 {
 	if(!PCE_InDebug)
 	 ReadPending = 24;
+
+        //if((LastCmd & 0x20) && LengthCount) puts("Ooppsssiieee");
+
         return ReadBuffer;
 }
 
 void ADPCM_WriteBuffer(uint8            data)
 {
-	//printf("%d ", SampleFreq);
         WritePending = (int32)(76 + -3.428571428571428354f * (SampleFreq + 1)); // Icky kludge for Sherlock Holmes and Record of Lodoss War
         WritePendingValue = data;
 }
 
 uint8 ADPCM_Read180D(void)
 {
-	if(!PCE_InDebug)
- 	 ADPCM_Update();
-
 	return(LastCmd);
 }
 
 void ADPCM_Write180D(uint8 data)
 {
-        ADPCM_Update();
+	//printf("Wr180D: %02x\n", data);
 	if ((LastCmd & 0x80) && !(data & 0x80))
         {        
 	 ADPCM_Reset();
@@ -354,7 +418,7 @@ void ADPCM_Write180D(uint8 data)
 
         if((data & 0x40) && !(LastCmd & 0x40))
         {
-         ad_sample = 0;
+         ad_sample = 2048;
          ad_ref_index = 0;
         }
 
@@ -386,14 +450,8 @@ bool ADPCM_IsBusyReading(void)
  return(ReadPending > 0);
 }
 
-static void ADPCM_PB_Run(int32 basetime, int32 run_time)
+static INLINE void ADPCM_PB_Run(int32 basetime, int32 run_time)
 {
- // Watch out for overflows, which are currently prevented
- // by calling ADPCM_Run() every VDC scanline.
-
- int32 bigdivacc = (int32)((double)1789772.727272 * 4 * 65536 / 32087.5 * pce_overclocked);
-
- //printf("%d\n", run_time);
  bigdiv -= run_time * 65536;
 
  while(bigdiv <= 0)
@@ -405,7 +463,7 @@ static void ADPCM_PB_Run(int32 basetime, int32 run_time)
    smalldiv --;
    while(smalldiv <= 0)
    {
-    int32 pcm = 0;
+    int32 pcm;
 
     smalldiv += (16 - SampleFreq);
     ReadAddr = (ReadAddr + 1) & 0x1FFFF;
@@ -419,15 +477,11 @@ static void ADPCM_PB_Run(int32 basetime, int32 run_time)
 
     if(!LengthCount)
     {
-     //puts("Play done");
      LastCmd &= ~0x60;
      _pfnNotification(ADPCM_STATE_FULL_PLAYED);
      break;
     }
     pcm = (pcm * CurrentVolume) >> 8;
-    extern int8 CDDAStatus;
-    //if(CDDAStatus == 1) pcm >>= 1;
-    if(CDDAStatus == 1) pcm = (int32)(pcm * 0.83);
     uint32 synthtime = ((basetime + (bigdiv >> 16)));
     if(pce_overclocked > 1) 
      synthtime /= pce_overclocked;
@@ -443,99 +497,93 @@ static void ADPCM_PB_Run(int32 basetime, int32 run_time)
  }
 }
 
-static uint32 ADPCM_AdvanceClock(int32 cycles);
-
-static INLINE int32 min(int32 a, int32 b, int32 c)
+static INLINE void ADPCM_AdvanceClock(int32  cycles)
 {
- if(a < b && a < c) return(a);
- if(b < a && b < c) return(b);
- return(c);
+        if (bFadeOut || bFadeIn)
+        {
+                ClockCount += cycles;
+
+                while (ClockCount >= FadeCycle * pce_overclocked)
+                {
+                        ClockCount -= FadeCycle * pce_overclocked;
+
+                        if (bFadeOut)
+                        {
+                                if (CurrentVolume > 0)
+                                {
+                                        CurrentVolume -= VolumeStep;
+                                        if (CurrentVolume < 0)
+                                        {
+                                                CurrentVolume = 0;
+                                                bFadeOut = FALSE;
+                                                break;
+                                        }
+                                }
+                        }
+                        else if (bFadeIn)
+                        {
+                                if (CurrentVolume < InitialVolume)
+                                {
+                                        CurrentVolume += VolumeStep;
+                                        if (CurrentVolume > InitialVolume)
+                                        {
+                                                CurrentVolume = InitialVolume;
+                                                bFadeIn = FALSE;
+                                                break;
+                                        }
+                                }
+                        }
+                }
+        }
 }
 
-static INLINE int32 min(int32 a, int32 b, int32 c, int32 d)
-{
- if(a < b && a < c && a < d) return(a);
- if(b < a && b < c && b < d) return(b);
- if(c < a && c < b && c < d) return(c);
- return(d);
-}
 
-void ADPCM_Update(void)
+void ADPCM_Run(int32 clocks)
 {
- int32 timestamp = HuCPU.timestamp;
- int32 run_time = timestamp - lastts;
- int32 meow;
+ ADPCM_AdvanceClock(clocks);
+ ADPCM_PB_Run(HuCPU.timestamp, clocks);
 
- if(ADPCMLP && SampleFreq != LSampleFreq)
+ if(WritePending)
  {
-  if(SampleFreq >= 14)
+  WritePending -= clocks;
+  if(WritePending <= 0)
   {
-   int rolloff = (int)((((double)32087.5 / (16 - SampleFreq)) / 2) * 0.70);
+   LengthCount += 2;
 
-   ADPCMSynth.treble_eq( blip_eq_t::blip_eq_t(-1000, rolloff, FSettings.SndRate));
+   if(LengthCount >= HalfCount)
+   {
+    _pfnNotification(ADPCM_STATE_NORMAL);
+   }
+   RAM[WriteAddr++] = WritePendingValue;
+   WritePending = 0;
   }
-  else
-  {
-   int rolloff = (int)((((double)32087.5 / (16 - SampleFreq)) / 2) * 0.80);
-
-   ADPCMSynth.treble_eq( blip_eq_t::blip_eq_t(-1000, rolloff, FSettings.SndRate));
-  }
-  LSampleFreq = SampleFreq;
  }
 
- while(run_time > 0)
+ if(!WritePending)
+  WritePending = PCECD_DoADFun(&WritePendingValue) ? 10 : 0;
+
+ if(ReadPending)
  {
-  meow = min(WritePending?WritePending:run_time, ReadPending?ReadPending:run_time, CDROM_GetNextEventTime(), run_time);
-  run_time -= meow;
-  lastts += meow;
-  ADPCM_PB_Run(lastts, meow);
-  CDROM_AdvanceClock(lastts, meow);
-  ADPCM_AdvanceClock(meow);
-
-  if(WritePending)
+  ReadPending -= clocks;
+  if(ReadPending <= 0)
   {
-   WritePending -= meow;
-   if(WritePending <= 0)
-   {
-    LengthCount += 2;
+   uint8 nib1, nib2;
+   nib1 = (RAM[(ReadAddr >> 1)] >> (((ReadAddr ^ 1) & 1) * 4)) & 0x0F;
+   ReadAddr = (ReadAddr + 1) & 0x1FFFF;
+   nib2 = (RAM[(ReadAddr >> 1)] >> (((ReadAddr ^ 1) & 1) * 4)) & 0x0F;
 
-    if(LengthCount >= HalfCount)
-    {
-     _pfnNotification(ADPCM_STATE_NORMAL);
-    }
-    RAM[WriteAddr++] = WritePendingValue;
-   }
-  }
-
-  if(!WritePending)
-   WritePending = CDROM_DoADFun(&WritePendingValue) ? 46 : 0;
-
-  if(ReadPending)
-  {
-   ReadPending -= meow;
-   if(ReadPending <= 0)
-   {
-    uint8 nib1, nib2;
-    nib1 = (RAM[(ReadAddr >> 1)] >> (((ReadAddr ^ 1) & 1) * 4)) & 0x0F;
-    ReadAddr = (ReadAddr + 1) & 0x1FFFF;
-    nib2 = (RAM[(ReadAddr >> 1)] >> (((ReadAddr ^ 1) & 1) * 4)) & 0x0F;
-
-    ReadAddr = (ReadAddr + 1) & 0x1FFFF;
-    ReadBuffer = (nib1 << 4) | nib2;
-    ReadPending = 0;
-   }
+   ReadAddr = (ReadAddr + 1) & 0x1FFFF;
+   ReadBuffer = (nib1 << 4) | nib2;
+   ReadPending = 0;
   }
  }
 }
 
-void ADPCM_ResetTS(void)
-{
- lastts = 0;
-}
-
 void ADPCM_SetFreq(uint32 freq)
 {
-	//printf("%d\n", freq);
+	if(freq != SampleFreq)
+	 RedoLPF(freq);
+
 	SampleFreq = freq;
 }
 
@@ -590,47 +638,6 @@ void ADPCM_FadeIn(int32	ms)
 	}
 }
 
-static uint32 ADPCM_AdvanceClock(int32	cycles)
-{
-	if (bFadeOut || bFadeIn)
-	{
-		ClockCount += cycles;
-
-		while (ClockCount >= FadeCycle * pce_overclocked)
-		{
-			ClockCount -= FadeCycle * pce_overclocked;
-
-			if (bFadeOut)
-			{
-				if (CurrentVolume > 0)
-				{
-					CurrentVolume -= VolumeStep;
-					if (CurrentVolume < 0)
-					{
-						CurrentVolume = 0;
-						bFadeOut = FALSE;
-						break;
-					}
-				}
-			}
-			else if (bFadeIn)
-			{
-				if (CurrentVolume < InitialVolume)
-				{
-					CurrentVolume += VolumeStep;
-					if (CurrentVolume > InitialVolume)
-					{
-						CurrentVolume = InitialVolume;
-						bFadeIn = FALSE;
-						break;
-					}
-				}
-			}
-		}
-	}
-	return 0;
-}
-
 int ADPCM_StateAction(StateMem *sm, int load, int data_only)
 {
  SFORMAT StateRegs[] =
@@ -669,8 +676,7 @@ int ADPCM_StateAction(StateMem *sm, int load, int data_only)
  int ret = MDFNSS_StateAction(sm, load, data_only, StateRegs, "APCM");
  if(load)
  {
-
-
+  RedoLPF(SampleFreq);
  }
  return(ret);
 }
