@@ -23,12 +23,15 @@
 #include "hes.h"
 #include "debug.h"
 #include "huc.h"
-#include "adpcm.h"
+#include "cdrom.h"
 
 
 HuC6280 HuCPU;
 uint8 *HuCPUFastMap[0x100];
+
+#ifdef WANT_DEBUGGER
 void (*HuC6280_Run)(int32 cycles) = NULL;
+#endif
 
 #define _PC              HuCPU.PC
 #define _A               HuCPU.A
@@ -50,6 +53,10 @@ void (*HuC6280_Run)(int32 cycles) = NULL;
  HuCPU.timestamp+=__x;        \
 }
 
+
+//#define TBOL(_chk)      if(!(_chk & 0x1FFF)) { printf("RAWR: %04x\n", _PC); ADDCYC(1); }
+#define TBOL(_chk)
+
 void HuC6280_StealCycles(int count)
 {
  ADDCYC(count);
@@ -94,6 +101,7 @@ static INLINE void WrMemNorm(unsigned int A, uint8 V)
  PCEWrite[wmpr]((wmpr << 13) | (A & 0x1FFF), V);
 }
 
+#ifdef WANT_DEBUGGER
 static INLINE uint8 RdMemHook(unsigned int A)
 {
  if(HuCPU.ReadHook)
@@ -135,6 +143,7 @@ static INLINE void WrMemHook(unsigned int A, uint8 V)
   HuCPU.WriteHook(&HuCPU,A,V);
  }
 }
+#endif
 
 #define PUSH(V) \
 {       \
@@ -174,6 +183,11 @@ static uint8 ZNTable[256];
 #define ST1_norm VDC_Write_ST(2, x)
 #define ST2_norm VDC_Write_ST(3, x)
 
+#define ST0_testbp { if(HuCPU.WriteHook) { HuCPU.WriteHook(&HuCPU, 0x80000000 | (0xFF * 8192 + 0), x); } VDC_Write_ST_TestBP(0, x); }
+#define ST1_testbp { if(HuCPU.WriteHook) { HuCPU.WriteHook(&HuCPU, 0x80000000 | (0xFF * 8192 + 2), x); } VDC_Write_ST_TestBP(2, x); }
+#define ST2_testbp { if(HuCPU.WriteHook) { HuCPU.WriteHook(&HuCPU, 0x80000000 | (0xFF * 8192 + 3), x); } VDC_Write_ST_TestBP(3, x); }
+
+
 #define LDA	   _A=x;X_ZN(_A)
 #define LDX	   _X=x;X_ZN(_X)
 #define LDY        _Y=x;X_ZN(_Y)
@@ -295,15 +309,12 @@ static uint8 ZNTable[256];
 		 X_ZNT(x);	\
 		}
 		 
-/* Icky icky thing for some undocumented instructions.  Can easily be
-   broken if names of local variables are changed.
-*/
-
 /* Absolute */
 #define GetAB(target) 	\
 {	\
  target=RdOp(_PC);	\
  _PC++;	\
+ TBOL(_PC);  \
  target|=RdOp(_PC)<<8;	\
  _PC++;	\
 }
@@ -399,20 +410,20 @@ static uint8 ZNTable[256];
 
 // Normal version:
 #define BMT_PREHONK_N(pork) HuCPU.in_block_move = IBM_##pork;
-#define BMT_HONKHONK_N(pork) if(_count < 0) { HuCPU.PC = pbackus; return;} continue_the_##pork:
+#define BMT_HONKHONK_N(pork) if(PCE_IsCD) { PCECD_Run(); } if(_count < 0) { HuCPU.PC = pbackus; return;} continue_the_##pork:
 
 // Version for breakpoints preexec:
 #define BMT_PREHONK_P(pork)	HuCPU.in_block_move = 1;
 #define BMT_HONKHONK_P(pork)  ;
 
-#define BMT_TDD	BMT_PREHONK(TDD); do { ADDCYC(5); WrMem(HuCPU.bmt_dest, RdMem(HuCPU.bmt_src)); HuCPU.bmt_src--; HuCPU.bmt_dest--; BMT_HONKHONK(TDD); HuCPU.bmt_length--; } while(HuCPU.bmt_length);
-#define BMT_TAI BMT_PREHONK(TAI); {HuCPU.bmt_alternate = 0; do { ADDCYC(5); WrMem(HuCPU.bmt_dest, RdMem(HuCPU.bmt_src + HuCPU.bmt_alternate)); HuCPU.bmt_dest++; HuCPU.bmt_alternate ^= 1; BMT_HONKHONK(TAI); HuCPU.bmt_length--; } while(HuCPU.bmt_length); }
-#define BMT_TIA BMT_PREHONK(TIA); {HuCPU.bmt_alternate = 0; do { ADDCYC(5); WrMem(HuCPU.bmt_dest + HuCPU.bmt_alternate, RdMem(HuCPU.bmt_src)); HuCPU.bmt_src++; HuCPU.bmt_alternate ^= 1; BMT_HONKHONK(TIA); HuCPU.bmt_length--; } while(HuCPU.bmt_length); } 
-#define BMT_TII BMT_PREHONK(TII); do { ADDCYC(5); WrMem(HuCPU.bmt_dest, RdMem(HuCPU.bmt_src)); HuCPU.bmt_src++; HuCPU.bmt_dest++; BMT_HONKHONK(TII); HuCPU.bmt_length--; } while(HuCPU.bmt_length); 
-#define BMT_TIN BMT_PREHONK(TIN); do { ADDCYC(5); WrMem(HuCPU.bmt_dest, RdMem(HuCPU.bmt_src)); HuCPU.bmt_src++; BMT_HONKHONK(TIN); HuCPU.bmt_length--; } while(HuCPU.bmt_length);
+#define BMT_TDD	BMT_PREHONK(TDD); do { ADDCYC(6); WrMem(HuCPU.bmt_dest, RdMem(HuCPU.bmt_src)); HuCPU.bmt_src--; HuCPU.bmt_dest--; BMT_HONKHONK(TDD); HuCPU.bmt_length--; } while(HuCPU.bmt_length);
+#define BMT_TAI BMT_PREHONK(TAI); {HuCPU.bmt_alternate = 0; do { ADDCYC(6); WrMem(HuCPU.bmt_dest, RdMem(HuCPU.bmt_src + HuCPU.bmt_alternate)); HuCPU.bmt_dest++; HuCPU.bmt_alternate ^= 1; BMT_HONKHONK(TAI); HuCPU.bmt_length--; } while(HuCPU.bmt_length); }
+#define BMT_TIA BMT_PREHONK(TIA); {HuCPU.bmt_alternate = 0; do { ADDCYC(6); WrMem(HuCPU.bmt_dest + HuCPU.bmt_alternate, RdMem(HuCPU.bmt_src)); HuCPU.bmt_src++; HuCPU.bmt_alternate ^= 1; BMT_HONKHONK(TIA); HuCPU.bmt_length--; } while(HuCPU.bmt_length); } 
+#define BMT_TII BMT_PREHONK(TII); do { ADDCYC(6); WrMem(HuCPU.bmt_dest, RdMem(HuCPU.bmt_src)); HuCPU.bmt_src++; HuCPU.bmt_dest++; BMT_HONKHONK(TII); HuCPU.bmt_length--; } while(HuCPU.bmt_length); 
+#define BMT_TIN BMT_PREHONK(TIN); do { ADDCYC(6); WrMem(HuCPU.bmt_dest, RdMem(HuCPU.bmt_src)); HuCPU.bmt_src++; BMT_HONKHONK(TIN); HuCPU.bmt_length--; } while(HuCPU.bmt_length);
 
 // Block memory transfer load
-#define LD_BMT(op)	{ PUSH(_Y); PUSH(_A); PUSH(_X); GetAB(HuCPU.bmt_src); GetAB(HuCPU.bmt_dest); GetAB(HuCPU.bmt_length); op; HuCPU.in_block_move = 0; _X = POP(); _A = POP(); _Y = POP(); break; }
+#define LD_BMT(op)	{ PUSH(_Y); PUSH(_A); PUSH(_X); GetAB(HuCPU.bmt_src); TBOL(_PC); GetAB(HuCPU.bmt_dest); TBOL(_PC); GetAB(HuCPU.bmt_length); TBOL(_PC); op; HuCPU.in_block_move = 0; _X = POP(); _A = POP(); _Y = POP(); break; }
 
 #define ST_ZP(r)	{uint8 EA; GetZP(EA); WrMem(0x2000 + EA, r); break;}
 #define ST_ZPX(r)	{uint8 EA; GetZPI(EA,_X); WrMem(0x2000 + EA, r); break;}
@@ -481,7 +492,9 @@ void HuC6280_Init(void)
 	 else if (x&0x80) ZNTable[x]=N_FLAG;
 	 else ZNTable[x]=0;
 
+	#ifdef WANT_DEBUGGER
 	HuC6280_Debug(NULL, NULL, NULL);
+	#endif
 }
 
 void HuC6280_Power(void)
@@ -498,6 +511,7 @@ void HuC6280_Power(void)
  HuC6280_Reset();
 }
 
+#ifdef WANT_DEBUGGER
 static void HuC6280_RunDebug(int32 cycles)
 {
         #define RdMem RdMemHook
@@ -556,9 +570,6 @@ static void HuC6280_RunDebug(int32 cycles)
             else if(_IRQlow & MDFN_IQIRQ2 & HuCPU.IRQMaskDelay)
              tmpa = 0xFFF6;
 
-            extern int32 needrbcreset;
-            if(needrbcreset > 0)
-                tmpa = 0;
             if(tmpa)
             {
              ADDCYC(8);
@@ -583,20 +594,19 @@ static void HuC6280_RunDebug(int32 cycles)
 
          _PC &= 0xFFFF;     // Our cpu core can only handle PC going about 8192 bytes over, so make sure it never gets that far...
 
+	 PCE_InDebug = 1;
+	 b1 = RdOp(_PC);
+
 	 if(HuCPU.ReadHook || HuCPU.WriteHook)
 	 {
 	  HuC6280 HuSave = HuCPU;
-
-	  PCE_InDebug = 1;
-
 	  HuCPU.preexec = 1;
-	  b1 = RdOp(_PC);
 	  _PC++;
           #define BMT_PREHONK     BMT_PREHONK_P
           #define BMT_HONKHONK    BMT_HONKHONK_P
-	  #define ST0
-	  #define ST1
-	  #define ST2
+	  #define ST0 ST0_testbp
+	  #define ST1 ST1_testbp
+	  #define ST2 ST2_testbp
 
 	  switch(b1)
 	  {
@@ -608,6 +618,8 @@ static void HuC6280_RunDebug(int32 cycles)
 	  PCE_InDebug = 0;
 	  HuCPU = HuSave;
 	 }
+	
+	 PCE_InDebug = 0;
 	 #undef ST0
 	 #undef ST1
 	 #undef ST2
@@ -623,7 +635,7 @@ static void HuC6280_RunDebug(int32 cycles)
 
 	 HuCPU.cpoint = 1;
          PCEDBG_TestFoundBPoint();
-         if(HuCPU.CPUHook) HuCPU.CPUHook(_PC);
+         if(HuCPU.CPUHook) HuCPU.CPUHook(_PC, b1);
 	 if(!HuCPU.cpoint) goto PenguinPower;
 	 HuCPU.cpoint = 0;
 
@@ -632,6 +644,21 @@ static void HuC6280_RunDebug(int32 cycles)
          b1=RdOp(_PC);
 
          ADDCYC(CycTable[b1]);
+
+	 if(PCE_IsCD)
+	  PCECD_Run();
+
+         _PC++;
+         #define BMT_PREHONK     BMT_PREHONK_N
+         #define BMT_HONKHONK    BMT_HONKHONK_N
+         switch(b1)
+         {
+          #include "ops.h"
+         }
+         #undef BMT_PREHONK
+         #undef BMT_HONKHONK
+         _P &= ~T_FLAG;
+         skip_T_flag_clear:;    // goto'd by the SET code
          temp=_tcount;
          _tcount=0;
 
@@ -649,23 +676,6 @@ static void HuC6280_RunDebug(int32 cycles)
            }
           }
          }
-
-	 if(PCE_IsCD)
-	  ADPCM_Update(); // Needed since ADPCM_Update() isn't called when viewing memory address space,
-			  // so that values read from status registers and the like will be updated at the correct time,
-			  // and not just when the machine code tries to read them.  I hope this makes sense...
-
-         _PC++;
-         #define BMT_PREHONK     BMT_PREHONK_N
-         #define BMT_HONKHONK    BMT_HONKHONK_N
-         switch(b1)
-         {
-          #include "ops.h"
-         }
-         #undef BMT_PREHONK
-         #undef BMT_HONKHONK
-         _P &= ~T_FLAG;
-         skip_T_flag_clear:;    // goto'd by the SET code
         }
 
 	#undef pbackus
@@ -678,6 +688,12 @@ static void HuC6280_RunDebug(int32 cycles)
 
 
 static void HuC6280_RunNormal(int32 cycles)
+#else
+         #define ST0 ST0_norm
+         #define ST1 ST1_norm
+         #define ST2 ST2_norm
+void HuC6280_Run(int32 cycles)
+#endif
 {
         #define RdMem RdMemNorm
 	#define RdOp  RdOpNorm
@@ -746,9 +762,6 @@ static void HuC6280_RunNormal(int32 cycles)
 	    else if(_IRQlow & MDFN_IQIRQ2 & HuCPU.IRQMaskDelay)
 	     tmpa = 0xFFF6;
 
-	    extern int32 needrbcreset;
-	    if(needrbcreset > 0)
-		tmpa = 0;
 	    if(tmpa)
 	    {
 	     ADDCYC(8);
@@ -779,24 +792,12 @@ static void HuC6280_RunNormal(int32 cycles)
 	 b1=RdOp(_PC);
 
 	 ADDCYC(CycTable[b1]);
-	 temp=_tcount;
-	 _tcount=0;
 
-	 HuCPU.timer_div -= temp;
-	 while(HuCPU.timer_div < 0)
-	 {
-	  HuCPU.timer_div += 1024 * pce_overclocked;
- 	  if(HuCPU.timer_status)
-	  {
-	   HuCPU.timer_value --;
-	   if(HuCPU.timer_value < 0)
-	   {
-		HuCPU.timer_value = HuCPU.timer_load;
-		HuC6280_IRQBegin(MDFN_IQTIMER);
-	   }
-	  }
-	 }
+         if(PCE_IsCD)
+          PCECD_Run();
+
 	 _PC++;
+	 TBOL(_PC);
 
          #define BMT_PREHONK     BMT_PREHONK_N
          #define BMT_HONKHONK    BMT_HONKHONK_N
@@ -808,6 +809,23 @@ static void HuC6280_RunNormal(int32 cycles)
 	 #undef BMT_HONKHONK
 	 _P &= ~T_FLAG;
 	 skip_T_flag_clear:;	// goto'd by the SET code
+         temp=_tcount;
+         _tcount=0;
+
+         HuCPU.timer_div -= temp;
+         while(HuCPU.timer_div < 0)
+         {
+          HuCPU.timer_div += 1024 * pce_overclocked;
+          if(HuCPU.timer_status)
+          {
+           HuCPU.timer_value --;
+           if(HuCPU.timer_value < 0)
+           {
+                HuCPU.timer_value = HuCPU.timer_load;
+                HuC6280_IRQBegin(MDFN_IQTIMER);
+           }
+          }
+         }
 	}
 
         #undef _PC
@@ -834,7 +852,7 @@ void HuC6280_TimerWrite(unsigned int A, uint8 V)
 	  {
 	  	if(HuCPU.timer_status == 0)
 		{
-			HuCPU.timer_div = 0;
+			HuCPU.timer_div = 1023;
 			HuCPU.timer_value = HuCPU.timer_load;
 		}
 	  }
@@ -932,20 +950,7 @@ int HuC6280_StateAction(StateMem *sm, int load, int data_only)
  if(load)
  {
   HuCPU.PC = tmp_PC;
-  if(load <= 0x0303)
-  {
-   HuCPU.timer_value /= 1024;
-   HuCPU.timer_load /= 1024;
-  }
 
-  if(load < 0x0600)
-  {
-   HuCPU.timer_value--;
-   HuCPU.timer_load--;
-  }
-
-  if(load < 0x0308)
-   HuCPU.IRQlow = (HuCPU.IRQlow & (0x4 | 0x20)) | ((HuCPU.IRQlow & 1) << 1) | ((HuCPU.IRQlow & 2) >> 1);
   // Update MPR cache
   HuC6280_FlushMPRCache();
   REDOSPEEDCACHE();
@@ -963,9 +968,10 @@ void HuC6280_DumpMem(char *filename, uint32 start, uint32 end)
  fclose(fp);
 }
 
-void HuC6280_Debug(void (*CPUHook)(uint32),
+#ifdef WANT_DEBUGGER
+void HuC6280_Debug(void (*CPUHook)(uint32, uint8),
                 uint8 (*ReadHook)(HuC6280 *, unsigned int),
-                void (*WriteHook)(HuC6280 *, unsigned int, uint8))
+                void (*WriteHook)(HuC6280 *, uint32, uint8))
 {
  if(CPUHook || ReadHook || WriteHook)
   HuC6280_Run = HuC6280_RunDebug;
@@ -977,4 +983,4 @@ void HuC6280_Debug(void (*CPUHook)(uint32),
  HuCPU.WriteHook = WriteHook;
 
 }
-
+#endif
