@@ -21,14 +21,16 @@
 #include "pce.h"
 #include "psg.h"
 #include "huc.h"
-#include "adpcm.h"
+#include "cdrom.h"
+#include "input.h"
+#include <trio/trio.h>
 
 static Blip_Synth<blip_good_quality, 4096> WaveSynth;
 static Blip_Synth<blip_med_quality, 4096> NoiseSynth;
 
 Blip_Buffer sbuf[2];		// Used in the CDROM ADPCM code
-
 t_psg psg;
+void PSG_Update(void);
 
 void PSG_SetSoundMultiplier(double multiplier)
 {
@@ -40,19 +42,10 @@ void PSG_SetSoundMultiplier(double multiplier)
 
 static void RedoVolume(void)
 {
- extern int8 CDDAStatus;
  if(PCE_IsCD)
  {
-  if(CDDAStatus == 1 )
-  {
-   WaveSynth.volume((double)FSettings.SoundVolume / 100 / 6 * 0.33);
-   NoiseSynth.volume((double)FSettings.SoundVolume / 100 / 6 * 0.33);
-  }
-  else
-  {
-   WaveSynth.volume((double)FSettings.SoundVolume / 100 / 6 * 0.50);
-   NoiseSynth.volume((double)FSettings.SoundVolume / 100 / 6 * 0.50);
-  }
+  WaveSynth.volume((double)FSettings.SoundVolume / 100 * 0.227 * 0.50 * psg.cdpsgvolume / 100);
+  NoiseSynth.volume((double)FSettings.SoundVolume / 100 * 0.227 * 0.50 * psg.cdpsgvolume / 100);
  }
  else
  {
@@ -68,9 +61,7 @@ void PSG_SetSoundVolume(uint32 volume)
 
 void PSG_Sound(int rate)
 {
- int x, y;
-
- for(y = 0; y < 2; y++)
+ for(int y = 0; y < 2; y++)
  {
   sbuf[y].set_sample_rate(rate ? rate : 44100, 50);
   sbuf[y].clock_rate((long)(1789772.727272 * 4));
@@ -101,15 +92,167 @@ static int32 dbtable[32][32 * 3];
  (ch)->dda_cache[1] = dbtable[(ch)->dda][vlr];	\
 }
 
+#ifdef WANT_DEBUGGER
+void PSG_GetAddressSpaceBytes(const char *name, uint32 Address, uint32 Length, uint8 *Buffer)
+{
+ int which_chan;
+
+ if(trio_sscanf(name, "psgram%d", &which_chan) == 1)
+ {
+  while(Length--)
+  {
+   Address &= 0x1F;
+   *Buffer = psg.channel[which_chan].waveform[Address];
+   Address++;
+   Buffer++;
+  }
+ }
+}
+
+void PSG_PutAddressSpaceBytes(const char *name, uint32 Address, uint32 Length, uint32 Granularity, bool hl, const uint8 *Buffer)
+{
+ int which_chan;
+
+ if(trio_sscanf(name, "psgram%d", &which_chan) == 1)
+ {
+  while(Length--)
+  {
+   Address &= 0x1F;
+   psg.channel[which_chan].waveform[Address] = *Buffer & 0x1F;
+   Address++;
+   Buffer++;
+  }
+ }
+}
+
+#define CHPDMOO(n)      \
+ { "CH"#n"Freq", "PSG Ch"#n" Frequency(Period)", 2 },   \
+ { "CH"#n"Ctrl", "PSG Ch"#n" Control", 1 },     \
+ { "CH"#n"Balance", "PSG Ch"#n" Balance", 1 },  \
+ { "CH"#n"WIndex", "PSG Ch"#n" Waveform Index", 1},     \
+ { "CH"#n"SCache", "PSG Ch"#n" Sample Cache", 1 }
+
+static RegType PSGRegs[] =
+{
+ { "Select", "PSG Channel Select", 1 },
+ { "GBalance", "PSG Global Balance", 1 },
+ { "LFOFreq", "PSG LFO Freq", 1 },
+ { "LFOCtrl", "PSG LFO Control", 1 },
+
+ CHPDMOO(0),
+ CHPDMOO(1),
+ CHPDMOO(2),
+ CHPDMOO(3),
+ CHPDMOO(4),
+ { "CH4NCtrl", "PSG Ch4 Noise Control", 1 },
+ { "CH4LFSR", "PSG Ch4 Noise LFSR", 2 },
+ CHPDMOO(5),
+ { "CH5NCtrl", "PSG Ch5 Noise Control", 1 },
+ { "CH5LFSR", "PSG Ch5 Noise LFSR", 2 },
+
+ { "", "", 0 },
+};
+
+static uint32 PSGDBG_GetRegister(const std::string &name, std::string *special)
+{
+ uint32 value = 0xDEADBEEF;
+
+ if(name == "Select")
+  value = psg.select;
+ else if(name == "GBalance")
+  value = psg.globalbalance;
+ else if(name == "LFOFreq")
+  value = psg.lfofreq;
+ else if(name == "LFOCtrl")
+  value = psg.lfoctrl;
+ else if(!strncmp(name.c_str(), "CH", 2))
+ { 
+  int ch = name[2] - '0';
+  char moomoo[64];
+  strncpy(moomoo, name.c_str() + 3, 63);
+  if(!strcmp(moomoo, "Freq"))
+   value = psg.channel[ch].frequency;
+  else if(!strcmp(moomoo, "Ctrl"))
+   value = psg.channel[ch].control;
+  else if(!strcmp(moomoo, "Balance"))
+   value = psg.channel[ch].balance;
+  else if(!strcmp(moomoo, "WIndex"))
+   value = psg.channel[ch].waveform_index;
+  else if(!strcmp(moomoo, "SCache"))
+   value = psg.channel[ch].dda;
+  else if(!strcmp(moomoo, "NCtrl"))
+   value = psg.channel[ch].noisectrl;
+  else if(!strcmp(moomoo, "LFSR"))
+   value = psg.channel[ch].lfsr & 0x7FFF;
+ }
+ return(value);
+}
+
+static void PSGDBG_SetRegister(const std::string &name, uint32 V)
+{
+ PSG_Update();
+
+ if(name == "Select")
+  PSG_Write(0x00, V);
+ else if(name == "GBalance")
+  PSG_Write(0x01, V);
+ else if(name == "LFOFreq")
+ {
+  PSG_Write(0x08, V);
+ }
+ else if(name == "LFOCtrl")
+  PSG_Write(0x09, V);
+ else if(!strncmp(name.c_str(), "CH", 2))
+ {
+  unsigned int psg_sel_save = psg.select;
+  int ch = name[2] - '0';
+  char moomoo[64];
+  strncpy(moomoo, name.c_str() + 3, 63);
+
+  PSG_Write(0x00, ch);
+
+  if(!strcmp(moomoo, "Freq"))
+  {
+   PSG_Write(0x02, V);
+   PSG_Write(0x03, V >> 8);
+  }
+  else if(!strcmp(moomoo, "Ctrl"))
+   PSG_Write(0x04, V);
+  else if(!strcmp(moomoo, "Balance"))
+   PSG_Write(0x05, V);
+  else if(!strcmp(moomoo, "WIndex"))
+   psg.channel[ch].waveform_index = V & 0x1F;
+  else if(!strcmp(moomoo, "SCache"))
+  {
+   psg.channel[ch].dda = V & 0x1F;
+   redo_ddacache(&psg.channel[ch]);
+  }
+  else if(!strcmp(moomoo, "NCtrl") && ch < 4)
+   psg.channel[ch].noisectrl = V;
+  else if(!strcmp(moomoo, "LFSR") && ch < 4)
+   psg.channel[ch].lfsr = V & 0x7FFF;
+
+  PSG_Write(0x00, psg_sel_save);
+ }
+}
+
+static RegGroupType PSGRegsGroup =
+{
+ PSGRegs,
+ PSGDBG_GetRegister,
+ PSGDBG_SetRegister
+};
+
+
+#endif
+
 int PSG_Init(bool WantMono)
 {
-    int x;
-
     memset(&psg, 0, sizeof(psg));
     if(FSettings.SndRate)
      PSG_Sound(FSettings.SndRate);
 
-    for(x=0;x<32 * 3;x++)
+    for(int x=0; x < 32 * 3; x++)
     {
      int y;
      double flub;
@@ -124,6 +267,26 @@ int PSG_Init(bool WantMono)
     }
     
     psg.forcemono = WantMono;
+    psg.cdpsgvolume = MDFN_GetSettingUI("pce.cdpsgvolume");
+
+    if(psg.cdpsgvolume != 100)
+    {
+     MDFN_printf(_("CD PSG Volume: %d%%\n"), psg.cdpsgvolume);
+    }
+
+    #ifdef WANT_DEBUGGER
+    for(int x = 0; x < 6; x++)
+    {
+     char tmpname[128], tmpinfo[128];
+
+     trio_snprintf(tmpname, 128, "psgram%d", x);
+     trio_snprintf(tmpinfo, 128, "PSG Ch %d RAM", x + 1);
+
+     PCE_AddASpace(PSG_GetAddressSpaceBytes, PSG_PutAddressSpaceBytes, tmpname, tmpinfo, 5);
+    }
+    PCE_AddRegGroup(&PSGRegsGroup);
+    #endif
+
     return (1);
 }
 
@@ -140,11 +303,9 @@ void PSG_Kill(void)
 
 /* Macro to access currently selected PSG channel */
 #define PSGCH   psg.channel[psg.select]
-void PSG_Update(void);
 
 DECLFW(PSG_Write)
 {
-    PSG_Update();
     int x;
 
     A &= 0x0F;
@@ -178,9 +339,11 @@ DECLFW(PSG_Write)
 
         case 0x04: /* Channel enable, DDA, volume */
 	    if(psg.select > 5) return; // no more than 6 channels, silly game.
+
+            if((PSGCH.control & 0x40) && !(V & 0x40)) PSGCH.waveform_index = 0;
+
             PSGCH.control = V;
 	    PSGCH.al = 0x1F - (PSGCH.control & 0x1F);
-            if((V & 0xC0) == 0x40) PSGCH.waveform_index = 0;
             redo_ddacache(&PSGCH);
             break;
 
@@ -193,17 +356,19 @@ DECLFW(PSG_Write)
             break;
 
         case 0x06: /* Channel waveform data */
-	    if(psg.select > 5) return; // no more than 6 channels, silly game.
+            if(psg.select > 5) return; // no more than 6 channels, silly game.
             V &= 0x1F;
-            PSGCH.waveform[PSGCH.waveform_index] = V;
 
-	    if((PSGCH.control & 0xC0) == 0x00)
+            if(!(PSGCH.control & 0x40))
+             PSGCH.waveform[PSGCH.waveform_index] = V;
+
+            if((PSGCH.control & 0xC0) == 0x00)
              PSGCH.waveform_index = ((PSGCH.waveform_index + 1) & 0x1F);
 
 	    if(PSGCH.control & 0x80)
-	     PSGCH.dda = V;
+             PSGCH.dda = V;
 
-	    redo_ddacache(&PSGCH);
+            redo_ddacache(&PSGCH);
             break;
 
         case 0x07: /* Noise enable and frequency */
@@ -288,12 +453,12 @@ int16 *PSG_Flush(int32 *len, int needrew)
 {
  int32 timestamp;
 
- if(PCE_IsCD)
-  ADPCM_Update();
-
  if(FSettings.SndRate);
+ {
   PSG_Update();
-
+  if(PCE_IsCD)
+   PCECD_Run();
+ }
  timestamp = HuCPU.timestamp / pce_overclocked;
  if(FSettings.SndRate)
  {
@@ -313,11 +478,15 @@ int16 *PSG_Flush(int32 *len, int needrew)
  if(PCE_IsCD)
   RedoVolume();
 
+ INPUT_FixTS();
+
  HuCPU.timestamp = 0;
  psg.lastts = 0;
 
  if(PCE_IsCD)
-  ADPCM_ResetTS();
+ {
+  PCECD_ResetTS();
+ }
 
  if(!FSettings.SndRate)
  {
@@ -328,16 +497,6 @@ int16 *PSG_Flush(int32 *len, int needrew)
  return(psg.WaveIL);
 }
 
-void PSG_Reset(void)
-{
- memset(psg.channel, 0, sizeof(psg.channel));
- psg.select = 0;
- psg.globalbalance = 0;
- psg.lfofreq = 0;
- psg.lfoctrl = 0;
- psg.lfo_counter = 0;
-}
-
 void PSG_Power(void)
 {
  memset(psg.channel, 0, sizeof(psg.channel));
