@@ -20,6 +20,7 @@
 #include "../video.h"
 #include "vdc.h"
 #include "interrupt.h"
+#include "debug.h"
 
 #define ALPHA_MASK	0x200
 
@@ -79,6 +80,7 @@ static INLINE void FixTileCache(fx_vdc_t *vdc, uint16 A)
 #define REGSETP(_reg, _data, _msb) { _reg &= 0xFF << (_msb ? 0 : 8); _reg |= _data << (_msb ? 8 : 0); }
 #define REGGETP(_reg, _msb) ((_reg >> (_msb ? 8 : 0)) & 0xFF)
 
+void snortus(void);
 uint8 FXVDC_Read(fx_vdc_t *vdc, uint32 A)
 {
  uint8 ret = 0;
@@ -89,9 +91,7 @@ uint8 FXVDC_Read(fx_vdc_t *vdc, uint32 A)
  switch(A)
  {
   case 0x0: ret = vdc->status;
-
 	    vdc->status &= ~0x3F;
-
             PCFXIRQ_Assert(vdc->ilevel, 0);
 	    break;
   case 0x2:
@@ -101,10 +101,20 @@ uint8 FXVDC_Read(fx_vdc_t *vdc, uint32 A)
 	   {
 	    if(msb) 
 	    {
+	     #ifdef WANT_DEBUGGER
+	     if(vdc->Aux0BPBase != ~0)
+              PCFXDBG_CheckBP(BPOINT_AUX_READ, vdc->Aux0BPBase + vdc->MARR, 1);
+	     #endif
 	     vdc->MARR += vram_inc_tab[(vdc->CR >> 11) & 0x3];
-	     vdc->read_buffer = vdc->VRAM[vdc->MARR & 0x7FFF];
+	     vdc->read_buffer = vdc->VRAM[vdc->MARR];
 	    }
 	   }
+	   else
+	   {
+	    printf("Bad VDC register read: %d %02x\n", vdc->ilevel, vdc->select);
+		//snortus();
+	    //exit(1);
+	   }
 	   break;
  }
  return(ret);
@@ -116,11 +126,11 @@ static void DoDMA(fx_vdc_t *vdc)
      for(int i = 0; i < 455; i++)
      {
       if(!vdc->DMAReadWrite)
-       vdc->DMAReadBuffer = vdc->VRAM[vdc->SOUR & 0x7FFF];
+       vdc->DMAReadBuffer = vdc->VRAM[vdc->SOUR];
       else
       {
-       vdc->VRAM[vdc->DESR & 0x7FFF] = vdc->DMAReadBuffer;
-       FixTileCache(vdc, vdc->DESR & 0x7FFF);
+       vdc->VRAM[vdc->DESR] = vdc->DMAReadBuffer;
+       FixTileCache(vdc, vdc->DESR);
 
        vdc->DESR += (((vdc->DCR & 0x4) >> 1) ^ 2) - 1;
        vdc->SOUR += (((vdc->DCR & 0x8) >> 2) ^ 2) - 1;
@@ -140,13 +150,15 @@ static void DoDMA(fx_vdc_t *vdc)
       vdc->DMAReadWrite ^= 1;
      } // for()
 }
-
+uint32 v810_getPC(void);
 void FXVDC_Write(fx_vdc_t *vdc, uint32 A, uint8 V)
 {
  int msb = A & 1;
 
  A &= 0x3;
 
+ //if((v810_getPC() < 0x20000) && (!A || vdc->select >=0x05))
+ // printf("VDC Write: %d %04x %02x\n", vdc->ilevel, A, V);
  switch(A)
  {
   case 0x0: vdc->select = V & 0x1F; break;
@@ -157,16 +169,25 @@ void FXVDC_Write(fx_vdc_t *vdc, uint32 A, uint8 V)
 	    case 0x00: REGSETP(vdc->MAWR, V, msb); break;
 	    case 0x01: REGSETP(vdc->MARR, V, msb);
 		       if(msb)
+		       {
+	                #ifdef WANT_DEBUGGER 
+	                if(vdc->Aux0BPBase != ~0)
+          	         PCFXDBG_CheckBP(BPOINT_AUX_READ, vdc->Aux0BPBase + vdc->MARR, 1);
+			#endif
 			vdc->read_buffer = vdc->VRAM[vdc->MARR];
+		       }
 	               break;
 	    case 0x02: if(!msb) vdc->write_latch = V;
 		       else
 		       {
-			if(vdc->MAWR < 0x8000)
+	                #ifdef WANT_DEBUGGER 
+             		if(vdc->Aux0BPBase != ~0)
 			{
- 			 vdc->VRAM[vdc->MAWR & 0x7fff] = (V << 8) | vdc->write_latch;
-			 FixTileCache(vdc, vdc->MAWR & 0x7FFF);
-			} 
+		         PCFXDBG_CheckBP(BPOINT_AUX_WRITE, vdc->Aux0BPBase + vdc->MAWR, 1);
+			}
+			#endif
+			vdc->VRAM[vdc->MAWR] = (V << 8) | vdc->write_latch;
+			FixTileCache(vdc, vdc->MAWR);
 	                vdc->MAWR += vram_inc_tab[(vdc->CR >> 11) & 0x3];
 		       }
 		       break;
@@ -202,7 +223,7 @@ void FXVDC_Write(fx_vdc_t *vdc, uint32 A, uint8 V)
 		       }
 		       break;
 	    case 0x13: REGSETP(vdc->SATB, V, msb); vdc->SATBPending = 1; break;
-//	    default: printf("Oops 2: %04x %02x\n", vdc->select, V);break;
+	    default: printf("Oops 2: %04x %02x\n", vdc->select, V);break;
 	   }
 	   break;
  }
@@ -213,6 +234,28 @@ static const unsigned int bat_width_shift_tab[4] = { 5, 6, 7, 7 };
 static const unsigned int bat_height_tab[2] = { 32, 64 };
 static const unsigned int ClockModeWidths[2] = { 288, 384};
 
+static const uint32 trans_lut[256] =
+{
+ ALPHA_MASK, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+
+ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
+};
+
+
 static void DrawBG(fx_vdc_t *vdc, uint32 *target)
 {
  unsigned int width = (M_vdc_HDW + 1) * 8;
@@ -220,31 +263,25 @@ static void DrawBG(fx_vdc_t *vdc, uint32 *target)
  if(width > ClockModeWidths[vdc->dot_clock])
   width = ClockModeWidths[vdc->dot_clock];
 
- int start = (ClockModeWidths[vdc->dot_clock] - width) / 2;
+ int start = ((M_vdc_HDS) * 8) + 8;
  int end = start + width;
+
+ if(start > 512) start = 512;
+ if(end > 512) end = 512;
+
  int bat_width = bat_width_tab[(vdc->MWR >> 4) & 3];
  int bat_width_mask = bat_width - 1;
  int bat_width_shift = bat_width_shift_tab[(vdc->MWR >> 4) & 3];
  int bat_height_mask = bat_height_tab[(vdc->MWR >> 6) & 1] - 1;
 
-
- // Pseudo-hack for Asuka 120%'s odd video timings
- if(vdc->dot_clock == 1 && M_vdc_HDS == 5 && M_vdc_HDE == 6 && M_vdc_HDW == 43 && M_vdc_HSW == 2)
-  start += 8;
- else if(vdc->dot_clock == 0 && M_vdc_HDS == 2 && M_vdc_HDE == 3 && M_vdc_HDW == 33 && M_vdc_HSW == 2)
-  start += 4;
- //printf("%d %d\n", vdc->BG_XOffset, vdc->BG_YOffset);
-
  if(!(vdc->CR & 0x80)) // BG is disabled
  {
   uint32 color = (vdc->CR & 0x40) ? 0x100 : 0x000;
-  MDFN_FastU32MemsetM8(target, color, ClockModeWidths[vdc->dot_clock]);
-  for(int x = start; x < end; x++)
-   target[x] = color;
+  MDFN_FastU32MemsetM8(target, color | ALPHA_MASK, 512);
   return;
  }
 
- uint32 overscan_color = 0x100;
+ uint32 overscan_color = 0x100 | ALPHA_MASK;
 
  //if(enabled)
  {
@@ -262,75 +299,37 @@ static void DrawBG(fx_vdc_t *vdc, uint32 *target)
    int palette_index = ((bat >> 12) & 0x0F) << 4;
    uint32 raw_pixel;
 
-   raw_pixel = vdc->bg_tile_cache[bat & 0x7FF][vdc->BG_YOffset & 7][vdc->BG_XOffset & 0x7];
-   target[x] = palette_index | raw_pixel;
+   raw_pixel = vdc->bg_tile_cache[bat & 0xFFF][vdc->BG_YOffset & 7][vdc->BG_XOffset & 0x7];
+   target[x] = palette_index | raw_pixel | (raw_pixel ? 0 : ALPHA_MASK);
 
    vdc->BG_XOffset++;
   }
-
   int bat_boom = (vdc->BG_XOffset >> 3) & bat_width_mask;
   int line_sub = vdc->BG_YOffset & 7;
   for(int x = first_end; x < end; x+=8) // This will draw past the right side of the buffer, but since our pitch is 1024, and max width is ~512, we're safe.  Also,
 					// any overflow that is on the visible screen are will be hidden by the overscan color code below this code.
   {
    uint16 bat = vdc->VRAM[bat_boom | bat_y];
+
    uint32 pix_or = (bat >> 8) & 0xF0;
-   uint8 *pix_lut = vdc->bg_tile_cache[bat & 0x7FF][line_sub];
+   uint8 *pix_lut = vdc->bg_tile_cache[bat & 0xFFF][line_sub];
 
-   #ifdef LSB_FIRST
-    #if SIZEOF_LONG == 8
-    uint64 doh = *(uint64 *)pix_lut;
+   (target + 0)[x] = pix_lut[0] | trans_lut[pix_lut[0]] | pix_or;
+   (target + 1)[x] = pix_lut[1] | trans_lut[pix_lut[1]] | pix_or;
+   (target + 2)[x] = pix_lut[2] | trans_lut[pix_lut[2]] | pix_or;
+   (target + 3)[x] = pix_lut[3] | trans_lut[pix_lut[3]] | pix_or;
+   (target + 4)[x] = pix_lut[4] | trans_lut[pix_lut[4]] | pix_or;
+   (target + 5)[x] = pix_lut[5] | trans_lut[pix_lut[5]] | pix_or;
+   (target + 6)[x] = pix_lut[6] | trans_lut[pix_lut[6]] | pix_or;
+   (target + 7)[x] = pix_lut[7] | trans_lut[pix_lut[7]] | pix_or;
 
-    (target + 0)[x] = (doh & 0xFF) | pix_or;
-    doh >>= 8;
-    (target + 1)[x] = (doh & 0xFF) | pix_or;
-    doh >>= 8;
-    (target + 2)[x] = (doh & 0xFF) | pix_or;
-    doh >>= 8;
-    (target + 3)[x] = (doh & 0xFF) | pix_or;
-    doh >>= 8;
-    (target + 4)[x] = (doh & 0xFF) | pix_or;
-    doh >>= 8;
-    (target + 5)[x] = (doh & 0xFF) | pix_or;
-    doh >>= 8;
-    (target + 6)[x] = (doh & 0xFF) | pix_or;
-    doh >>= 8;
-    (target + 7)[x] = (doh) | pix_or;
-    #else
-    uint32 doh = *(uint32 *)pix_lut;
-    (target + 0)[x] = (doh & 0xFF) | pix_or;
-    doh >>= 8;
-    (target + 1)[x] = (doh & 0xFF) | pix_or;
-    doh >>= 8;
-    (target + 2)[x] = (doh & 0xFF) | pix_or;
-    doh >>= 8;
-    (target + 3)[x] = (doh) | pix_or;
-    doh = *(uint32 *)(pix_lut + 4);
-    (target + 4)[x] = (doh & 0xFF) | pix_or;
-    doh >>= 8;
-    (target + 5)[x] = (doh & 0xFF) | pix_or;
-    doh >>= 8;
-    (target + 6)[x] = (doh & 0xFF) | pix_or;
-    doh >>= 8;
-    (target + 7)[x] = (doh) | pix_or;
-    #endif
-   #else
-   (target + 0)[x] = pix_lut[0] | pix_or;
-   (target + 1)[x] = pix_lut[1] | pix_or;
-   (target + 2)[x] = pix_lut[2] | pix_or;
-   (target + 3)[x] = pix_lut[3] | pix_or;
-   (target + 4)[x] = pix_lut[4] | pix_or;
-   (target + 5)[x] = pix_lut[5] | pix_or;
-   (target + 6)[x] = pix_lut[6] | pix_or;
-   (target + 7)[x] = pix_lut[7] | pix_or;
-   #endif
    bat_boom = (bat_boom + 1) & bat_width_mask;
    vdc->BG_XOffset++;
   }
-  MDFN_FastU32MemsetM8(target, overscan_color, (start + 1) & ~1);
+//  MDFN_FastU32MemsetM8(target, overscan_color, (start + 1) & ~1);
   {
    uint32 end_begin = ((end + 1) & ~ 1);
-   MDFN_FastU32MemsetM8(target + end_begin, overscan_color, ClockModeWidths[vdc->dot_clock] - end_begin);
+//   MDFN_FastU32MemsetM8(target + end_begin, overscan_color, 512 - end_begin);
   }
  }
 }
@@ -356,14 +355,14 @@ static void DrawSprites(fx_vdc_t *vdc, uint32 *target, int enabled)
 {
  int active_sprites = 0;
  SPRLE SpriteList[64 * 2]; // (see unlimited_sprites option, *2 to accomodate 32-pixel-width sprites ) //16];
- uint32 sprite_line_buf[1024] __attribute__ ((aligned (16)));
+ uint32 sprite_line_buf[512] __attribute__ ((aligned (16)));
 
  // First, grab the up to 16 sprites.
  for(int i = 0; i < 64; i++)
  {
   int16 y = (vdc->SAT[i * 4 + 0] & 0x3FF) - 0x40;
   uint16 x = (vdc->SAT[i * 4 + 1] & 0x3FF);
-  uint16 no = (vdc->SAT[i * 4 + 2] >> 1) & 0x1FF;	// Todo, cg mode bit
+  uint16 no = (vdc->SAT[i * 4 + 2] >> 1) & 0x3FF;	// Todo, cg mode bit
   uint16 flags = (vdc->SAT[i * 4 + 3]);
 
   uint32 palette_index = (flags & 0xF) << 4;
@@ -457,21 +456,13 @@ static void DrawSprites(fx_vdc_t *vdc, uint32 *target, int enabled)
  if(display_width > ClockModeWidths[vdc->dot_clock])
   display_width = ClockModeWidths[vdc->dot_clock];
 
- uint32 start = (ClockModeWidths[vdc->dot_clock] - display_width) / 2;;
-
- if(vdc->dot_clock == 1 && M_vdc_HDS == 5 && M_vdc_HDE == 6 && M_vdc_HDW == 43 && M_vdc_HSW == 2)
-  start += 8;
- else if(vdc->dot_clock == 0 && M_vdc_HDS == 2 && M_vdc_HDE == 3 && M_vdc_HDW == 33 && M_vdc_HSW == 2)
-  start += 4;
-
+ uint32 start = ((M_vdc_HDS) * 8) + 8;
  uint32 end = start + display_width;
 
- if(end > ClockModeWidths[vdc->dot_clock])
- {
-  end = ClockModeWidths[vdc->dot_clock];
- }
+ if(start > 512) start = 512;
+ if(end > 512) end = 512;
 
- MDFN_FastU32MemsetM8(sprite_line_buf, ALPHA_MASK, (end + 1) & ~1);
+ MDFN_FastU32MemsetM8(sprite_line_buf, ALPHA_MASK, 512);
 
  for(int i = (active_sprites - 1) ; i >= 0; i--)
  {
@@ -554,43 +545,64 @@ static void DrawSprites(fx_vdc_t *vdc, uint32 *target, int enabled)
   }
 }
 
-void FXVDC_DoLine(fx_vdc_t *vdc, uint32 frame_counter, uint32 *linebuffer, int skip)
+enum
+{
+ VDC_PM_VSW = 0,
+ VDC_PM_VDS,
+ VDC_PM_VDW,
+ VDC_PM_VCR,
+};
+
+static void DoVBlank(fx_vdc_t *vdc)
+{
+ //printf("VBlank: %d\n", vdc->ilevel);
+
+ vdc->VBlank_Test_Occurred = TRUE;
+
+ if(vdc->CR & 0x08)
+ {
+  vdc->status |= VDCS_VD;
+  PCFXIRQ_Assert(vdc->ilevel, 1);
+ }
+
+ if(vdc->SATBPending || (vdc->DCR & 0x10))
+ {
+  vdc->SATBPending = 0;
+  vdc->sat_dma_slcounter = 2;
+  for(int x = 0; x < 256; x++)
+   vdc->SAT[x] = vdc->VRAM[(vdc->SATB + x) & 0xFFFF];
+ }
+
+}
+
+void FXVDC_VSync(fx_vdc_t *vdc)
+{
+ if(!vdc->VBlank_Test_Occurred)
+ {
+  //printf("Poo: %d\n", vdc->ilevel);
+  DoVBlank(vdc);
+ }
+
+ vdc->PeriodCounter = M_vdc_VDS;
+ vdc->PeriodMode = VDC_PM_VDS;
+ vdc->VBlank_Test_Occurred = FALSE;
+}
+
+void FXVDC_DoLine(fx_vdc_t *vdc, uint32 *linebuffer, int skip)
 {
  static const unsigned int ClockPixelWidths[3] = { 341, 455, 682 };
- int need_vbi = 0;
- int have_free_time = 1;
+ int have_free_time = TRUE;
 
- if(frame_counter == 0)
+ if((int)vdc->RCRCount == ((int)vdc->RCR - 0x40) && (vdc->CR & 0x04))
  {
-  vdc->VDS_cache = M_vdc_VDS;
-  vdc->VSW_cache = M_vdc_VSW;
-  vdc->VDW_cache = M_vdc_VDW;
-  vdc->VCR_cache = M_vdc_VCR;
-  vdc->VBlankFL_cache = vdc->VDS_cache + vdc->VSW_cache + vdc->VDW_cache + 1;
-
-  if(vdc->VBlankFL_cache > 261)
-   vdc->VBlankFL_cache = 261;
+  vdc->status |= VDCS_RR;
+  PCFXIRQ_Assert(vdc->ilevel, 1);
  }
 
- if(frame_counter == 0)
- {
-  vdc->display_counter = 0;
-  vdc->burst_mode = !(vdc->CR & 0xC0);
- }
- 
- if(vdc->burst_mode)
- {
-  if(vdc->display_counter == (vdc->VDS_cache + vdc->VSW_cache))
-   vdc->RCRCount = 0;
- }
- else if(vdc->display_counter >= (vdc->VDS_cache + vdc->VSW_cache) && vdc->display_counter < (vdc->VDS_cache + vdc->VSW_cache + vdc->VDW_cache + 1))
- {
-  have_free_time = 0;
-  if(vdc->display_counter == (vdc->VDS_cache + vdc->VSW_cache))
-   vdc->RCRCount = 0;
- }
- else	// Hmm, overscan...
+ if(vdc->PeriodMode == VDC_PM_VDW)
  {
+  if(!vdc->burst_mode)
+   have_free_time = FALSE;
  }
 
  if(have_free_time) // We're outside of the active display area.  Weehee
@@ -599,72 +611,32 @@ void FXVDC_DoLine(fx_vdc_t *vdc, uint32 frame_counter, uint32 *linebuffer, int s
    DoDMA(vdc);
  }
 
- if(vdc->display_counter == vdc->VBlankFL_cache)
- {
-  need_vbi = 1;
-  if(vdc->SATBPending || (vdc->DCR & 0x10))
-  {
-   vdc->SATBPending = 0;
-   vdc->sat_dma_slcounter = 2;
-
-   if(vdc->SATB < 0x8000)
-   {
-    uint32 len = 256;
-    if(vdc->SATB > 0x7F00)
-     len = 0x8000 - vdc->SATB;
-    memcpy(vdc->SAT, &vdc->VRAM[vdc->SATB], len * sizeof(uint16));
-   }
-  }
- }
-
- if((int)vdc->RCRCount == ((int)vdc->RCR - 0x40) && (vdc->CR & 0x04))
- {
-  printf("RCR Interrupt: %d\n", vdc->display_counter);
-  vdc->status |= VDCS_RR;
-  PCFXIRQ_Assert(vdc->ilevel, 1);
- }
-
  if(vdc->burst_mode)
  {
-  if(frame_counter >= 14 && frame_counter < (14 + 242))
-  {
-   if(!skip)
-    MDFN_FastU32MemsetM8(linebuffer, 0x000, 512);
-  }
+  if(!skip)
+   MDFN_FastU32MemsetM8(linebuffer, 0x000 | ALPHA_MASK, 512);
  }
- else if(vdc->display_counter >= (vdc->VDS_cache + vdc->VSW_cache) && vdc->display_counter < (vdc->VDS_cache + vdc->VSW_cache + vdc->VDW_cache + 1))
+ else if(vdc->PeriodMode == VDC_PM_VDW)
  {
-  if(vdc->display_counter == (vdc->VDS_cache + vdc->VSW_cache))
+  if(!vdc->RCRCount)
    vdc->BG_YOffset = vdc->BYR;
   else
    vdc->BG_YOffset++;
+
   vdc->BG_XOffset = vdc->BXR;
-  if(frame_counter >= 14 && frame_counter < (14 + 242))
-  {
-   if(!skip)
-    DrawBG(vdc, linebuffer);
-   if(vdc->CR & 0x40)
-    DrawSprites(vdc, linebuffer, skip ? 0 : 1);
-  }
-  // Draw screen, joy.
+  // Make sure we at least draw transparency if a layer is disabled
+  if(!skip && vdc->LayerDisable)
+    MDFN_FastU32MemsetM8(linebuffer, 0x000 | ALPHA_MASK, 512);
+  if(!(vdc->LayerDisable & 1) && !skip)
+   DrawBG(vdc, linebuffer);
+  if(vdc->CR & 0x40)
+   DrawSprites(vdc, linebuffer, !(skip || (vdc->LayerDisable & 0x2)));
  }
  else // Hmm, overscan...
  {
-  if(frame_counter >= 14 && frame_counter < (14 + 242))
-  {
-   MDFN_FastU32MemsetM8(linebuffer, 0x000, 512);
-  }
+  MDFN_FastU32MemsetM8(linebuffer, 0x000 | ALPHA_MASK, 512);
  }
 
- if((vdc->CR & 0x08) && need_vbi)
- {
-  vdc->status |= VDCS_VD;
-  PCFXIRQ_Assert(vdc->ilevel, 1);
- }
-
- vdc->RCRCount++;
- vdc->display_counter++;
-
  if(vdc->sat_dma_slcounter)
  {
   vdc->sat_dma_slcounter--;
@@ -677,10 +649,42 @@ void FXVDC_DoLine(fx_vdc_t *vdc, uint32 frame_counter, uint32 *linebuffer, int s
    }
   }
  }
+ 
+}
 
- if(vdc->display_counter == (vdc->VDS_cache + vdc->VSW_cache + vdc->VDW_cache + vdc->VCR_cache + 3))
+
+void FXVDC_DoLinePost(fx_vdc_t *vdc)
+{
+ vdc->PeriodCounter--;
+ vdc->RCRCount++;
+
+ if(!vdc->PeriodCounter)
  {
-  vdc->display_counter = 0;
+  if(vdc->PeriodMode == VDC_PM_VSW)
+  {
+   vdc->PeriodCounter = M_vdc_VDS;
+   vdc->PeriodMode = VDC_PM_VDS;
+  }
+  else if(vdc->PeriodMode == VDC_PM_VDS)
+  {
+   vdc->PeriodCounter = M_vdc_VDW + 1;
+   vdc->PeriodMode = VDC_PM_VDW;
+
+   vdc->burst_mode = !(vdc->CR & 0xC0);
+   vdc->RCRCount = 0;
+  }
+  else if(vdc->PeriodMode == VDC_PM_VDW)
+  {
+   DoVBlank(vdc);   
+   vdc->PeriodCounter = M_vdc_VCR;
+   vdc->PeriodMode = VDC_PM_VCR;
+  }
+  else if(vdc->PeriodMode == VDC_PM_VCR)
+  {
+   vdc->PeriodMode = VDC_PM_VSW;
+   vdc->PeriodCounter = M_vdc_VSW;
+   //printf("bork: %d");
+  }
  }
 }
 
@@ -697,22 +701,148 @@ void FXVDC_Power(fx_vdc_t *vdc)
  FXVDC_Reset(vdc);
 }
 
-fx_vdc_t *FXVDC_Init(int ilevel)
+fx_vdc_t *FXVDC_Init(int ilevel, bool disable_sprite_limit)
 {
  fx_vdc_t *ret = (fx_vdc_t *)calloc(1, sizeof(fx_vdc_t));
 
  ret->ilevel = ilevel;
- unlimited_sprites = MDFN_GetSettingB("pce.nospritelimit");
+ ret->Aux0BPBase = ~0;
+
+ unlimited_sprites = disable_sprite_limit;
 
  return(ret);
 }
 
+void FXVDC_Close(fx_vdc_t *vdc)
+{
+ free(vdc);
+}
+
+void FXVDC_SetAux0BPBpase(fx_vdc_t *vdc, uint32 new_base)
+{
+ vdc->Aux0BPBase = new_base;
+}
+
+#ifdef WANT_DEBUGGER
+void FXVDC_GetAddressSpaceBytes(fx_vdc_t *vdc, const char *name, uint32 Address, uint32 Length, uint8 *Buffer)
+{
+ if(!strcmp(name, "vram"))
+ {
+  while(Length--)
+  {
+   Address &= 0x1FFFF;
+   *Buffer = vdc->VRAM[Address >> 1] >> ((Address & 1) * 8);
+   Address++;
+   Buffer++;
+  }
+ }
+}
+
+void FXVDC_PutAddressSpaceBytes(fx_vdc_t *vdc, const char *name, uint32 Address, uint32 Length, uint32 Granularity, bool hl, const uint8 *Buffer)
+{
+ if(!strcmp(name, "vram"))
+ {
+  while(Length--)
+  {
+   Address &= 0x1FFFF;
+   REGSETP(vdc->VRAM[Address >> 1], *Buffer, Address & 1);
+   FixTileCache(vdc, Address >> 1);
+   Address++;
+   Buffer++;
+  }
+ }
+}
+
+
+void FXVDC_SetRegister(fx_vdc_t *vdc, const std::string &name, uint32 value)
+{
+ if(name == "MAWR")
+  vdc->MAWR = value;
+ else if(name == "MARR")
+  vdc->MARR = value;
+ else if(name == "Status")
+ {
+  vdc->status = value & 0x7F;
+  PCFXIRQ_Assert(vdc->ilevel, (value & 0x3F) ? TRUE : FALSE);
+ }
+ else if(name == "CR")
+ {
+  vdc->CR = value;
+ }
+
+}
+
+uint32 FXVDC_GetRegister(fx_vdc_t *vdc, const std::string &name, std::string *special)
+{
+ uint32 value = 0xDEADBEEF;
+
+ if(name == "MAWR")
+  value = vdc->MAWR;
+ else if(name == "MARR")
+  value = vdc->MARR;
+ else if(name == "Status")
+  value = vdc->status;
+ else if(name == "CR")
+ {
+  value = vdc->CR;
+  if(special)
+  {
+   char buf[256];
+   snprintf(buf, 256, "Sprite Hit IRQ: %s, Sprite Overflow IRQ: %s, RCR IRQ: %s, VBlank IRQ: %s, Sprites: %s, Background: %s", (value & 1) ? "On" : "Off", (value & 2) ? "On" : "Off",
+       (value & 4) ? "On" : "Off", (value & 8) ? "On" : "Off", (value & 0x40) ? "On" : "Off", (value & 0x80) ? "On" : "Off");
+   *special = std::string(buf);
+  }
+ }
+ else if(name == "RCR")
+  value = vdc->RCR;
+ else if(name == "BXR")
+  value = vdc->BXR;
+ else if(name == "BYR")
+  value = vdc->BYR;
+ else if(name == "MWR")
+  value = vdc->MWR;
+ else if(name == "HSR")
+  value = vdc->HSR;
+ else if(name == "HDR")
+  value = vdc->HDR;
+ else if(name == "VSR")
+ {
+  value = vdc->VSR;
+  if(special)
+  {
+   char buf[256];
+   snprintf(buf, 256, "VSW: %02x, VDS: %02x", value & 0x1F, (value >> 8) & 0xFF);
+   *special = std::string(buf);
+  }
+ }
+ else if(name == "VDR")
+  value = vdc->VDR;
+ else if(name == "VCR")
+  value = vdc->VCR;
+ else if(name == "DCR")
+  value = vdc->DCR;
+ else if(name == "SOUR")
+  value = vdc->SOUR;
+ else if(name == "DESR")
+  value = vdc->DESR;
+ else if(name == "LENR")
+  value = vdc->LENR;
+ else if(name == "SATB")
+  value = vdc->SATB;
+
+ return(value);
+}
+#endif
+
+
 int FXVDC_StateAction(StateMem *sm, int load, int data_only, fx_vdc_t *vdc, const char *name)
 {
  SFORMAT VDC_StateRegs[] = 
  {
-	SFVARN(vdc->display_counter, "display_counter"),
+	SFVARN(vdc->PeriodMode, "Period_Mode"),
+	SFVARN(vdc->PeriodCounter, "Period_Counter"),
         SFVARN(vdc->sat_dma_slcounter, "sat_dma_slcounter"),
+	SFVARN(vdc->VBlank_Test_Occurred, "VB_test_occurred"),
 
         SFVARN(vdc->select, "select"),
         SFVARN(vdc->MAWR, "MAWR"),
@@ -742,7 +872,7 @@ int FXVDC_StateAction(StateMem *sm, int load, int data_only, fx_vdc_t *vdc, cons
         SFVARN(vdc->status, "status"),
         SFARRAY16N(vdc->SAT, 0x100, "SAT"),
 
-        SFARRAY16N(vdc->VRAM, 32768, "VRAM"),
+        SFARRAY16N(vdc->VRAM, 65536, "VRAM"),
         SFVARN(vdc->DMAReadBuffer, "DMAReadBuffer"),
         SFVARN(vdc->DMAReadWrite, "DMAReadWrite"),
         SFVARN(vdc->DMARunning, "DMARunning"),
@@ -752,7 +882,7 @@ int FXVDC_StateAction(StateMem *sm, int load, int data_only, fx_vdc_t *vdc, cons
         SFVARN(vdc->BG_YOffset, "BG_YOffset"),
         SFVARN(vdc->BG_XOffset, "BG_XOffset"),
 	SFVARN(vdc->VDS_cache, "VDS_cache"),
-        SFVARN(vdc->VSW_cache, "VDS_cache"),
+        SFVARN(vdc->VSW_cache, "VSW_cache"),
         SFVARN(vdc->VDW_cache, "VDW_cache"),
         SFVARN(vdc->VCR_cache, "VCR_cache"),
         SFVARN(vdc->VBlankFL_cache, "VBlankFL_cache"),
@@ -762,10 +892,99 @@ int FXVDC_StateAction(StateMem *sm, int load, int data_only, fx_vdc_t *vdc, cons
 
   if(load)
   {
-   for(int x = 0; x < 32768; x++)
+   for(int x = 0; x < 65536; x++)
     FixTileCache(vdc, x);
   }
 
  return(ret);
 }
 
+void FXVDC_SetDotClock(fx_vdc_t *vdc, unsigned int clock)
+{
+ vdc->dot_clock = clock;
+}
+
+bool FXVDC_ToggleLayer(fx_vdc_t *vdc, int which)
+{
+ vdc->LayerDisable ^= 1 << which;
+
+ return(!((vdc->LayerDisable >> which) & 1));
+}
+
+
+void FXVDC_DoGfxDecode(fx_vdc_t *vdc, uint32 *palette_ptr, uint32 *target, int w, int h, int scroll, bool DecodeSprites)
+{
+ if(DecodeSprites)
+ {
+  for(int y = 0; y < h; y++)
+  {
+   for(int x = 0; x < w; x += 16)
+   {
+    int which_tile = (x / 16) + (scroll + (y / 16)) * (w / 16);
+
+    if(which_tile >= 0x400)
+    {
+     for(int sx = 0; sx < 16; sx++)
+     {
+      target[x + sx] = MK_COLORA(0, 0, 0, 0);
+      target[x + w * 1 + sx] = 0;
+      target[x + w * 2 + sx] = 0;
+     }
+     continue;
+    }
+    //which_tile &= 0x3FF;
+
+    uint16 cg[4];
+    cg[0] = vdc->VRAM[which_tile * 64 + (y & 15)];
+    cg[1] = vdc->VRAM[which_tile * 64 + (y & 15) + 16];
+    cg[2] = vdc->VRAM[which_tile * 64 + (y & 15) + 32];
+    cg[3] = vdc->VRAM[which_tile * 64 + (y & 15) + 48];
+
+    for(int sx = 0; sx < 16; sx++)
+    {
+     int rev_sx = 15 - sx;
+     target[x + sx] = palette_ptr[(((cg[0] >> rev_sx) & 1) << 0) | 
+		(((cg[1] >> rev_sx) & 1) << 1) | (((cg[2] >> rev_sx) & 1) << 2) | (((cg[3] >> rev_sx) & 1) << 3)];
+     target[x + w * 1 + sx] = which_tile;
+     target[x + w * 2 + sx] = which_tile * 64;
+    }
+   }
+   target += w * 3;
+  }
+ }
+ else for(int y = 0; y < h; y++)
+ {
+  for(int x = 0; x < w; x+=8)
+  {
+   int which_tile = (x / 8) + (scroll + (y / 8)) * (w / 8);
+
+   if(which_tile >= 0x1000)
+   {
+    for(int sx = 0; sx < 8; sx++)
+    {
+     target[x + sx] = MK_COLORA(0, 0, 0, 0);
+     target[x + w * 1 + sx] = 0;
+     target[x + w * 2 + sx] = 0;
+    }
+    continue;
+   }
+   //which_tile &= 0xFFF;
+
+   target[x + 0] = palette_ptr[ vdc->bg_tile_cache[which_tile][y & 0x7][0]];
+   target[x + 1] = palette_ptr[ vdc->bg_tile_cache[which_tile][y & 0x7][1]];
+   target[x + 2] = palette_ptr[ vdc->bg_tile_cache[which_tile][y & 0x7][2]];
+   target[x + 3] = palette_ptr[ vdc->bg_tile_cache[which_tile][y & 0x7][3]];
+   target[x + 4] = palette_ptr[ vdc->bg_tile_cache[which_tile][y & 0x7][4]];
+   target[x + 5] = palette_ptr[ vdc->bg_tile_cache[which_tile][y & 0x7][5]];
+   target[x + 6] = palette_ptr[ vdc->bg_tile_cache[which_tile][y & 0x7][6]];
+   target[x + 7] = palette_ptr[ vdc->bg_tile_cache[which_tile][y & 0x7][7]];
+
+   target[x + w*1 + 0]=target[x + w*1 + 1]=target[x + w*1 + 2]=target[x + w*1 + 3] =
+   target[x + w*1 + 4]=target[x + w*1 + 5]=target[x + w*1 + 6]=target[x + w*1 + 7] = which_tile;
+
+   target[x + w*2 + 0]=target[x + w*2 + 1]=target[x + w*2 + 2]=target[x + w*2 + 3] =
+   target[x + w*2 + 4]=target[x + w*2 + 5]=target[x + w*2 + 6]=target[x + w*2 + 7] = which_tile * 16;
+  }
+  target += w * 3;
+ }
+}
