@@ -157,6 +157,27 @@ void SexiALI_Convert(SexyAL_format *srcformat, SexyAL_format *destformat, void *
   }
  }
 
+ if(destformat->sampformat == SEXYAL_FMT_PCMFLOAT)
+ {
+  float *dest = vdest;
+  for(x=0;x<frames*fmul;x++)
+  {
+   float tmp;
+
+   tmp = (float)*src / 32768;
+   src++;
+   if(downit)
+   {
+    tmp += (float)*src / 32768;
+    tmp /= 2;
+    src++;
+   }
+   *dest++ = tmp;
+   if(upit)
+    *dest++ = tmp;
+  }
+ }
+
 
  if(destformat->byteorder != srcformat->byteorder)
  {
