@@ -34,10 +34,6 @@
 #include "video.h"
 
 static int SaveStateStatus[10];
-static uint32 StateShow;
-static uint32 *StateShowPB = NULL; //[160 * 120]; // = NULL; //[160 * 120]; // 320 x 240 -> 160 * 120
-static uint32 StateShowPBWidth;
-static uint32 StateShowPBHeight;
 
 #define RLSB 		MDFNSTATE_RLSB	//0x80000000
 
@@ -131,10 +127,39 @@ int smem_read32le(StateMem *st, uint32 *b)
  return(4);
 }
 
+
+static bool ValidateSFStructure(SFORMAT *sf)
+{
+ SFORMAT *saved_sf = sf;
+
+ while(sf->s || sf->desc)
+ {
+  SFORMAT *sub_sf = saved_sf;
+  while(sub_sf->s || sub_sf->desc)
+  {
+   if(sf != sub_sf)
+   {
+    if(!strncmp(sf->desc, sub_sf->desc, 32))
+    {
+     printf("Duplicate state variable name: %.32s\n", sf->desc);
+    }
+   }
+   sub_sf++;
+  }
+
+  sf++;
+ }
+ return(1);
+}
+
+
 static int SubWrite(StateMem *st, SFORMAT *sf, int data_only, gzFile fp)
 {
  uint32 acc=0;
 
+ // FIXME?  It's kind of slow, and we definitely don't want it on with state rewinding...
+ //ValidateSFStructure(sf);
+
  while(sf->s || sf->desc)	// Size can sometimes be zero, so also check for the text description.  These two should both be zero only at the end of a struct.
  {
   if(!sf->s || !sf->v)
@@ -167,8 +192,11 @@ static int SubWrite(StateMem *st, SFORMAT *sf, int data_only, gzFile fp)
 
    memset(desco, 0, 32);
 
-   if(slen > 32) slen = 32;
-
+   if(slen > 32) 
+   {
+    printf("Warning:  state variable name too long: %s %d\n", sf->desc, slen);
+    slen = 32;
+   }
    memcpy(desco, sf->desc, slen);
 
    if(!data_only)
@@ -255,7 +283,10 @@ static SFORMAT *CheckS(SFORMAT *sf, uint32 tsize, const char *desc)
    uint32 bytesize = sf->s&(~(MDFNSTATE_RLSB32 | MDFNSTATE_RLSB16 | RLSB));
 
    if(tsize != bytesize)
+   {
+    printf("tsize != bytesize: %.32s\n", desc);
     return(0);
+   }
    return(sf);
   }
   sf++;
@@ -289,47 +320,6 @@ static void DOReadChunk(StateMem *st, SFORMAT *sf)
  }
 }
 
-bool MDFNSS_WriteMovieJoyChunk(StateMem *st, gzFile fp, SFORMAT *sf)
-{
- return(SubWrite(st, sf, 1, fp));
-}
-
-bool MDFNSS_ReadMovieJoyChunk(StateMem *st, gzFile fp, SFORMAT *sf)
-{
- while(sf->s || sf->desc)       // Size can sometimes be zero, so also check for the text description.
-                                // These two should both be zero only at the end of a struct.
- {
-  if(!sf->s || !sf->v)
-  {
-   sf++;
-   continue;
-  }
-
-  if(sf->s == (uint32) ~0) // Link to another SFORMAT struct
-  {
-   MDFNSS_ReadMovieJoyChunk(st, fp, (SFORMAT *)sf->v);
-   sf++;
-   continue;
-  }
-
-  int32 bytesize = sf->s&(~(MDFNSTATE_RLSB32 | MDFNSTATE_RLSB16 | RLSB));
-
-  if(st)
-   smem_read(st, (uint8 *)sf->v, bytesize);
-  else
-   gzread(fp, (uint8 *)sf->v, bytesize);
-  if(sf->s & MDFNSTATE_RLSB32)
-   Endian_A32_LE_to_NE(sf->v, bytesize / sizeof(uint32));
-  else if(sf->s & MDFNSTATE_RLSB16)
-   Endian_A16_LE_to_NE(sf->v, bytesize / sizeof(uint16));
-  else if(sf->s&RLSB)
-   Endian_V_LE_to_NE(sf->v, bytesize);
-
-  sf++;
- }
- return(1);
-}
-
 static int ReadStateChunk(StateMem *st, SFORMAT *sf, int size, int data_only)
 {
  SFORMAT *tmp;
@@ -348,20 +338,11 @@ static int ReadStateChunk(StateMem *st, SFORMAT *sf, int size, int data_only)
    char toa[32];
 
    if(smem_read(st, toa, 32) <= 0)
-    return 0;
-
-   int x, nr;
-
-   for(x = nr = 0; x < 32; x++)
    {
-    if(toa[nr] == '"') // Strip out quotes to fix loading save states created in 0.2.0 through 0.3.1
-    {
-     if(nr < 32 && toa[nr] != 0)
-      nr++;
-    }
-    toa[x] = (nr >= 32) ? 0 : toa[nr];
-    nr++;
+    puts("Unexpected EOF?");
+    return 0;
    }
+
    smem_read32le(st, &tsize);
    if((tmp=CheckS(sf,tsize,toa)))
    {
@@ -378,7 +359,10 @@ static int ReadStateChunk(StateMem *st, SFORMAT *sf, int size, int data_only)
    }
    else
     if(smem_seek(st,tsize,SEEK_CUR) < 0)
+    {
+     puts("Seek error");
      return(0);
+    }
   } // while(...)
  }
  return 1;
@@ -387,55 +371,85 @@ static int ReadStateChunk(StateMem *st, SFORMAT *sf, int size, int data_only)
 int CurrentState = 0;
 extern int geniestage;
 
-static void MakeStatePreview(uint8 *dest, uint32 *fb, MDFN_Rect *LineWidths, int xdiv, int ydiv)
+static void MakeStatePreview(uint8 *dest, uint32 *fb, MDFN_Rect *LineWidths)
 {
- int sqx, sqy;
- int bitcow = 8;
-
- if(xdiv == 2) bitcow++;
- if(ydiv == 2) bitcow++;
+ int x, y;
 
  if(!fb || !LineWidths) return;
- for(sqy=MDFNGameInfo->DisplayRect.y / ydiv;sqy<(MDFNGameInfo->DisplayRect.y + MDFNGameInfo->DisplayRect.h) / ydiv;sqy++)
+
+ for(y = MDFNGameInfo->DisplayRect.y; y < (MDFNGameInfo->DisplayRect.y + MDFNGameInfo->DisplayRect.h); y++)
  {
-  for(sqx=0;sqx<MDFNGameInfo->width / xdiv;sqx++)
+  uint32 mw;
+  uint32 whole_pix;
+
+  if(LineWidths[0].w != ~0)
+   mw = LineWidths[y].w;
+  else
+   mw = MDFNGameInfo->DisplayRect.w;
+
+  whole_pix = 256 * mw / MDFNGameInfo->ss_preview_width;
+
+  // TODO:  Add support for scaling when (mw / MDFNGameInfo->ss_preview_width) > 2 and it's not an integer
+  // (no emulated systems as of Jan 31, 2007 are like this, though)
+  // int whole_pix_i = (whole_pix >> 8) - 2;
+  // if(whole_pix_i < 0) whole_pix_i = 0;
+
+  for(x = 0; x < MDFNGameInfo->ss_preview_width; x++)
   {
-   int x, y;
-   uint32 r,g,b;
+   uint32 accum_r, accum_g, accum_b;
+   uint32 pixel;
+   int nr, ng, nb;
+   uint32 real_x;
 
-   r=g=b=0;
-   x = 0;
+   accum_r = 0;
+   accum_g = 0;
+   accum_b = 0;
 
-   for(y=0;y<ydiv;y++)
-    for(x = 0; x < xdiv; x++)
+   real_x = x * 256 * mw / MDFNGameInfo->ss_preview_width;
+
+   if(!(whole_pix & 0xFF) && whole_pix)
+   {
+    for(unsigned int px = 0; px < (whole_pix >> 8); px++)
     {
-     uint32 real_x;
-     uint32 pixel; 
-     int nr, ng, nb;
-
-     if(LineWidths[0].w != ~0)
-      real_x = 256 * LineWidths[sqy * ydiv + y].x + 256 * (LineWidths[sqy * ydiv + y].w * (sqx * xdiv + x)) / MDFNGameInfo->width;
-     else
-      real_x = 256 * MDFNGameInfo->DisplayRect.x + 256 * (MDFNGameInfo->DisplayRect.w * (sqx * xdiv + x)) / MDFNGameInfo->width;
-
-     pixel = fb[(real_x >> 8) + ((sqy * ydiv) + y) * MDFNGameInfo->pitch / sizeof(uint32)];
+     pixel = fb[(real_x >> 8) + px + LineWidths[y].x + y * MDFNGameInfo->pitch / sizeof(uint32)];
      DECOMP_COLOR(pixel, nr, ng, nb);
-
-     r += nr * (256 - (real_x & 0xFF));
-     g += ng * (256 - (real_x & 0xFF));
-     b += nb * (256 - (real_x & 0xFF));
-
-     pixel = fb[((real_x >> 8) + 1) + ((sqy * ydiv) + y) * MDFNGameInfo->pitch / sizeof(uint32)];
-     DECOMP_COLOR(pixel, nr, ng, nb);
-
-     r += nr * ((real_x & 0xFF));
-     g += ng * ((real_x & 0xFF));
-     b += nb * ((real_x & 0xFF));
-
+  
+     accum_r += nr * 256 / (whole_pix >> 8);
+     accum_g += ng * 256 / (whole_pix >> 8);
+     accum_b += nb * 256 / (whole_pix >> 8);
     }
-   *dest++ = r >> bitcow;
-   *dest++ = g >> bitcow;
-   *dest++ = b >> bitcow;
+   }
+   else
+   {
+    pixel = fb[(real_x >> 8) + LineWidths[y].x + y * MDFNGameInfo->pitch / sizeof(uint32)];
+    DECOMP_COLOR(pixel, nr, ng, nb);
+
+    accum_r += (256 - (real_x & 0xFF)) * nr;
+    accum_g += (256 - (real_x & 0xFF)) * ng;
+    accum_b += (256 - (real_x & 0xFF)) * nb;
+
+    pixel = fb[(real_x >> 8) + 1 + LineWidths[y].x + y * MDFNGameInfo->pitch / sizeof(uint32)];
+    DECOMP_COLOR(pixel, nr, ng, nb);
+
+    accum_r += (real_x & 0xFF) * nr;
+    accum_g += (real_x & 0xFF) * ng;
+    accum_b += (real_x & 0xFF) * nb;
+   }
+
+   {
+    int r = (accum_r + 127) >> 8;
+    int g = (accum_g + 127) >> 8;
+    int b = (accum_b + 127) >> 8;
+
+    if(r > 255) r = 255;
+    if(g > 255) g = 255;
+    if(b > 255) b = 255;
+
+    *dest++ = r;
+    *dest++ = g;
+    *dest++ = b;
+   }
+
   }
  }
 }
@@ -468,7 +482,10 @@ int MDFNSS_StateAction(StateMem *st, int load, int data_only, std::vector <SSDes
      if(!memcmp(sname, section->name, 4))
      {
       if(!ReadStateChunk(st, section->sf, tmp_size, 0))
+      {
+       printf("Error reading chunk: %.4s\n", section->name);
        return(0);
+      }
       found = 1;
       break;
      } 
@@ -476,13 +493,20 @@ int MDFNSS_StateAction(StateMem *st, int load, int data_only, std::vector <SSDes
      {
 	//puts("SEEK");
       if(smem_seek(st, tmp_size, SEEK_CUR) < 0)
+      {
+       puts("Chunk seek failure");
        return(0);
+      }
      }
     }
     if(smem_seek(st, -total, SEEK_CUR) < 0)
+    {
+     puts("Reverse seek error");
      return(0);
+    }
     if(!found && !section->optional) // Not found.  We are sad!
     {
+	printf("Chunk missing:  %.4s\n", section->name);
 	return(0);
     }
    }
@@ -509,21 +533,10 @@ int MDFNSS_SaveSM(StateMem *st, int wantpreview, int data_only, uint32 *fb, MDFN
 {
         static uint8 header[32]="MEDNAFENSVESTATE";
 	int neowidth, neoheight;
-	bool OriginalImage = 0;
 
-	neowidth = MDFNGameInfo->width;
+	neowidth = MDFNGameInfo->ss_preview_width;
 	neoheight = MDFNGameInfo->DisplayRect.h;
 
-	if(neowidth < 256 && neoheight < 224) // Fudge, FIXME
-	{
-	 OriginalImage = 1;
-	}
-	else
-	{
-	 neowidth /= 2;
-	 neoheight /= 2;
-	}
-
 	if(!data_only)
 	{
          memset(header+16,0,16);
@@ -537,13 +550,14 @@ int MDFNSS_SaveSM(StateMem *st, int wantpreview, int data_only, uint32 *fb, MDFN
 	{
          uint8 *previewbuffer = (uint8 *)malloc(3 * neowidth * neoheight);
 
-         MakeStatePreview(previewbuffer, fb, LineWidths, OriginalImage ? 1 : 2, OriginalImage ? 1 : 2);
+         MakeStatePreview(previewbuffer, fb, LineWidths);
          smem_write(st, previewbuffer, 3 * neowidth * neoheight);
 
 	 free(previewbuffer);
 	}
 
-	MDFNGameInfo->StateAction(st, 0, data_only);
+	if(!MDFNGameInfo->StateAction(st, 0, data_only))
+	 return(0);
 
 	if(!data_only)
 	{
@@ -578,7 +592,15 @@ int MDFNSS_Save(const char *fname, const char *suffix, uint32 *fb, MDFN_Rect *Li
 	 return(0);
 	}
 
-	MDFNSS_SaveFP(st, fb, LineWidths);
+	if(!MDFNSS_SaveFP(st, fb, LineWidths))
+        {
+         SaveStateStatus[CurrentState] = 0;
+         gzclose(st);
+         if(!fname && !suffix)
+          MDFN_DispMessage((UTF8 *)_("State %d save error."),CurrentState);
+	 return(0);
+        }
+
 	SaveStateStatus[CurrentState] = 1;
 	gzclose(st);
 
@@ -603,9 +625,13 @@ int MDFNSS_LoadSM(StateMem *st, int haspreview, int data_only)
           return(0);
 
 	 stateversion = MDFN_de32lsb(header + 16);
-	}
 
-	if(!stateversion) stateversion = 1;
+	 if(stateversion < 0x0600)
+ 	 {
+	  printf("State too old: %08x\n", stateversion);
+	  return(0);
+	 }
+	}
 
 	if(haspreview)
         {
@@ -613,16 +639,6 @@ int MDFNSS_LoadSM(StateMem *st, int haspreview, int data_only)
          uint32 height = MDFN_de32lsb(header + 28);
 	 uint32 psize;
 
-         if(!width) width = MDFNGameInfo->width / 2;
-	 if(!height) height = MDFNGameInfo->height / 2;
-
-	 if(MDFNGameInfo->system == GISYS_PCE)
-	 {
-	  if(stateversion < 0x0303)
-	   height = 240;
-	  else if(stateversion < 0x0305)
-	   height = 224;
-	 }
 	 psize = width * height * 3;
 	 smem_seek(st, psize, SEEK_CUR);	// Skip preview
  	}
@@ -755,15 +771,20 @@ void MDFNSS_CheckStates(void)
         }
 
 	CurrentState=0;
-	StateShow=0;
+	MDFND_SetStateStatus(NULL);
 }
 
 void MDFNI_SelectState(int w)
 {
  gzFile fp;
+ uint32 StateShow;
+ uint32 *StateShowPB = NULL;
+ uint32 StateShowPBWidth;
+ uint32 StateShowPBHeight;
 
  if(w == -1) 
  {  
+  MDFND_SetStateStatus(NULL);
   StateShow = 0; 
   return; 
  }
@@ -786,15 +807,11 @@ void MDFNI_SelectState(int w)
  if(fp)
  {
   uint8 header[32];
-  int x,y;
 
   gzread(fp, header, 32);
   uint32 width = MDFN_de32lsb(header + 24);
   uint32 height = MDFN_de32lsb(header + 28);
 
-  if(!width) width = MDFNGameInfo->width / 2;
-  if(!height) height = MDFNGameInfo->height / 2;
-
   if(width > 512) width = 512;
   if(height > 512) height = 512;
 
@@ -813,8 +830,8 @@ void MDFNI_SelectState(int w)
    StateShowPBWidth = width;
    StateShowPBHeight = height;
 
-   for(y=0; y<height; y++)
-    for(x=0;x<width; x++)
+   for(unsigned int y=0; y<height; y++)
+    for(unsigned int x=0; x<width; x++)
     {
      StateShowPB[x + y * width] = MK_COLORA(rptr[0],rptr[1],rptr[2], 0xFF);
      rptr+=3;
@@ -830,21 +847,34 @@ void MDFNI_SelectState(int w)
    free(StateShowPB);
    StateShowPB = NULL;
   }
-  StateShowPBWidth = MDFNGameInfo->width / 2;
-  StateShowPBHeight = MDFNGameInfo->DisplayRect.h / 2;
+  StateShowPBWidth = MDFNGameInfo->ss_preview_width;
+  StateShowPBHeight = MDFNGameInfo->DisplayRect.h;
  }
  MDFN_ResetMessages();
+
+ StateStatusStruct *status = (StateStatusStruct*)calloc(1, sizeof(StateStatusStruct));
+ 
+ memcpy(status->status, SaveStateStatus, 10 * sizeof(int));
+ status->current = CurrentState;
+ status->gfx = StateShowPB;
+ status->w = StateShowPBWidth;
+ status->h = StateShowPBHeight;
+ status->pitch = StateShowPBWidth;
+
+ MDFND_SetStateStatus(status);
 }  
 
 void MDFNI_SaveState(const char *fname, const char *suffix, uint32 *fb, MDFN_Rect *LineWidths)
 {
- StateShow=0;
+ MDFND_SetStateStatus(NULL);
  MDFNSS_Save(fname, suffix, fb, LineWidths);
 }
 
+extern int doothack;
+
 void MDFNI_LoadState(const char *fname, const char *suffix)
 {
- StateShow=0;
+ MDFND_SetStateStatus(NULL);
 
  /* For network play and movies, be load the state locally, and then save the state to a temporary buffer,
     and send or record that.  This ensures that if an older state is loaded that is missing some
@@ -860,51 +890,59 @@ void MDFNI_LoadState(const char *fname, const char *suffix)
   if(MDFNMOV_IsRecording())
    MDFNMOV_RecordState();
  }
+ doothack = 100;
 }
 
-bool MDFN_DrawSaveStates(uint32 *XBuf, const MDFN_Rect *OSDRect, uint32 pitch32)
-{
- if(StateShow < MDFND_GetTime()) return(0);
-
- MDFN_DrawStateMovieRow((uint32 *)XBuf, OSDRect, pitch32, StateShowPB,StateShowPBWidth, StateShowPBHeight, SaveStateStatus,CurrentState, (UTF8 *)_("-select state-"));
- return(1);
-}
-
-
 #include "minilzo.h"
+#include "quicklz.h"
+
+enum
+{
+ SRW_COMPRESSOR_MINILZO = 0,
+ SRW_COMPRESSOR_QUICKLZ
+};
 
 typedef struct
 {
-	uint8 *compressed_data;
+	uint8 *data;
 	uint32 compressed_len;
 	uint32 uncompressed_len;
 
 	StateMem MovieLove;
-} StateMemLZO;
+} StateMemPacket;
 
 #define SRW_NUM		600
 
+static int SRWCompressor;
 static int EvilEnabled = 0;
-static StateMemLZO *bcs;
-static signed int bcspos;
-static StateMem sm_compress_buffer;
+static StateMemPacket *bcs;
+static int32 bcspos;
 
 void MDFN_StateEvilBegin(void)
 {
  int x;
+ std::string srwcompstring;
 
  if(!EvilEnabled)
   return;
 
- bcs = (StateMemLZO *)calloc(SRW_NUM, sizeof(StateMemLZO));
- bcspos = 0;
+ SRWCompressor = SRW_COMPRESSOR_MINILZO;
+ srwcompstring = MDFN_GetSettingS("srwcompressor");
 
- memset(&sm_compress_buffer, 0, sizeof(StateMem));
+ if(srwcompstring == "minilzo")
+  SRWCompressor = SRW_COMPRESSOR_MINILZO;
+ else if(srwcompstring == "quicklz")
+  SRWCompressor  = SRW_COMPRESSOR_QUICKLZ;
+
+ bcs = (StateMemPacket *)calloc(SRW_NUM, sizeof(StateMemPacket));
+ bcspos = 0;
 
  for(x=0;x<SRW_NUM;x++)
  {
-  bcs[x].compressed_data = NULL;
+  bcs[x].data = NULL;
   bcs[x].compressed_len = 0;
+  bcs[x].uncompressed_len = 0;
+  memset(&bcs[x].MovieLove, 0, sizeof(StateMem));
  }
 }
 
@@ -927,13 +965,10 @@ void MDFN_StateEvilEnd(void)
 
   for(x = 0;x < SRW_NUM; x++)
   {
-   if(sm_compress_buffer.data)
-    free(sm_compress_buffer.data);
-   sm_compress_buffer.data = NULL;
 
-   if(bcs[x].compressed_data)
-    free(bcs[x].compressed_data);
-   bcs[x].compressed_data = NULL;
+   if(bcs[x].data)
+    free(bcs[x].data);
+   bcs[x].data = NULL;
    bcs[x].compressed_len = 0;
   }
   free(bcs);
@@ -947,7 +982,7 @@ void MDFN_StateEvilFlushMovieLove(void)
  {
   if(bcs[bahpos].MovieLove.data)
   {
-   if(bcs[x].compressed_data)
+   if(bcs[x].data)
     MDFNMOV_ForceRecord(&bcs[bahpos].MovieLove);
    free(bcs[bahpos].MovieLove.data);
    bcs[bahpos].MovieLove.data = NULL;
@@ -963,42 +998,60 @@ int MDFN_StateEvil(int rewind)
 
  if(rewind)
  {
-  int dootpos = bcspos - 1;
-  if(dootpos < 0) dootpos += SRW_NUM;
+  int32 next_bcspos = bcspos;
+  bool NeedDataFlush = FALSE;
 
-  if(bcs[dootpos].compressed_data)
-  {
-   if(bcs[bcspos].MovieLove.data)
-   {
-    free(bcs[bcspos].MovieLove.data);
-    bcs[bcspos].MovieLove.data = NULL;
-   }
-   free(bcs[bcspos].compressed_data);
-   bcs[bcspos].compressed_data = NULL;
-   bcs[bcspos].compressed_len = 0;
-  }
   bcspos--;
   if(bcspos < 0) bcspos += SRW_NUM;
-  if(!bcs[bcspos].compressed_len)
+
+  if(!bcs[bcspos].data)
    bcspos = (bcspos + 1) % SRW_NUM;
+  else
+   NeedDataFlush = TRUE;
 
   if(bcs[bcspos].compressed_len)
   {
-   StateMem sm;
    uint8 *tmp_buf;
    lzo_uint dst_len = bcs[bcspos].uncompressed_len;
 
    tmp_buf = (uint8 *)malloc(bcs[bcspos].uncompressed_len);
 
-   lzo1x_decompress(bcs[bcspos].compressed_data, bcs[bcspos].compressed_len, tmp_buf, &dst_len, NULL);
+   if(SRWCompressor == SRW_COMPRESSOR_QUICKLZ)
+    dst_len = qlz_decompress((char*)bcs[bcspos].data, tmp_buf);
+   else
+    lzo1x_decompress(bcs[bcspos].data, bcs[bcspos].compressed_len, tmp_buf, &dst_len, NULL);
 
-   sm.data = tmp_buf;
+   for(uint32 x = 0; x < bcs[bcspos].uncompressed_len && x < bcs[next_bcspos].uncompressed_len; x++)
+    tmp_buf[x] ^= bcs[next_bcspos].data[x];
+
+   free(bcs[bcspos].data);
+   bcs[bcspos].data = tmp_buf;
+   bcs[bcspos].compressed_len = 0;
+  }
+
+  if(NeedDataFlush)
+  {
+   if(bcs[next_bcspos].MovieLove.data)
+   {
+    free(bcs[next_bcspos].MovieLove.data);
+    bcs[next_bcspos].MovieLove.data = NULL;
+   }
+   free(bcs[next_bcspos].data);
+   bcs[next_bcspos].data = NULL;
+   bcs[next_bcspos].compressed_len = 0;
+   bcs[next_bcspos].uncompressed_len = 0;
+  }
+
+  if(bcs[bcspos].uncompressed_len)
+  {
+   StateMem sm;
+
+   sm.data = bcs[bcspos].data;
    sm.loc = 0;
    sm.initial_malloc = 0;
    sm.malloced = sm.len = bcs[bcspos].uncompressed_len;
 
    MDFNSS_LoadSM(&sm, 0, 1);
-   free(tmp_buf);
 
    free(MDFNMOV_GrabRewindJoy().data);
    return(1);
@@ -1007,11 +1060,14 @@ int MDFN_StateEvil(int rewind)
  else
  {
   uint8 workmem[LZO1X_1_MEM_COMPRESS];
+  StateMem sm;
+  int32 prev_bcspos = bcspos;
+
   bcspos = (bcspos + 1) % SRW_NUM;
 
   if(MDFNMOV_IsRecording())
   {
-   if(bcs[bcspos].compressed_data && bcs[bcspos].MovieLove.data)
+   if(bcs[bcspos].data && bcs[bcspos].MovieLove.data)
    {
     //printf("Force: %d\n", bcspos);
     MDFNMOV_ForceRecord(&bcs[bcspos].MovieLove);
@@ -1019,26 +1075,52 @@ int MDFN_StateEvil(int rewind)
     bcs[bcspos].MovieLove.data = NULL;
    }
   }
-  if(bcs[bcspos].compressed_data)
+  if(bcs[bcspos].data)
   {
-   free(bcs[bcspos].compressed_data);
-   bcs[bcspos].compressed_data = NULL;
+   free(bcs[bcspos].data);
+   bcs[bcspos].data = NULL;
   }
   if(bcs[bcspos].MovieLove.data)
   {
    free(bcs[bcspos].MovieLove.data);
    bcs[bcspos].MovieLove.data = NULL;
   }
-  sm_compress_buffer.loc = 0;
-  sm_compress_buffer.len = 0;
-  MDFNSS_SaveSM(&sm_compress_buffer, 0, 1);
-  uint8 * tmp_buf = (uint8 *)malloc((size_t)(1.10 * sm_compress_buffer.len));
-  lzo_uint dst_len = (lzo_uint)(1.10 * sm_compress_buffer.len);
-  lzo1x_1_compress(sm_compress_buffer.data, sm_compress_buffer.len, tmp_buf, &dst_len, workmem);
+  memset(&sm, 0, sizeof(sm));
+  MDFNSS_SaveSM(&sm, 0, 1);
 
-  bcs[bcspos].compressed_data = (uint8 *)realloc(tmp_buf, dst_len);
-  bcs[bcspos].compressed_len = dst_len;
-  bcs[bcspos].uncompressed_len = sm_compress_buffer.len;
+  bcs[bcspos].data = sm.data;
+  bcs[bcspos].compressed_len = 0;
+  bcs[bcspos].uncompressed_len = sm.len;
+
+  // Compress the previous save state.
+  if(bcs[prev_bcspos].data)
+  {
+   for(uint32 x = 0; x < bcs[prev_bcspos].uncompressed_len && x < sm.len; x++)
+    bcs[prev_bcspos].data[x] ^= sm.data[x];
+
+   if(SRWCompressor == SRW_COMPRESSOR_QUICKLZ)
+   {
+    uint8 *tmp_buf = (uint8 *)malloc(bcs[prev_bcspos].uncompressed_len + 36000);
+    uint32 dst_len = bcs[prev_bcspos].uncompressed_len + 36000;
+
+    dst_len = qlz_compress(bcs[prev_bcspos].data, (char*)tmp_buf, bcs[prev_bcspos].uncompressed_len);
+
+    free(bcs[prev_bcspos].data);
+    bcs[prev_bcspos].data = (uint8 *)realloc(tmp_buf, dst_len);
+    bcs[prev_bcspos].compressed_len = dst_len;
+   }
+   else
+   {
+    uint8 * tmp_buf = (uint8 *)malloc((size_t)(1.10 * bcs[prev_bcspos].uncompressed_len));
+    lzo_uint dst_len = (lzo_uint)(1.10 * bcs[prev_bcspos].uncompressed_len);
+
+    lzo1x_1_compress(bcs[prev_bcspos].data, bcs[prev_bcspos].uncompressed_len, tmp_buf, &dst_len, workmem);
+
+    free(bcs[prev_bcspos].data);
+    bcs[prev_bcspos].data = (uint8 *)realloc(tmp_buf, dst_len);
+    bcs[prev_bcspos].compressed_len = dst_len;
+   }
+  }
 
   if(MDFNMOV_IsRecording())
    bcs[bcspos].MovieLove = MDFNMOV_GrabRewindJoy();
